unit Unit1;

interface
uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, OmegaImageList, OmegaScreen, OmegaInput, OmegaBitmapFont,
  OmegaTimer, OmegaUtils, D3DX8, {$I UseD3D8.inc};

type
  TForm1 = class(TForm)
    OmegaScreen1: TOmegaScreen;
    OmegaTimer1: TOmegaTimer;
    OmegaBitmapFont1: TOmegaBitmapFont;
    OmegaInput1: TOmegaInput;
    procedure FormCreate(Sender: TObject);
    procedure OmegaTimer1Timer(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormDestroy(Sender: TObject);
  private
    function InitVertexBuffer: boolean;
    procedure SetupMatrices;
  end;

var Form1: TForm1;

implementation
{$R *.DFM}

var
  VertexBuffer : IDirect3DVertexBuffer8;
  // Matrices
  matWorld : TD3DXMatrix; // World
  matView  : TD3DXMatrix; // View
  matProj  : TD3DXMatrix; // Projection

type
  // The structure of the custom vertex.
  PCustomVertex = ^TCustomVertex;
  TCustomVertex = record
                    X, Y, Z : single; // Position for the vertex.
                    DColor  : dWord;  // Diffuse color.
                  end;

const
  // Flexible vector format (FVF) description.
  // Describes the custom vertex structure (TCustomVertex).
  D3DFVF_CUSTOMVERTEX = D3DFVF_XYZ or D3DFVF_DIFFUSE;

function GetD3DXVector3(_x, _y, _z: single): TD3DXVector3;
begin
  Result.x := _x;
  Result.y := _y;
  Result.z := _z;
end;

function GetCustomVertex(_x, _y, _z: single;
                         _dcolor: dWord): TCustomVertex;
begin
  Result.X := _x;
  Result.Y := _y;
  Result.Z := _z;
  Result.DColor := _dcolor;
end;

procedure TForm1.SetupMatrices;
var _v1, _v2, _v3 : TD3DXVector3;
begin
  // World.
  D3DXMatrixRotationY(matWorld, sin(GetTickCount/500) * 0.9);
  OmegaScreen1.Device.SetTransform(D3DTS_WORLD, matWorld);
  // View.
  _v1 := GetD3DXVector3( 0.0, 0.0, -5.0);
  _v2 := GetD3DXVector3( 0.0, 0.0,  0.0);
  _v3 := GetD3DXVector3( 0.0, 1.0,  0.0);
  D3DXMatrixLookAtLH(matView, _v1, _v2, _v3);
  OmegaScreen1.Device.SetTransform(D3DTS_VIEW, matView);
  // Projection.
  D3DXMatrixPerspectiveFovLH(matProj, PI/4, 1.0, 1.0, 100.0);
  OmegaScreen1.Device.SetTransform(D3DTS_PROJECTION, matProj);
end;

function TForm1.InitVertexBuffer: boolean;
var _vb  : array[0..2] of TCustomVertex;
    _ptr : pByte;
begin
  Result := false;
  // Initialize the vertices.
  _vb[0] := GetCustomVertex(-1, -1, 0.0, $FFFF0000);
  _vb[1] := GetCustomVertex( 1, -1, 0.0, $FF00FF00);
  _vb[2] := GetCustomVertex( 0,  1, 0.0, $FF0000FF);
  // Create a vertex buffer.
  if failed(OmegaScreen1.Device.CreateVertexBuffer(SizeOf(_vb), 0,
                                       D3DFVF_CUSTOMVERTEX,
                                       D3DPOOL_MANAGED,
                                       VertexBuffer)) then exit;
  // Lock, fill and unlock vertex buffer.
  if failed(VertexBuffer.Lock(0, SizeOf(_vb), _ptr, 0)) then exit;
  Move(_vb, _ptr^, SizeOf(_vb));
  VertexBuffer.Unlock;
  // All right
  Result := true;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
 self.ClientWidth := 800;
 self.ClientHeight := 600;
 OmegaScreen1.Width := self.ClientWidth;
 OmegaScreen1.Height := self.ClientHeight;
 OmegaScreen1.Init;
 with OmegaScreen1.Device do
 begin
  // Turn off culling, so we see the front and back of the triangle
  SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);
  // Turn off D3D lighting, since we are providing our own vertex colors
  SetRenderState(D3DRS_LIGHTING, Ord(false));
 end;
 InitVertexBuffer;
 OmegaTimer1.Enabled := True;
end;

procedure TForm1.OmegaTimer1Timer(Sender: TObject);
begin
 if not OmegaScreen1.CanDraw then exit;
 OmegaInput1.Update;
 if oisbutton1 in OmegaInput1.keyboard.statesclicked then
  OmegaScreen1.ToggleFullScreen;
 OmegaScreen1.BeginRender;
 OmegaScreen1.ClearScreen(0, 0, 0);

 // Clear texture
 OmegaScreen1.Device.SetTexture(0, nil);
 // Set Transform states
 SetupMatrices;
 // Set the stream source.
 OmegaScreen1.Device.SetStreamSource( 0, VertexBuffer, sizeof(TCustomVertex));
 // Set the vertex shader.
 OmegaScreen1.Device.SetVertexShader(D3DFVF_CUSTOMVERTEX);
 // Render the vertices.
 OmegaScreen1.Device.DrawPrimitive(D3DPT_TRIANGLELIST, 0, 1);

 OmegaScreen1.SetBlendMode(bmNormal);
 OmegaBitmapFont1.Print(44, 510, 'Press "space" to toggle fullscreen');
 OmegaScreen1.EndRender;
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
 OmegaTimer1.Enabled := False;
end;

procedure TForm1.FormKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
 if key = 27 then close;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
 VertexBuffer := NIL;
end;

end.



