unit Unit1;

interface
uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, OmegaImageList, OmegaScreen, OmegaInput, OmegaBitmapFont,
  OmegaTimer, OmegaUtils, D3DX8, {$I UseD3D8.inc};

type
  TForm1 = class(TForm)
    OmegaScreen1: TOmegaScreen;
    OmegaTimer1: TOmegaTimer;
    OmegaBitmapFont1: TOmegaBitmapFont;
    OmegaInput1: TOmegaInput;
    procedure FormCreate(Sender: TObject);
    procedure OmegaTimer1Timer(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormDestroy(Sender: TObject);
  private
    function InitVertexBuffer: boolean;
    procedure SetupMatrices;
  end;

var Form1: TForm1;

implementation
{$R *.DFM}

var
  Texture      : IDirect3DTexture8;
  VertexBuffer : IDirect3DVertexBuffer8;
  // Matrices
  matWorld : TD3DXMatrix; // World
  matView  : TD3DXMatrix; // View
  matProj  : TD3DXMatrix; // Projection

type
 // The structure of the custom vertex.
 TCustomVertex = record
                    Position : TD3DXVector3; // Position.
                    DColor   : TD3DColor;    // Diffuse Color.
                    U, V     : single;       // Texture coordinates.
                  end;
  PCustomVertex = ^TCustomVertex;

const
  // Flexible vector format (FVF) description.
  // Describes the custom vertex structure (TCustomVertex).
  D3DFVF_CUSTOMVERTEX = D3DFVF_XYZ or D3DFVF_DIFFUSE or D3DFVF_TEX1;

  // Automatic texture coordinate generation.
  SHOW_HOW_TO_USE_TCI   = false;
  // Texture filtering.
  USE_TEXTURE_FILTERING = true;
  TEXTFILENAME = 'CLOUDS.PNG';

function GetD3DXVector3(_x, _y, _z: single): TD3DXVector3;
begin
  Result.x := _x;
  Result.y := _y;
  Result.z := _z;
end;

function GetCustomVertex(_px, _py, _pz, _u, _v: single;
                         _dcolor: dWord): TCustomVertex;

begin
  with Result do
   begin
     Position.x := _px;
     Position.y := _py;
     Position.z := _pz;
     U := _u;
     V := _v;
     DColor := _dcolor;
   end;
end;
 
procedure TForm1.SetupMatrices;
var _v1, _v2, _v3 : TD3DXVector3;
begin
  // World.
  D3DXMatrixRotationX(matWorld, sin(GetTickCount/1000) * 1.0);
  OmegaScreen1.Device.SetTransform(D3DTS_WORLD, matWorld);

  // View.
  _v1 := GetD3DXVector3( 0.0, 0.0, -5.0);
  _v2 := GetD3DXVector3( 0.0, 0.0,  0.0);
  _v3 := GetD3DXVector3( 0.0, 1.0,  0.0);
  D3DXMatrixLookAtLH(matView, _v1, _v2, _v3);
  OmegaScreen1.Device.SetTransform(D3DTS_VIEW, matView);

  // Projection.
  D3DXMatrixPerspectiveFovLH(matProj, PI/4, 1.0, 1.0, 100.0);
  OmegaScreen1.Device.SetTransform(D3DTS_PROJECTION, matProj);
end;

function TForm1.InitVertexBuffer: boolean;
var _vb  : array[0..99] of TCustomVertex;
    _cnt : longint;
    _ptr : pByte;
    _theta : single;
begin
  Result := false;

  // Tutorial 5 - Step 2 **********************************************

  // Initialize the vertices.
  for _cnt := 0 to 49 do
    begin
      _theta := (2 * PI * _cnt) /49;
      _vb[2 * _cnt] := GetCustomVertex(sin(_theta), -1.0, cos(_theta),
                                       _cnt/49, 1.0, $FFFF7F00);
      _vb[2 * _cnt + 1] := GetCustomVertex(sin(_theta),  1.0, cos(_theta),
                                           _cnt/49, 0.0, $FFFFFFFF);
    end;

  // ******************************************************************

  // Create a vertex buffer.
  if failed(OmegaScreen1.Device.CreateVertexBuffer(SizeOf(_vb), 0,
                                       D3DFVF_CUSTOMVERTEX,
                                       D3DPOOL_MANAGED,
                                       VertexBuffer)) then exit;

  // Lock, fill and unlock vertex buffer.
  if failed(VertexBuffer.Lock(0, SizeOf(_vb), _ptr, 0)) then exit;
  Move(_vb, _ptr^, SizeOf(_vb));
  VertexBuffer.Unlock;

  // All right
  Result := true;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
 self.ClientWidth := 800;
 self.ClientHeight := 600;
 OmegaScreen1.Width := self.ClientWidth;
 OmegaScreen1.Height := self.ClientHeight;
 OmegaScreen1.Init;
 with OmegaScreen1.Device do
 begin
  // Load Texture.
  D3DXCreateTextureFromFileA(OmegaScreen1.Device, TEXTFILENAME, Texture);
  // Initialize VertexBuffer
  InitVertexBuffer;
  // Disable culling.
  SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);
  // Disable lights.
  SetRenderState(D3DRS_LIGHTING, 0);
  // Set ambiental light.
  SetRenderState(D3DRS_AMBIENT, $404040);
  // Set texture filtering
  if USE_TEXTURE_FILTERING then
  begin
   SetTextureStageState(0, D3DTSS_MAGFILTER, ord(D3DTEXF_LINEAR));
   SetTextureStageState(0, D3DTSS_MINFILTER, ord(D3DTEXF_LINEAR));
  end;
  // Set the texture that the will be used to render.
  SetTexture(0, Texture);
  // Set the texture stage state values.
  SetTextureStageState( 0, D3DTSS_COLOROP,   D3DTOP_MODULATE );
  SetTextureStageState( 0, D3DTSS_COLORARG1, D3DTA_TEXTURE );
  SetTextureStageState( 0, D3DTSS_COLORARG2, D3DTA_DIFFUSE );
  SetTextureStageState( 0, D3DTSS_ALPHAOP,   D3DTOP_DISABLE );
 end;
 OmegaTimer1.Enabled := True;
end;

procedure TForm1.OmegaTimer1Timer(Sender: TObject);
var _mat : TD3DXMatrix;
begin
 if not OmegaScreen1.CanDraw then exit;
 OmegaInput1.Update;
 if oisbutton1 in OmegaInput1.keyboard.statesclicked then
  OmegaScreen1.ToggleFullScreen;
 OmegaScreen1.BeginRender;
 OmegaScreen1.ClearScreen(255, 0, 0);
 with OmegaScreen1.Device do
 begin
  // Set Transform states
  SetupMatrices;
  
  // TCI (texture coordinate index)
  if SHOW_HOW_TO_USE_TCI then
    begin
      with _mat do
        begin
          _11 := 0.25; _12 := 0.00; _13 := 0.00; _14 := 0.00;
          _21 := 0.00; _22 :=-0.25; _23 := 0.00; _24 := 0.00;
          _31 := 0.00; _32 := 0.00; _33 := 1.00; _34 := 0.00;
          _41 := 0.50; _42 := 0.50; _43 := 0.00; _44 := 1.00;
       end;
      SetTransform( D3DTS_TEXTURE0, _mat );
      SetTextureStageState( 0, D3DTSS_TEXTURETRANSFORMFLAGS, D3DTTFF_COUNT2 );
      SetTextureStageState( 0, D3DTSS_TEXCOORDINDEX, D3DTSS_TCI_CAMERASPACEPOSITION );
      // Set the texture stage state values.
      SetTextureStageState( 0, D3DTSS_COLOROP,   D3DTOP_MODULATE );
      SetTextureStageState( 0, D3DTSS_COLORARG1, D3DTA_TEXTURE );
      SetTextureStageState( 0, D3DTSS_COLORARG2, D3DTA_DIFFUSE );
      SetTextureStageState( 0, D3DTSS_ALPHAOP,   D3DTOP_DISABLE );
   end;

  SetTexture(0, Texture);
  // Set the stream source.
  SetStreamSource( 0, VertexBuffer, sizeof(TCustomVertex));
  // Set the vertex shader.
  SetVertexShader(D3DFVF_CUSTOMVERTEX);
  // Render the vertices.
  DrawPrimitive(D3DPT_TRIANGLESTRIP, 0, 98);

  SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTOP_MODULATE);
 end;

 OmegaScreen1.SetBlendMode(bmNormal);
 OmegaBitmapFont1.Print(44, 510, 'Press "space" to toggle fullscreen');
 
 OmegaScreen1.EndRender;
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
 OmegaTimer1.Enabled := False;
end;

procedure TForm1.FormKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
 if key = 27 then close;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
 VertexBuffer := NIL;
 Texture := NIL; // Free Texture;
end;

end.



