unit Unit1;

interface
uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, OmegaImageList, OmegaScreen, OmegaInput, OmegaBitmapFont,
  OmegaTimer, OmegaUtils, D3DX8, {$I UseD3D8.inc};

type
  TForm1 = class(TForm)
    OmegaScreen1: TOmegaScreen;
    OmegaTimer1: TOmegaTimer;
    OmegaBitmapFont1: TOmegaBitmapFont;
    OmegaInput1: TOmegaInput;
    procedure FormCreate(Sender: TObject);
    procedure OmegaTimer1Timer(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormDestroy(Sender: TObject);
  private
    function InitVertexBuffer: boolean;
    procedure SetupMatrices;
    procedure SetupMaterial;
    procedure Pick;
  end;

var Form1: TForm1;

implementation
{$R *.DFM}

var
  VertexBuffer1 : IDirect3DVertexBuffer8;
  VertexBuffer2 : IDirect3DVertexBuffer8;
  NumPickedTriangles : dWord;
  PickT, PickU, PickV : single;

  // Matrices
  matWorld : TD3DXMatrix; // World
  matView  : TD3DXMatrix; // View
  matProj  : TD3DXMatrix; // Projection

type
  // The structure of the custom vertex.
  PCustomVertex = ^TCustomVertex;
  TCustomVertex = record
                    Position : TD3DXVector3; // Position
                  end;
  PCV = ^TCV;
  TCV = array[0..0] of TCustomVertex;

const
  // Flexible vector format (FVF) description.
  // Describes the custom vertex structure (TCustomVertex).
  D3DFVF_CUSTOMVERTEX = D3DFVF_XYZ;
  // Number of faces
  FACENUM = 4;

  // Display resolution
  DWIDTH  = 800;
  DHEIGHT = 600;

procedure TForm1.SetupMaterial;
var _mat : TD3DMaterial8;
begin
  // Set Material
  FillChar(_mat, sizeof(_mat), 0 );
  with _mat do
    begin
      Ambient.r := 1.0;
      Ambient.g := 1.0;
      Ambient.b := 0.0;
      Ambient.a := 1.0;
      OmegaScreen1.Device.SetMaterial(_mat);
    end;
end;

function GetD3DXVector3(_x, _y, _z: single): TD3DXVector3;
begin
  Result.x := _x;
  Result.y := _y;
  Result.z := _z;
end;

function GetCustomVertex(_px, _py, _pz: single): TCustomVertex;
begin
  with Result do
   begin
     Position.x := _px;
     Position.y := _py;
     Position.z := _pz;
   end;
end;

procedure TForm1.SetupMatrices;
var _v1, _v2, _v3 : TD3DXVector3;
    _time : dWord;
begin
  // World.
  D3DXMatrixRotationX(matWorld, 0.0);
  OmegaScreen1.Device.SetTransform(D3DTS_WORLD, matWorld);

  // Rotate the camera
  _time := GetTickCount;
  _v2 := GetD3DXVector3(0.0, 0.0, 0.0);
  _v3 := GetD3DXVector3(0.0, 1.0, 0.0);
  _v1.x := -cos(_time/3000.0) * 4.0;
  _v1.y := 1.0;
  _v1.z :=  sin(_time/3000.0) * 4.0;

  // View.
  D3DXMatrixLookAtLH(matView, _v1, _v2, _v3);
  OmegaScreen1.Device.SetTransform(D3DTS_VIEW, matView);

  // Projection.
  D3DXMatrixPerspectiveFovLH(matProj, PI/4, 1.0, 1.0, 100.0);
  OmegaScreen1.Device.SetTransform(D3DTS_PROJECTION, matProj);
end;

// -------------------------------------------------------------------
// Name: IntersectTriangle()
// Desc: Given a ray origin (orig) and direction (dir), and three
//       vertices of of a triangle, this function returns TRUE and
//       the interpolated texture coordinates if the ray intersects
//       the triangle.
// -------------------------------------------------------------------

function IntersectTriangle(_orig, _dir, _v0, _v1, _v2 : TD3DXVector3;
                           var _t, _u, _v: single): boolean;

var _edge1, _edge2 : TD3DXVector3;
    _pvec, _tvec, _qvec : TD3DXVector3;
    _det, _invDet : single;

begin
  Result := false;

  // Find vectors for two edges sharing vert0
  _edge1.x := _v1.x - _v0.x;
  _edge1.y := _v1.y - _v0.y;
  _edge1.z := _v1.z - _v0.z;

  _edge2.x := _v2.x - _v0.x;
  _edge2.y := _v2.y - _v0.y;
  _edge2.z := _v2.z - _v0.z;

  // Begin calculating determinant - also used to calculate U parameter
  D3DXVec3Cross(_pvec, _dir, _edge2);

  // If determinant is near zero, ray lies in plane of triangle
  _det := D3DXVec3Dot(_edge1, _pvec);
  if (_det < 0.0001) then exit;

  // Calculate distance from vert0 to ray origin
  _tvec.x := _orig.x - _v0.x;
  _tvec.y := _orig.y - _v0.y;
  _tvec.z := _orig.z - _v0.z;

  // Calculate U parameter and test bounds
  _u := D3DXVec3Dot(_tvec, _pvec);
  if (_u < 0.0) or (_u > _det) then exit;

  // Prepare to test V parameter
  D3DXVec3Cross(_qvec, _tvec, _edge1);

  // Calculate V parameter and test bounds
  _v := D3DXVec3Dot(_dir, _qvec);
  if (_v < 0.0) or (_u + _v > _det) then exit;

  // Calculate t, scale parameters, ray intersects triangle
  _t := D3DXVec3Dot(_edge2, _qvec);
  _invDet := 1.0 / _det;
  _t := _t * _invDet;
  _u := _u * _invDet;
  _v := _v * _invDet;

  Result := true;
end;

procedure TForm1.Pick;
var _rayDir, _rayOrig : TD3DXVector3;
    _matProj, _matView, _m : TD3DXMatrix;
    _cursor : TPoint;
    _v : TD3DXVector3;
    _p, _p2 : PCV;
    _i : dWord;
    _v0, _v1, _v2 : TD3DXVector3;
begin
  NumPickedTriangles := 0;
  // Get the pick ray from the mouse position.
  if GetCapture <> 0 then
    begin
      OmegaScreen1.Device.GetTransform(D3DTS_PROJECTION, _matProj);
      GetCursorPos(_cursor);
      _cursor := ScreenToClient(_cursor);
      // Compute the vector of the pick ray in screen space
      _v.x :=  (((2.0 * _cursor.x) / DWIDTH) - 1) / _matProj._11;
      _v.y := -(((2.0 * _cursor.y) / DHEIGHT) - 1) / _matProj._22;
      _v.z := 1.0;
      // Get the inverse view matrix
      OmegaScreen1.Device.GetTransform(D3DTS_VIEW, _matView);
      D3DXMatrixInverse(_m, NIL, _matView);
      // Transform the screen space pick ray into 3D space
      _rayDir.x  := _v.x*_m._11 + _v.y*_m._21 + _v.z*_m._31;
      _rayDir.y  := _v.x*_m._12 + _v.y*_m._22 + _v.z*_m._32;
      _rayDir.z  := _v.x*_m._13 + _v.y*_m._23 + _v.z*_m._33;
      _rayOrig.x := _m._41;
      _rayOrig.y := _m._42;
      _rayOrig.z := _m._43;
    end;
  // Get the picked triangle
  if GetCapture <> 0 then
    begin
      VertexBuffer1.Lock(0, 0, pByte(_p), 0);
      VertexBuffer2.Lock(0, 0, pByte(_p2), 0);
      for _i := 0 to FACENUM - 1 do
        begin
          _v0 := _p[_i*3+0].Position;
          _v1 := _p[_i*3+1].Position;
          _v2 := _p[_i*3+2].Position;
          // Check if the pick ray passes through this point
          if IntersectTriangle(_rayOrig, _rayDir, _v0, _v1, _v2, PickT, PickU, PickV) then
            begin
              _p2[NumPickedTriangles*3+0].Position := _v0;
              _p2[NumPickedTriangles*3+1].Position := _v1;
              _p2[NumPickedTriangles*3+2].Position := _v2;
              inc(NumPickedTriangles);
            end;
        end;
      VertexBuffer2.Unlock;
      VertexBuffer1.Unlock;
    end;
end;

function TForm1.InitVertexBuffer: boolean;
var _vb  : array[0..11] of TCustomVertex;
    _ptr : pByte;
begin
  Result := false;

  // Initialize the vertices.
  _vb[ 0] := GetCustomVertex( -1.0, -1.0,  0.0);
  _vb[ 1] := GetCustomVertex(  1.0,  1.0,  0.0);
  _vb[ 2] := GetCustomVertex(  1.0, -1.0,  0.0);

  _vb[ 3] := GetCustomVertex( -1.0, -1.0,  0.0);
  _vb[ 4] := GetCustomVertex( -1.0,  1.0,  0.0);
  _vb[ 5] := GetCustomVertex(  1.0,  1.0,  0.0);

  _vb[ 6] := GetCustomVertex(  1.0, -1.0,  0.0);
  _vb[ 7] := GetCustomVertex(  1.0,  1.0,  0.0);
  _vb[ 8] := GetCustomVertex( -1.0, -1.0,  0.0);

  _vb[ 9] := GetCustomVertex(  1.0,  1.0,  0.0);
  _vb[10] := GetCustomVertex( -1.0,  1.0,  0.0);
  _vb[11] := GetCustomVertex( -1.0, -1.0,  0.0);

  // Create a vertex buffers.
  if failed(OmegaScreen1.Device.CreateVertexBuffer(SizeOf(_vb), 0,
                                       D3DFVF_CUSTOMVERTEX,
                                       D3DPOOL_MANAGED,
                                       VertexBuffer1)) then exit;

  if failed(OmegaScreen1.Device.CreateVertexBuffer(SizeOf(_vb), 0,
                                       D3DFVF_CUSTOMVERTEX,
                                       D3DPOOL_MANAGED,
                                       VertexBuffer2)) then exit;

  // Lock, fill and unlock vertex buffer 1.
  if failed(VertexBuffer1.Lock(0, SizeOf(_vb), _ptr, 0)) then exit;
  Move(_vb, _ptr^, SizeOf(_vb));
  VertexBuffer1.Unlock;

  // All right
  Result := true;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
 self.ClientWidth := 800;
 self.ClientHeight := 600;
 OmegaScreen1.Width := self.ClientWidth;
 OmegaScreen1.Height := self.ClientHeight;
 OmegaScreen1.Init;
 with OmegaScreen1.Device do
 begin
  InitVertexBuffer;
  SetupMaterial;
 end;
 OmegaTimer1.Enabled := True;
end;

procedure TForm1.OmegaTimer1Timer(Sender: TObject);
var _cur : TPoint;
begin
 if not OmegaScreen1.CanDraw then exit;
 OmegaInput1.Update;
 if oisbutton1 in OmegaInput1.keyboard.statesclicked then
  OmegaScreen1.ToggleFullScreen;

 // Set up the cursor
 GetCursorPos(_cur);
 _cur := ScreenToClient(_cur);
 OmegaScreen1.Device.SetCursorPosition(_cur.x, _cur.y, 0);
 // Check for picked triangles
 Pick;
 OmegaScreen1.BeginRender;
 OmegaScreen1.ClearScreen(55, 0, 0);
 with OmegaScreen1.Device do
 begin
  // Clear texture
  SetTexture(0, nil);
  // Set Transform states.
  SetupMatrices;
  // Set the vertex shader.
  SetVertexShader(D3DFVF_CUSTOMVERTEX);
  // Set the stream source.
  SetStreamSource(0, VertexBuffer1, sizeof(TCustomVertex));
  // Render the vertices.
  SetRenderState(D3DRS_FILLMODE, D3DFILL_WIREFRAME);
  DrawPrimitive(D3DPT_TRIANGLELIST, 0, FACENUM);
  // Render picked faces.
  if NumPickedTriangles > 0 then
  begin
   SetStreamSource(0, VertexBuffer2, sizeof(TCustomVertex));
   SetRenderState(D3DRS_FILLMODE, D3DFILL_SOLID);
   DrawPrimitive(D3DPT_TRIANGLELIST, 0, NumPickedTriangles);
  end;

  SetRenderState(D3DRS_FILLMODE, D3DFILL_SOLID);
 end;

 OmegaScreen1.SetBlendMode(bmNormal);
 OmegaBitmapFont1.Print(44, 510, 'Press "space" to toggle fullscreen');

 OmegaScreen1.EndRender;
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
 OmegaTimer1.Enabled := False;
end;

procedure TForm1.FormKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
 if key = 27 then close;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
 VertexBuffer1 := NIL;
 VertexBuffer2 := NIL;
end;

end.



