unit Unit1;

interface
uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, OmegaImageList, OmegaScreen, OmegaInput, OmegaBitmapFont,
  D3DFile, D3DUtil, DXUtil,
  OmegaTimer, OmegaUtils, D3DX8, {$I UseD3D8.inc};

type
  TForm1 = class(TForm)
    OmegaScreen1: TOmegaScreen;
    OmegaTimer1: TOmegaTimer;
    OmegaBitmapFont1: TOmegaBitmapFont;
    OmegaInput1: TOmegaInput;
    function FrameMove: HRESULT;
    procedure FormCreate(Sender: TObject);
    procedure OmegaTimer1Timer(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormDestroy(Sender: TObject);
    procedure InitData;
  end;

var Form1: TForm1;

implementation
{$R *.DFM}

const
 WATER_COLOR      = $00004080;
 D3DFVF_D3DVERTEX = (D3DFVF_XYZ or D3DFVF_NORMAL or D3DFVF_TEX1);

type
  PD3DVERTEX= ^D3DVERTEX;
  D3DVERTEX = packed record
    p: TD3DXVector3;
    n: TD3DXVector3;
    tu, tv: Single;
  end;
  PAD3DVERTEX = ^AD3DVERTEX;
  AD3DVERTEX = array[0..0] of D3DVERTEX;

var
    // Transform matrices
    m_matWorld: TD3DXMatrix;
    m_matView: TD3DXMatrix;
    m_matProj: TD3DXMatrix;

    // Dolphin object
    m_pDolphinTexture: IDirect3DTexture8;
    m_pDolphinVB1: IDirect3DVertexBuffer8;
    m_pDolphinVB2: IDirect3DVertexBuffer8;
    m_pDolphinVB3: IDirect3DVertexBuffer8;
    m_pDolphinIB: IDirect3DIndexBuffer8;
    m_dwNumDolphinVertices: DWORD;
    m_dwNumDolphinFaces: DWORD;
    m_dwDolphinVertexShader: DWORD;
    m_dwDolphinVertexShader2: DWORD;

    // Seafloor object
    m_pSeaFloorTexture: IDirect3DTexture8;
    m_pSeaFloorVB: IDirect3DVertexBuffer8;
    m_pSeaFloorIB: IDirect3DIndexBuffer8;
    m_dwNumSeaFloorVertices: DWORD;
    m_dwNumSeaFloorFaces: DWORD;
    m_dwSeaFloorVertexShader: DWORD;
    m_dwSeaFloorVertexShader2: DWORD;

    // Water caustics
    m_pCausticTextures:array[0..31] of IDirect3DTexture8;
    m_pCurrentCausticTexture: IDirect3DTexture8;

procedure TForm1.FormCreate(Sender: TObject);
const
  RAND_MAX = $7fff;
var
  pMeshSourceVB: IDirect3DVertexBuffer8;
  pMeshSourceIB: IDirect3DIndexBuffer8;
  pSrc: PAD3DVERTEX;
  pDst: PAD3DVERTEX;
  DolphinMesh01: CD3DMesh;
  DolphinMesh02: CD3DMesh;
  DolphinMesh03: CD3DMesh;
  SeaFloorMesh: CD3DMesh;
  t, i: DWORD;
  strName: array[0..79] of Char;
begin
 self.ClientWidth := 800;
 self.ClientHeight := 600;
 OmegaScreen1.Width := self.ClientWidth;
 OmegaScreen1.Height := self.ClientHeight;
 OmegaScreen1.Init;
 with OmegaScreen1.Device do
 begin
  // Create texture for the dolphin
  D3DUtil_CreateTexture(OmegaScreen1.Device, 'Dolphin.bmp', m_pDolphinTexture);
  // Create textures for the seafloor
  D3DUtil_CreateTexture(OmegaScreen1.Device, 'SeaFloor.bmp', m_pSeaFloorTexture);
  // Create textures for the water caustics
  for t:= 0 to 31 do
  begin
    StrFmt(strName, 'Caust%.2d.tga', [t]);
    D3DUtil_CreateTexture(OmegaScreen1.Device, strName,
     m_pCausticTextures[t]);
  end;
  DolphinMesh01 := CD3DMesh.Create(nil);
  // Load the file-based mesh objects
  DolphinMesh01.Create_(OmegaScreen1.Device, Pchar('dolphin1.x'));
  DolphinMesh02:= CD3DMesh.Create(nil);
  DolphinMesh02.Create_(OmegaScreen1.Device, Pchar('dolphin2.x'));
  DolphinMesh03:= CD3DMesh.Create(nil);
  DolphinMesh03.Create_(OmegaScreen1.Device, Pchar('dolphin3.x'));
  SeaFloorMesh:= CD3DMesh.Create(nil);
  SeaFloorMesh.Create_(OmegaScreen1.Device, Pchar('SeaFloor.x'));
  // Set the FVF type to match the vertex format we want
  DolphinMesh01.SetFVF(OmegaScreen1.Device, D3DFVF_D3DVERTEX);
  DolphinMesh02.SetFVF(OmegaScreen1.Device, D3DFVF_D3DVERTEX);
  DolphinMesh03.SetFVF(OmegaScreen1.Device, D3DFVF_D3DVERTEX);
  SeaFloorMesh.SetFVF(OmegaScreen1.Device, D3DFVF_D3DVERTEX);
  // Get the number of vertices and faces for the meshes
  m_dwNumDolphinVertices  := DolphinMesh01.GetSysMemMesh.GetNumVertices;
  m_dwNumDolphinFaces     := DolphinMesh01.GetSysMemMesh.GetNumFaces;
  m_dwNumSeaFloorVertices := SeaFloorMesh.GetSysMemMesh.GetNumVertices;
  m_dwNumSeaFloorFaces    := SeaFloorMesh.GetSysMemMesh.GetNumFaces;
  // Create the dolphin and seafloor vertex and index buffers
  CreateVertexBuffer(m_dwNumDolphinVertices * sizeof(D3DVERTEX),
   D3DUSAGE_WRITEONLY, 0, D3DPOOL_MANAGED, m_pDolphinVB1);
  CreateVertexBuffer(m_dwNumDolphinVertices * sizeof(D3DVERTEX),
   D3DUSAGE_WRITEONLY, 0, D3DPOOL_MANAGED, m_pDolphinVB2);
  CreateVertexBuffer(m_dwNumDolphinVertices * sizeof(D3DVERTEX),
   D3DUSAGE_WRITEONLY, 0, D3DPOOL_MANAGED, m_pDolphinVB3);
  CreateVertexBuffer(m_dwNumSeaFloorVertices * sizeof(D3DVERTEX),
   D3DUSAGE_WRITEONLY, 0, D3DPOOL_MANAGED, m_pSeaFloorVB);
  CreateIndexBuffer(m_dwNumDolphinFaces * 3 * sizeof(WORD),
   D3DUSAGE_WRITEONLY, D3DFMT_INDEX16, D3DPOOL_MANAGED, m_pDolphinIB);
  CreateIndexBuffer(m_dwNumSeaFloorFaces * 3 * sizeof(WORD),
   D3DUSAGE_WRITEONLY, D3DFMT_INDEX16, D3DPOOL_MANAGED, m_pSeaFloorIB);
  // Copy vertices for mesh 01
  DolphinMesh01.GetSysMemMesh.GetVertexBuffer(pMeshSourceVB);
  m_pDolphinVB1.Lock(0, 0, PBYTE(pDst), 0);
  pMeshSourceVB.Lock(0, 0, PBYTE(pSrc), 0);
  Move(pSrc^,pDst^,m_dwNumDolphinVertices * sizeof(D3DVERTEX));
  m_pDolphinVB1.Unlock;
  pMeshSourceVB.Unlock;
  pMeshSourceVB:= nil;
  // Copy vertices for mesh 2
  DolphinMesh02.GetSysMemMesh.GetVertexBuffer(pMeshSourceVB);
  m_pDolphinVB2.Lock(0, 0, PBYTE(pDst), 0);
  pMeshSourceVB.Lock(0, 0, PBYTE(pSrc), 0);
  Move(pSrc^, pDst^, m_dwNumDolphinVertices * sizeof(D3DVERTEX));
  m_pDolphinVB2.Unlock;
  pMeshSourceVB.Unlock;
  pMeshSourceVB:= nil;
  // Copy vertices for mesh 3
  DolphinMesh03.GetSysMemMesh.GetVertexBuffer( pMeshSourceVB );
  m_pDolphinVB3.Lock(0, 0, PBYTE(pDst), 0);
  pMeshSourceVB.Lock(0, 0, PBYTE(pSrc), 0);
  move(pSrc^, pDst^, m_dwNumDolphinVertices * sizeof(D3DVERTEX));
  m_pDolphinVB3.Unlock;
  pMeshSourceVB.Unlock;
  pMeshSourceVB:= nil;
  // Copy vertices for the seafloor mesh, and add some bumpiness
  SeaFloorMesh.GetSysMemMesh.GetVertexBuffer(pMeshSourceVB);
  m_pSeaFloorVB.Lock(0, 0, PBYTE(pDst), 0);
  pMeshSourceVB.Lock(0, 0, PBYTE(pSrc), 0);
  Move(pSrc^,pDst^, m_dwNumSeaFloorVertices * sizeof(D3DVERTEX));
  //srand(5);
  Randomize;
  for i:= 0 to m_dwNumSeaFloorVertices-1 do
  begin
    pDst[i].p.y := pDst[i].p.y+(random(RAND_MAX)/RAND_MAX);
    pDst[i].p.y := pDst[i].p.y+(random(RAND_MAX)/RAND_MAX);
    pDst[i].p.y := pDst[i].p.y+(random(RAND_MAX)/RAND_MAX);
    pDst[i].tu  := pDst[i].tu * 10;
    pDst[i].tv  := pDst[i].tv * 10;
  end;
  m_pSeaFloorVB.Unlock;
  pMeshSourceVB.Unlock;
  pMeshSourceVB:= nil;
  // Copy indices for the dolphin mesh
  DolphinMesh01.GetSysMemMesh.GetIndexBuffer(pMeshSourceIB);
  m_pDolphinIB.Lock(0, 0, PBYTE(pDst), 0);
  pMeshSourceIB.Lock(0, 0, PBYTE(pSrc), 0);
  Move(pSrc^, pDst^, 3 * m_dwNumDolphinFaces * SizeOf(Word));
  m_pDolphinIB.Unlock;
  pMeshSourceIB.Unlock;
  pMeshSourceIB:= nil;
  // Copy indices for the seafloor mesh
  SeaFloorMesh.GetSysMemMesh.GetIndexBuffer(pMeshSourceIB);
  m_pSeaFloorIB.Lock(0, 0, PBYTE(pDst), 0);
  pMeshSourceIB.Lock(0, 0, PBYTE(pSrc), 0);
  Move(pSrc^, pDst^, 3 * m_dwNumSeaFloorFaces * SizeOf(Word));
  m_pSeaFloorIB.Unlock;
  pMeshSourceIB.Unlock;
  pMeshSourceIB:= nil;
  InitData;
  DXUtil_Timer(TIMER_START);
 end;
 OmegaTimer1.Enabled := True;
end;

procedure TForm1.InitData;
var
  vEyePt: TD3DXVector3;
  vLookatPt: TD3DXVector3;
  vUpVec: TD3DXVector3;
  fAspect: Single;
  dwDolphinVertexDecl: array[0..12] of DWORD;
  dwSeaFloorVertexDecl: array[0..4] of DWORD;
begin
  // Set the transform matrices
  vEyePt    := D3DXVector3(0.0, 0.0, -5.0);
  vLookatPt := D3DXVector3(0.0, 0.0,  0.0);
  vUpVec    := D3DXVector3(0.0, 1.0,  0.0);
  fAspect := ClientWidth / ClientHeight;
  D3DXMatrixIdentity(m_matWorld);
  D3DXMatrixLookAtLH(m_matView, vEyePt, vLookatPt, vUpVec);
  D3DXMatrixPerspectiveFovLH(m_matProj, D3DX_PI/3, fAspect, 1.0, 10000.0);

  // Set default render states
  with OmegaScreen1.Device do
  begin
   SetTextureStageState(0, D3DTSS_MINFILTER, D3DTEXF_LINEAR);
   SetTextureStageState(0, D3DTSS_MAGFILTER, D3DTEXF_LINEAR);
   SetRenderState(D3DRS_ZENABLE,   1);
   SetRenderState(D3DRS_FOGENABLE, 1);
   SetRenderState(D3DRS_FOGCOLOR,  WATER_COLOR);
  end;
  // Create vertex shader for the dolphin
  dwDolphinVertexDecl[0]  := D3DVSD_STREAM(0);
  dwDolphinVertexDecl[1]  := D3DVSD_REG(0, D3DVSDT_FLOAT3); // Position of first mesh
  dwDolphinVertexDecl[2]  := D3DVSD_REG(3, D3DVSDT_FLOAT3); // Normal
  dwDolphinVertexDecl[3]  := D3DVSD_REG(6, D3DVSDT_FLOAT2); // Tex coords
  dwDolphinVertexDecl[4]  := D3DVSD_STREAM(1);
  dwDolphinVertexDecl[5]  := D3DVSD_REG(1, D3DVSDT_FLOAT3); // Position of second mesh
  dwDolphinVertexDecl[6]  := D3DVSD_REG(4, D3DVSDT_FLOAT3); // Normal
  dwDolphinVertexDecl[7]  := D3DVSD_REG(7, D3DVSDT_FLOAT2); // Tex coords
  dwDolphinVertexDecl[8]  := D3DVSD_STREAM(2);
  dwDolphinVertexDecl[9]  := D3DVSD_REG(2, D3DVSDT_FLOAT3); // Position of second mesh
  dwDolphinVertexDecl[10] := D3DVSD_REG(5, D3DVSDT_FLOAT3); // Normal
  dwDolphinVertexDecl[11] := D3DVSD_REG(8, D3DVSDT_FLOAT2); // Tex coords
  dwDolphinVertexDecl[12] := D3DVSD_END;
  D3DUtil_CreateVertexShader(OmegaScreen1.Device, 'DolphinTween.vsh',
   @dwDolphinVertexDecl, m_dwDolphinVertexShader);
  D3DUtil_CreateVertexShader(OmegaScreen1.Device, 'DolphinTween2.vsh',
   @dwDolphinVertexDecl, m_dwDolphinVertexShader2);
  // Create vertex shader for the seafloor
  dwSeaFloorVertexDecl[0] := D3DVSD_STREAM(0);
  dwSeaFloorVertexDecl[1] := D3DVSD_REG(0, D3DVSDT_FLOAT3); // Position of first mesh
  dwSeaFloorVertexDecl[2] := D3DVSD_REG(3, D3DVSDT_FLOAT3); // Normal
  dwSeaFloorVertexDecl[3] := D3DVSD_REG(6, D3DVSDT_FLOAT2); // Tex coords
  dwSeaFloorVertexDecl[4] := D3DVSD_END;
  D3DUtil_CreateVertexShader(OmegaScreen1.Device, 'SeaFloor.vsh',
   @dwSeaFloorVertexDecl, m_dwSeaFloorVertexShader);
  D3DUtil_CreateVertexShader(OmegaScreen1.Device, 'SeaFloor2.vsh',
   @dwSeaFloorVertexDecl, m_dwSeaFloorVertexShader2);
end;

procedure TForm1.FormDestroy(Sender: TObject);
var i: DWORD;
begin
 // Clean up vertex shaders
 if (m_dwDolphinVertexShader <> 0) then
 begin
   OmegaScreen1.Device.DeleteVertexShader(m_dwDolphinVertexShader);
   m_dwDolphinVertexShader := 0;
 end;
 if (m_dwDolphinVertexShader2 <> 0) then
 begin
  OmegaScreen1.Device.DeleteVertexShader(m_dwDolphinVertexShader2);
  m_dwDolphinVertexShader2 := 0;
 end;
 if (m_dwSeaFloorVertexShader <> 0) then
 begin
  OmegaScreen1.Device.DeleteVertexShader(m_dwSeaFloorVertexShader);
  m_dwSeaFloorVertexShader := 0;
 end;
 if (m_dwSeaFloorVertexShader2 <> 0) then
 begin
  OmegaScreen1.Device.DeleteVertexShader(m_dwSeaFloorVertexShader2);
  m_dwSeaFloorVertexShader2 := 0;
 end;
 m_pDolphinTexture := nil;
 m_pDolphinVB1 := nil;
 m_pDolphinVB2 := nil;
 m_pDolphinVB3 := nil;
 m_pDolphinIB := nil;
 // Clean up seafloor objects
 m_pSeaFloorTexture := nil;
 m_pSeaFloorVB := nil;
 m_pSeaFloorIB := nil;
 // Clean up textures for water caustics
 for i:= 0 to 31 do
  m_pCausticTextures[i] := nil;
end;

function TForm1.FrameMove: HRESULT;
{WRITEABLECONST ON}
const
  fLight: array[0..3] of Single   = (0.0, 1.0, 0.0, 0.0);
  fLightDolphinSpace: array[0..3] of Single = (0.0, 1.0, 0.0, 0.0);
  fDiffuse: array[0..3] of Single = (1.00, 1.00, 1.00, 1.00);
  fAmbient: array[0..3] of Single = (0.25, 0.25, 0.25, 0.25);
  fFog: array[0..3] of Single     = (0.5, 50.0, 1.0/(50.0-1.0), 0.0);
  fCaustics: array[0..3] of Single = (0.05, 0.05, 0, 0);
{WRITEABLECONST OFF}
var
  fKickFreq,fPhase,fPhase2,fBlendWeight:Single;
  matDolphin, matTrans, matRotate1, matRotate2: TD3DXMatrix;
  fWeight1, fWeight2, fWeight3: Single;
  vZero, vOne: TD3DXVector4;
  vWeight: TD3DXVector4;
  tex: DWORD;
  mat, matCamera, matTranspose, matCameraTranspose: TD3DXMatrix;
  matViewTranspose, matProjTranspose: TD3DXMatrix;
  matDolphinInv: TD3DXMatrix;
  m_fTime: single;
begin
  m_fTime        := DXUtil_Timer(TIMER_GETAPPTIME);
  // Animation attributes for the dolphin
  fKickFreq    := 2 * m_fTime;
  fPhase       := m_fTime / 3;
  fPhase2      := fPhase2 + 0.006;
  fBlendWeight := sin( fKickFreq );

  // Move the dolphin in a circle
  D3DXMatrixScaling(matDolphin, 0.01, 0.01, 0.01);
  D3DXMatrixRotationZ(matRotate1, -cos(fKickFreq)/6);
  D3DXMatrixMultiply(matDolphin, matDolphin, matRotate1);
  D3DXMatrixRotationY(matRotate2, fPhase);
  D3DXMatrixMultiply(matDolphin, matDolphin, matRotate2);
  D3DXMatrixTranslation(matTrans, -5*sin(fPhase), Sin(fKickFreq)/2, 10-10*cos(fPhase));
  D3DXMatrixMultiply(matDolphin, matDolphin, matTrans);

  // Animate the caustic textures
  tex := Trunc(m_fTime * 32) mod 32;
  m_pCurrentCausticTexture := m_pCausticTextures[tex];

  // Set the vertex shader constants. Note: outside of the blend matrices,
  // most of these values don't change, so don't need to really be set every
  // frame. It's just done here for clarity
  begin
    // Some basic constants
    vZero:= D3DXVector4(0.0, 0.0, 0.0, 0.0);
    vOne:= D3DXVector4(1.0, 0.5, 0.2, 0.05);

    if (fBlendWeight > 0.0) then
    begin
      fWeight1 := abs(fBlendWeight);
      fWeight2 := 1.0 - abs(fBlendWeight);
      fWeight3 := 0.0;
    end else
    begin
      fWeight1 := 0.0;
      fWeight2 := 1.0 - abs(fBlendWeight);
      fWeight3 := abs(fBlendWeight);
    end;
    vWeight:= D3DXVector4(fWeight1, fWeight2, fWeight3, 0.0);

    // Lighting vectors (in world space and in dolphin model space)
    // and other constants
//    fCaustics[2]:= Sin(m_fTime)/8;
  //  fCaustics[3]:= Cos(m_fTime)/10;

    D3DXMatrixInverse(matDolphinInv, nil, matDolphin);
    D3DXVec4Transform(PD3DXVector4(@fLightDolphinSpace)^, PD3DXVector4(@fLight)^, matDolphinInv);
    D3DXVec4Normalize(PD3DXVector4(@fLightDolphinSpace)^, PD3DXVector4(@fLightDolphinSpace)^);

    // Vertex shader operations use transposed matrices
    D3DXMatrixMultiply(matCamera, matDolphin, m_matView);
    D3DXMatrixMultiply(mat, matCamera, m_matProj);
    D3DXMatrixTranspose(matTranspose, mat);
    D3DXMatrixTranspose(matCameraTranspose, matCamera);
    D3DXMatrixTranspose(matViewTranspose, m_matView);
    D3DXMatrixTranspose(matProjTranspose, m_matProj);

    with OmegaScreen1.Device do
    begin
     // Set the vertex shader constants
     SetVertexShaderConstant( 0, vZero,     1);
     SetVertexShaderConstant( 1, vOne,      1);
     SetVertexShaderConstant( 2, vWeight,   1);
     SetVertexShaderConstant( 4, matTranspose, 4 );
     SetVertexShaderConstant( 8, matCameraTranspose,  4);
     SetVertexShaderConstant(12, matViewTranspose,  4);
     SetVertexShaderConstant(19, fLightDolphinSpace, 1);
     SetVertexShaderConstant(20, fLight,    1);
     SetVertexShaderConstant(21, fDiffuse,  1);
     SetVertexShaderConstant(22, fAmbient,  1);
     SetVertexShaderConstant(23, fFog,      1);
     SetVertexShaderConstant(24, fCaustics, 1);
     SetVertexShaderConstant(28, matProjTranspose, 4);
    end;
  end;
  Result := S_OK;
end;

procedure TForm1.OmegaTimer1Timer(Sender: TObject);
const
  fAmbientLight: array[0..3] of Single = (0.25, 0.25, 0.25, 0.25);
  fAmbientDark:  array[0..3] of Single = (0.0, 0.0, 0.0, 0.0);
begin
 if not OmegaScreen1.CanDraw then exit;
 OmegaInput1.Update;
 if oisbutton1 in OmegaInput1.keyboard.statesclicked then
 begin
  OmegaScreen1.ToggleFullScreen;
  InitData;
 end; 
 OmegaScreen1.Device.Clear(0, nil, D3DCLEAR_TARGET or D3DCLEAR_ZBUFFER,
                     WATER_COLOR, 1.0, 0);
 OmegaScreen1.BeginRender;
 with OmegaScreen1.Device do
 begin
  SetVertexShaderConstant(22, fAmbientLight, 1);
  // Render the seafloor
  SetTexture(0, m_pSeaFloorTexture);
  SetVertexShader(m_dwSeaFloorVertexShader);
  SetStreamSource(0, m_pSeaFloorVB, SizeOf(D3DVERTEX));
  SetIndices(m_pSeaFloorIB, 0);
  DrawIndexedPrimitive(D3DPT_TRIANGLELIST,
   0, m_dwNumSeaFloorVertices, 0, m_dwNumSeaFloorFaces);
  // Render the dolphin
  SetTexture(0, m_pDolphinTexture);
  SetVertexShader(m_dwDolphinVertexShader);
  SetStreamSource(0, m_pDolphinVB1, sizeof(D3DVERTEX));
  SetStreamSource(1, m_pDolphinVB2, sizeof(D3DVERTEX));
  SetStreamSource(2, m_pDolphinVB3, sizeof(D3DVERTEX));
  SetIndices(m_pDolphinIB, 0 );
  DrawIndexedPrimitive(D3DPT_TRIANGLELIST,
   0, m_dwNumDolphinVertices, 0, m_dwNumDolphinFaces);
  // Now, we are going to do a 2nd pass, to alpha-blend in the caustics.
  // The caustics use a 2nd set of texture coords that are generated
  // by the vertex shaders. Lighting from the light above is used, but
  // ambient is turned off to avoid lighting objects from below (for
  // instance, we don't want caustics appearing on the dolphin's
  // underbelly). Finally, fog color is set to black, so that caustics
  // fade in distance.
  // Turn on alpha blending
  SetRenderState(D3DRS_ALPHABLENDENABLE, 1);
  SetRenderState(D3DRS_SRCBLEND,  D3DBLEND_ONE);
  SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ONE);
  // Setup the caustic texture
  SetTexture(0, m_pCurrentCausticTexture);
  // Set ambient and fog colors to black
  SetVertexShaderConstant(22, fAmbientDark, 1);
  SetRenderState(D3DRS_FOGCOLOR, $00000000);
  // Render the caustic effects for the seafloor
  SetVertexShader(m_dwSeaFloorVertexShader2);
  SetStreamSource(0, m_pSeaFloorVB, sizeof(D3DVERTEX));
  SetIndices(m_pSeaFloorIB, 0);
  DrawIndexedPrimitive(D3DPT_TRIANGLELIST,
   0, m_dwNumSeaFloorVertices, 0, m_dwNumSeaFloorFaces);
  // Finally, render the caustic effects for the dolphin
  SetVertexShader(m_dwDolphinVertexShader2);
  SetStreamSource(0, m_pDolphinVB1, sizeof(D3DVERTEX));
  SetStreamSource(1, m_pDolphinVB2, sizeof(D3DVERTEX));
  SetStreamSource(2, m_pDolphinVB3, sizeof(D3DVERTEX));
  SetIndices(m_pDolphinIB, 0);
  DrawIndexedPrimitive(D3DPT_TRIANGLELIST,
   0, m_dwNumDolphinVertices, 0, m_dwNumDolphinFaces);
  // Restore modified render states
  SetRenderState(D3DRS_ALPHABLENDENABLE, 0);
  SetRenderState(D3DRS_FOGCOLOR, WATER_COLOR);
  FrameMove;
 end; 
// OmegaScreen1.SetBlendMode(bmNormal);
// OmegaBitmapFont1.Print(44, 510, 'Press "space" to toggle fullscreen');
 OmegaScreen1.EndRender;
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
 OmegaTimer1.Enabled := False;
end;

procedure TForm1.FormKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
 if key = 27 then close;
end;

end.





