unit Unit1;

interface
uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, OmegaImageList, OmegaScreen, OmegaInput, OmegaBitmapFont,
  DXUtil, D3DUtil, OmegaTimer, OmegaUtils, D3DX8, {$I UseD3D8.inc};

type
  TForm1 = class(TForm)
    OmegaScreen1: TOmegaScreen;
    OmegaTimer1: TOmegaTimer;
    OmegaBitmapFont1: TOmegaBitmapFont;
    OmegaInput1: TOmegaInput;
    procedure FormCreate(Sender: TObject);
    procedure OmegaTimer1Timer(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure UpdateCamera;
    procedure FormDestroy(Sender: TObject);
   private
    showhelp: boolean;
    fSpeed: Single;
    m_fTime, m_fElapsedTime: single;
    procedure InitData;
  end;

type
  PParticle = ^TParticle;
  TParticle = packed record
    m_bSpark:   Boolean;      // Sparks are less energetic particles that
                              // are generated where/when the main particles
                              // hit the ground
    m_vPos:     TD3DXVector3;  // Current position
    m_vVel:     TD3DXVector3;  // Current velocity
    m_vPos0:    TD3DXVector3; // Initial position
    m_vVel0:    TD3DXVector3; // Initial velocity
    m_fTime0:   Single;       // Time of creation
    m_clrDiffuse:       TD3DXColor;  // Initial diffuse color
    m_clrFade:          TD3DXColor;  // Faded diffuse color
    m_fFade:            Single;      // Fade progression
    m_pNext: PParticle;      // Next particle in list
  end;
  TParticleColors = (COLOR_WHITE, COLOR_RED, COLOR_GREEN, COLOR_BLUE, NUM_COLORS);

var
  g_clrColor: array[COLOR_WHITE..COLOR_BLUE] of TD3DXColor =
  (
    (r: 1.0;   g: 1.0; b: 1.0; a: 1.0),
    (r: 1.0;   g: 0.5; b: 0.5; a: 1.0),
    (r: 0.5;   g: 0.5; b: 0.5; a: 1.0),
    (r: 0.125; g: 0.5; b: 1.0; a: 1.0)
  );

  g_clrColorFade: array[COLOR_WHITE..COLOR_BLUE] of TD3DXColor =
  (
    (r: 1.0;   g: 0.25; b: 0.25; a: 1.0),
    (r: 1.0;   g: 0.25; b: 0.25; a: 1.0),
    (r: 0.25;  g: 0.75; b: 0.25; a: 1.0),
    (r: 0.125; g: 0.25; b: 0.75; a: 1.0)
  );

type
  CParticleSystem = class
  protected
    m_fRadius:         Single;
    m_dwBase:          DWORD;
    m_dwFlush:         DWORD;
    m_dwDiscard:       DWORD;
    m_dwParticles:     DWORD;
    m_dwParticlesLim:  DWORD;
    m_pParticles:      PParticle;
    m_pParticlesFree:  PParticle;
    // Geometry
    m_pVB:             IDirect3DVertexBuffer8;
  public
    constructor Create(dwFlush, dwDiscard: DWord; fRadius: Single);
    destructor Destroy; override;
    function RestoreDeviceObjects(pd3dDevice: IDirect3DDevice8): HResult;
    function InvalidateDeviceObjects: HResult;
    function Update(fSecsPerFrame: Single; dwNumParticlesToEmit: DWord;
                    const clrEmitColor, clrFadeColor: TD3DXColor;
                    fEmitVel: Single; vPosition: TD3DXVector3): HResult;
    function Render(pd3dDevice: IDirect3DDevice8): HResult;
  end;

var Form1: TForm1;

implementation
{$R *.DFM}

//-----------------------------------------------------------------------------
// Custom vertex types
//-----------------------------------------------------------------------------
type
  PColorVertex = ^TColorVertex;
  TColorVertex = packed record
    v: TD3DXVector3;
    color: TD3DColor;
    tu: Single;
    tv: Single;
  end;
  PColorVertexArray = ^TColorVertexArray;
  TColorVertexArray = array [0..0] of TColorVertex;

const
  D3DFVF_COLORVERTEX = (D3DFVF_XYZ or D3DFVF_DIFFUSE or D3DFVF_TEX1);
  D3DFVF_POINTVERTEX = (D3DFVF_XYZ or D3DFVF_DIFFUSE);

type
  PPointVertex = ^TPointVertex;
  TPointVertex = packed record
    v: TD3DXVector3;
    color: TD3DColor;
  end;
  PPointVertexArray = ^TPointVertexArray;
  TPointVertexArray = array [0..0] of TPointVertex;

//-----------------------------------------------------------------------------
// Global structs and data for the ground object
//-----------------------------------------------------------------------------
const
  GROUND_WIDTH          = 256.0;
  GROUND_HEIGHT         = 256.0;
  GROUND_GRIDSIZE       = 8;
  GROUND_TILE           = 32;
  GROUND_COLOR          = $cccccccc;

//-----------------------------------------------------------------------------
// Global data for the particles
//-----------------------------------------------------------------------------
var
    // Ground stuff
    m_pGroundTexture:   IDirect3DTexture8;
    m_planeGround:      TD3DXPlane;
    m_pGroundVB:            IDirect3DVertexBuffer8;
    m_pGroundIB:            IDirect3DIndexBuffer8;
    m_dwNumGroundIndices:   DWORD;
    m_dwNumGroundVertices:  DWORD;
    // Particle stuff
    m_pParticleTexture:     IDirect3DTexture8;
    m_pParticleSystem:      CParticleSystem;
    m_dwNumParticlesToEmit: DWORD;
    m_dwParticleColor:      TParticleColors;
    m_bAnimateEmitter:      Boolean;
    m_bKey:                 array[0..255] of Byte;
    m_bDrawReflection:      Boolean;
    m_bCanDoAlphaBlend:     Boolean;
    m_bDrawHelp:            Boolean;
    // Variables for determining view position
    m_vPosition:            TD3DXVector3;
    m_vVelocity:            TD3DXVector3;
    m_fYaw:                 Single;
    m_fYawVelocity:         Single;
    m_fPitch:               Single;
    m_fPitchVelocity:       Single;
    m_matView:              TD3DXMatrix;
    m_matOrientation:       TD3DXMatrix;

function FtoDW(f: Single): DWORD;
begin
  Result:= PDWord(@f)^;
end;

procedure TForm1.FormCreate(Sender: TObject);
var
  pVertices: PColorVertex;
  zz, xx: DWord;
  pIndices: PWord;
  z, x: DWord;
  vtx: DWord; 
begin
 showhelp := true;
 self.ClientWidth := 800;
 self.ClientHeight := 600;
 OmegaScreen1.Width := self.ClientWidth;
 OmegaScreen1.Height := self.ClientHeight;
 OmegaScreen1.Init;

  m_pParticleSystem      := CParticleSystem.Create(512, 2048, 0.03);
  m_pParticleTexture     := nil;
  m_dwNumParticlesToEmit := 10;
  m_bAnimateEmitter      := FALSE;
  m_dwParticleColor      := COLOR_WHITE;

  m_pGroundTexture       := nil;
  m_dwNumGroundVertices  := (GROUND_GRIDSIZE + 1) * (GROUND_GRIDSIZE + 1);
  m_dwNumGroundIndices   := (GROUND_GRIDSIZE * GROUND_GRIDSIZE) * 6;
  m_pGroundVB            := nil;
  m_pGroundIB            := nil;
  m_planeGround          := D3DXPlane(0.0, 1.0, 0.0, 0.0);

  m_bDrawReflection      := FALSE;
  m_bCanDoAlphaBlend     := FALSE;
  m_bDrawHelp            := FALSE;

  ZeroMemory(@m_bKey, 256);
  m_vPosition      := D3DXVector3( 0.0, 3.0,-4.0);
  m_vVelocity      := D3DXVector3( 0.0, 0.0, 0.0);
  m_fYaw           := 0.03;
  m_fYawVelocity   := 0.0;
  m_fPitch         := 0.5;
  m_fPitchVelocity := 0.0;
  D3DXMatrixTranslation(m_matView, 0.0, 0.0, 10.0);
  D3DXMatrixTranslation(m_matOrientation, 0.0, 0.0, 0.0);
  // Create textures
  D3DUtil_CreateTexture(OmegaScreen1.Device, 'Ground2.bmp', m_pGroundTexture);
  D3DUtil_CreateTexture(OmegaScreen1.Device, 'Particle.bmp', m_pParticleTexture);
  // Check if we can do the reflection effect
  m_bCanDoAlphaBlend:= ((OmegaScreen1.FD3DDEVCAPS8.SrcBlendCaps  and D3DPBLENDCAPS_SRCALPHA) <> 0) and
                       ((OmegaScreen1.FD3DDEVCAPS8.DestBlendCaps and D3DPBLENDCAPS_INVSRCALPHA) <> 0);

  if (m_bCanDoAlphaBlend) then m_bDrawReflection := TRUE;

  // Create ground object
  begin
    // Create vertex buffer for ground object
    OmegaScreen1.Device.CreateVertexBuffer(m_dwNumGroundVertices*SizeOf(TColorVertex),
                            D3DUSAGE_WRITEONLY, D3DFVF_COLORVERTEX, D3DPOOL_MANAGED, m_pGroundVB);
    // Fill vertex buffer
    m_pGroundVB.Lock(0, 0, PByte(pVertices), 0);
    // Fill in vertices
    for zz:= 0 to GROUND_GRIDSIZE do
    begin
      for xx := 0 to GROUND_GRIDSIZE do
      begin
        pVertices.v.x   := GROUND_WIDTH * (xx/GROUND_GRIDSIZE-0.5);
        pVertices.v.y   := 0.0;
        pVertices.v.z   := GROUND_HEIGHT * (zz/GROUND_GRIDSIZE-0.5);
        pVertices.color := GROUND_COLOR;
        pVertices.tu    := xx*GROUND_TILE/GROUND_GRIDSIZE;
        pVertices.tv    := zz*GROUND_TILE/GROUND_GRIDSIZE;
        Inc(pVertices);
      end;
    end;
    m_pGroundVB.Unlock;
    // Create the index buffer
    OmegaScreen1.Device.CreateIndexBuffer(m_dwNumGroundIndices*SizeOf(Word),
                            D3DUSAGE_WRITEONLY, D3DFMT_INDEX16, D3DPOOL_MANAGED,
                            m_pGroundIB);
    // Fill the index buffer
    m_pGroundIB.Lock(0, m_dwNumGroundIndices*sizeof(WORD), PByte(pIndices), 0);
    // Fill in indices
    for z:= 0 to GROUND_GRIDSIZE - 1 do
    begin
      for x:= 0 to GROUND_GRIDSIZE - 1 do
      begin
        vtx:= x + z * (GROUND_GRIDSIZE+1);
        pIndices^:= (vtx + 1);                          Inc(pIndices);
        pIndices^:= (vtx + 0);                          Inc(pIndices);
        pIndices^:= (vtx + 0 + (GROUND_GRIDSIZE+1));    Inc(pIndices);
        pIndices^:= (vtx + 1);                          Inc(pIndices);
        pIndices^:= (vtx + 0 + (GROUND_GRIDSIZE+1));    Inc(pIndices);
        pIndices^:= (vtx + 1 + (GROUND_GRIDSIZE+1));    Inc(pIndices);
      end;
    end;
    m_pGroundIB.Unlock;
  end;
 InitData;
 OmegaTimer1.Enabled := True;
 OmegaInput1.Keyboard.KeyAssigns[oisButton2] := VKtoDIK(ord('W'));
 OmegaInput1.Keyboard.KeyAssigns[oisButton3] := VKtoDIK(ord('S'));
 OmegaInput1.Keyboard.KeyAssigns[oisButton4] := VKtoDIK(ord('E'));
 OmegaInput1.Keyboard.KeyAssigns[oisButton5] := VKtoDIK(ord('Q'));
 OmegaInput1.Keyboard.KeyAssigns[oisButton6] := VKtoDIK(ord('Z'));
 OmegaInput1.Keyboard.KeyAssigns[oisButton7] := VKtoDIK(ord('A'));
 OmegaInput1.Keyboard.KeyAssigns[oisButton8] := VKtoDIK(ord('V'));
 OmegaInput1.Keyboard.KeyAssigns[oisButton9] := VKtoDIK(ord('B'));
 OmegaInput1.Keyboard.KeyAssigns[oisButton10] := VKtoDIK(ord('R'));
 OmegaInput1.Keyboard.KeyAssigns[oisButton11] := VKtoDIK(ord('N'));
 OmegaInput1.Keyboard.KeyAssigns[oisButton12] := VKtoDIK(ord('C'));
 OmegaInput1.Keyboard.KeyAssigns[oisButton13] := VKtoDIK(ord('H'));
end;

procedure TForm1.OmegaTimer1Timer(Sender: TObject);
var
  vEmitterPostion: TD3DXVector3;
  matReflectedView: TD3DXMatrix;
begin
 if not OmegaScreen1.CanDraw then exit;
 OmegaInput1.Update;
 if oisLeft in OmegaInput1.Keyboard.States then
  m_vVelocity.x    := m_vVelocity.x - fSpeed; // Slide Left
 if oisRight in OmegaInput1.Keyboard.States then
  m_vVelocity.x    := m_vVelocity.x + fSpeed; // Slide Right
 if oisUp in OmegaInput1.Keyboard.States then
  m_vVelocity.y    := m_vVelocity.y + fSpeed; // Slide Up
 if oisDown in OmegaInput1.Keyboard.States then
  m_vVelocity.y    := m_vVelocity.y - fSpeed; // Slide Down
 if oisbutton2 in OmegaInput1.keyboard.states then // W
  m_vVelocity.z    := m_vVelocity.z + fSpeed; // Move Forward
 if oisbutton3 in OmegaInput1.keyboard.states then // S
  m_vVelocity.z    := m_vVelocity.z - fSpeed; // Move Backward
 if oisbutton4 in OmegaInput1.keyboard.states then // E
  m_fYawVelocity   := m_fYawVelocity + fSpeed; // Turn Right
 if oisbutton5 in OmegaInput1.keyboard.states then // Q
  m_fYawVelocity   := m_fYawVelocity - fSpeed; // Turn Left
 if oisbutton6 in OmegaInput1.keyboard.states then // Z
  m_fPitchVelocity := m_fPitchVelocity + fSpeed; // Turn Down
 if oisbutton7 in OmegaInput1.keyboard.states then // A
  m_fPitchVelocity := m_fPitchVelocity - fSpeed; // Turn Up
 if oisbutton8 in OmegaInput1.keyboard.states then // V
  if (m_dwNumParticlesToEmit < 10) then Inc(m_dwNumParticlesToEmit);
 if oisbutton9 in OmegaInput1.keyboard.states then // B
  if (m_dwNumParticlesToEmit > 0)  then Dec(m_dwNumParticlesToEmit);
 if oisbutton10 in OmegaInput1.keyboard.states then // R
  if m_bCanDoAlphaBlend then m_bDrawReflection := not m_bDrawReflection;
 if oisbutton11 in OmegaInput1.keyboard.states then // N
  m_bAnimateEmitter := not m_bAnimateEmitter;
 if oisbutton12 in OmegaInput1.keyboard.states then // C
 begin
  Inc(m_dwParticleColor);
  if (m_dwParticleColor >= NUM_COLORS) then
   m_dwParticleColor := COLOR_WHITE;
 end;
 if oisbutton13 in OmegaInput1.keyboard.statesclicked then // H
  showhelp := not showhelp;

 if oisbutton1 in OmegaInput1.keyboard.statesclicked then
  OmegaScreen1.ToggleFullScreen;
 OmegaScreen1.ClearScreen(0, 0, 0);
 OmegaScreen1.BeginRender;

  m_fElapsedTime := DXUtil_Timer(TIMER_GETELAPSEDTIME);
  m_fTime        := DXUtil_Timer(TIMER_GETAPPTIME);

  UpdateCamera;
  // Determine emitter position
  vEmitterPostion:= D3DXVector3Zero;
  if (m_bAnimateEmitter) then
    vEmitterPostion:= D3DXVector3(3*Sin(m_fTime), 0.0, 3*Cos(m_fTime));

  // Update particle system
  m_pParticleSystem.Update(m_fElapsedTime, m_dwNumParticlesToEmit,
                           g_clrColor[m_dwParticleColor],
                           g_clrColorFade[m_dwParticleColor], 8.0,
                           vEmitterPostion);
  with OmegaScreen1.Device do
  begin
  // Draw reflection of particles
  if m_bDrawReflection then
  begin
    D3DXMatrixReflect (matReflectedView, m_planeGround);
    D3DXMatrixMultiply(matReflectedView, matReflectedView, m_matView);
    SetRenderState(D3DRS_ZWRITEENABLE, iFALSE);
    SetRenderState(D3DRS_ALPHABLENDENABLE, iTRUE);
    SetRenderState(D3DRS_SRCBLEND, D3DBLEND_ONE);
    SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ONE);
    SetTransform(D3DTS_VIEW, matReflectedView);
    SetTexture(0, m_pParticleTexture);
    m_pParticleSystem.Render(OmegaScreen1.Device);
    SetRenderState(D3DRS_ZWRITEENABLE, iTRUE);
    SetRenderState(D3DRS_ALPHABLENDENABLE, iFALSE);
  end;
  // Draw the ground
  if m_bDrawReflection then
  begin
    SetRenderState(D3DRS_ALPHABLENDENABLE, iTRUE);
    SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
    SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
  end;
  SetTransform(D3DTS_VIEW, m_matView);
  SetTexture(0, m_pGroundTexture);
  SetVertexShader(D3DFVF_COLORVERTEX);
  SetStreamSource(0, m_pGroundVB, SizeOf(TColorVertex));
  SetIndices(m_pGroundIB, 0);
  DrawIndexedPrimitive(D3DPT_TRIANGLELIST, 0, m_dwNumGroundVertices,
    0, m_dwNumGroundIndices div 3);
  // Draw particles
  SetRenderState(D3DRS_ZWRITEENABLE, iFALSE);
  SetRenderState(D3DRS_ALPHABLENDENABLE, iTRUE);
  SetRenderState(D3DRS_SRCBLEND,  D3DBLEND_ONE);
  SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ONE);
  SetTexture(0, m_pParticleTexture);
  m_pParticleSystem.Render(OmegaScreen1.Device);
  SetRenderState(D3DRS_ZWRITEENABLE, iTRUE);
  SetRenderState(D3DRS_ALPHABLENDENABLE, iTRUE);
  SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
  SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA );
  end;

 OmegaScreen1.SetBlendMode(bmNormal);
 if showhelp then
 begin
  OmegaBitmapFont1.Print(44, 390, 'Press "H" to show/hide help');
  OmegaBitmapFont1.Print(44, 410, 'Press "W" and "S" to move forward/backward');
  OmegaBitmapFont1.Print(44, 430, 'Press "Q" and "E" to turn left/right');
  OmegaBitmapFont1.Print(44, 450, 'Press "A" and "Y" to turn up/down');
  OmegaBitmapFont1.Print(44, 470, 'Press arrow keys to slide');
  OmegaBitmapFont1.Print(44, 490, 'Press "N" to animate emitter');
  OmegaBitmapFont1.Print(44, 510, 'Press "C" to change color');
  OmegaBitmapFont1.Print(44, 530, 'Press "R" to toggle reflection');
  OmegaBitmapFont1.Print(44, 550, 'Press "space" to toggle fullscreen');
 end;
 OmegaScreen1.EndRender;
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
 OmegaTimer1.Enabled := False;
end;

procedure TForm1.FormKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
 if key = 27 then close;
end;

procedure TForm1.UpdateCamera;
var
  fElapsedTime: Single;
  fAngularSpeed: Single;
  vT: TD3DXVector3;
  qR: TD3DXQuaternion;
begin
  if (m_fElapsedTime > 0.0)
    then fElapsedTime:= m_fElapsedTime
    else fElapsedTime:= 0.05;
  fSpeed        := 0.3*fElapsedTime*5;
  fAngularSpeed := 1.0*fElapsedTime*5;
  // De-accelerate the camera movement (for smooth motion)
  D3DXVec3Scale(m_vVelocity, m_vVelocity, 0.9);
  m_fYawVelocity:= m_fYawVelocity   * 0.9;
  m_fPitchVelocity:= m_fPitchVelocity * 0.9;
  // Update the position vector
  D3DXVec3Scale(vT, m_vVelocity, fSpeed);
  D3DXVec3TransformNormal(vT, vT, m_matOrientation);
  D3DXVec3Add(m_vPosition, m_vPosition, vT);
  if (m_vPosition.y < 1.0) then m_vPosition.y := 1.0;
  // Update the yaw-pitch-rotation vector
  m_fYaw   := m_fYaw + fAngularSpeed * m_fYawVelocity;
  m_fPitch := m_fPitch + fAngularSpeed * m_fPitchVelocity;
  if (m_fPitch < 0.0)       then m_fPitch:= 0.0;
  if (m_fPitch > D3DX_PI/2) then m_fPitch:= D3DX_PI/2;
  // Set the view matrix
  D3DXQuaternionRotationYawPitchRoll(qR, m_fYaw, m_fPitch, 0.0);
  D3DXMatrixAffineTransformation(m_matOrientation, 1.25, nil, @qR, @m_vPosition);
  D3DXMatrixInverse(m_matView, nil, m_matOrientation);
end;

procedure TForm1.InitData;
var
  matWorld: TD3DXMatrix;
  matProj: TD3DXMatrix;
  fAspect: Single;
begin
  // Set the world matrix
  D3DXMatrixIdentity(matWorld);
  OmegaScreen1.Device.SetTransform(D3DTS_WORLD, matWorld);
  // Set projection matrix
  fAspect := (ClientWidth) / ClientHeight;
  D3DXMatrixPerspectiveFovLH(matProj, D3DX_PI/4, fAspect, 0.1, 100.0);
  with OmegaScreen1.Device do
  begin
   SetTransform(D3DTS_PROJECTION, matProj);
   SetTextureStageState(0, D3DTSS_MINFILTER, D3DTEXF_LINEAR);
   SetTextureStageState(0, D3DTSS_MAGFILTER, D3DTEXF_LINEAR);
   SetTextureStageState(0, D3DTSS_COLOROP,   D3DTOP_MODULATE);
   SetTextureStageState(0, D3DTSS_ALPHAOP,   D3DTOP_MODULATE);
   SetTextureStageState(1, D3DTSS_COLOROP,   D3DTOP_DISABLE);
   SetTextureStageState(1, D3DTSS_ALPHAOP,   D3DTOP_DISABLE);
   SetRenderState(D3DRS_SRCBLEND,  D3DBLEND_ONE);
   SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ONE);
   SetRenderState(D3DRS_LIGHTING,  iFALSE);
   SetRenderState(D3DRS_CULLMODE,  D3DCULL_CCW);
   SetRenderState(D3DRS_SHADEMODE, D3DSHADE_FLAT);
  end;
  DXUtil_Timer(TIMER_START);
  // Initialize the particle system
  m_pParticleSystem.RestoreDeviceObjects(OmegaScreen1.Device);
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
 m_pGroundTexture := nil;
 m_pParticleTexture := nil;
 m_pGroundVB := nil;
 m_pGroundIB := nil;
end;

constructor CParticleSystem.Create(dwFlush, dwDiscard: DWord; fRadius: Single);
begin
  m_fRadius        := fRadius;
  m_dwBase         := dwDiscard;
  m_dwFlush        := dwFlush;
  m_dwDiscard      := dwDiscard;
  m_dwParticles    := 0;
  m_dwParticlesLim := 2048;
  m_pParticles     := nil;
  m_pParticlesFree := nil;
  m_pVB            := nil;
end;

destructor CParticleSystem.Destroy;
var
  pSpark: PParticle;
begin
  InvalidateDeviceObjects;
  while Assigned(m_pParticles) do
  begin
    pSpark := m_pParticles;
    m_pParticles := pSpark.m_pNext;
    FreeMem(pSpark);
  end;
  while Assigned(m_pParticlesFree) do
  begin
    pSpark := m_pParticlesFree;
    m_pParticlesFree := pSpark.m_pNext;
    Dispose(pSpark);
  end;
  inherited;
end;

function CParticleSystem.RestoreDeviceObjects(pd3dDevice: IDirect3DDevice8): HResult;
begin
  // Create a vertex buffer for the particle system.  The size of this buffer
  // does not relate to the number of particles that exist.  Rather, the
  // buffer is used as a communication channel with the device.. we fill in
  // a bit, and tell the device to draw.  While the device is drawing, we
  // fill in the next bit using NOOVERWRITE.  We continue doing this until
  // we run out of vertex buffer space, and are forced to DISCARD the buffer
  // and start over at the beginning.
  pd3dDevice.CreateVertexBuffer(m_dwDiscard * SizeOf(TPointVertex),
   D3DUSAGE_DYNAMIC or D3DUSAGE_WRITEONLY or D3DUSAGE_POINTS,
   D3DFVF_POINTVERTEX, D3DPOOL_DEFAULT, m_pVB);
  Result := S_OK;
end;

function CParticleSystem.InvalidateDeviceObjects: HResult;
begin
 m_pVB := nil;
 Result := S_OK;
end;

function CParticleSystem.Update(fSecsPerFrame: Single;
  dwNumParticlesToEmit: DWord; const clrEmitColor, clrFadeColor: TD3DXColor;
  fEmitVel: Single; vPosition: TD3DXVector3): HResult;
{$WRITEABLECONST ON}
const
  fTime: Single = 0.0;
{$WRITEABLECONST OFF}
var
  p_Particle: PParticle;
  ppParticle: ^PParticle;
  fT: Single;
  fGravity: Single;
  v1: TD3DXVector3;
  i: Integer;
  pSpark: PParticle;
  fRand1: Single;
  fRand2: Single;
  dwParticlesEmit: DWord;
begin
  pSpark:= nil;
  p_Particle:= nil;
  fTime:= fTime + fSecsPerFrame;
  ppParticle:= @m_pParticles;
  while Assigned(ppParticle^) do
  begin
    p_Particle := ppParticle^;
    // Calculate new position
    fT := fTime - p_Particle.m_fTime0;
    if (p_Particle.m_bSpark) then
    begin
      fGravity := -5.0;
      p_Particle.m_fFade:= p_Particle.m_fFade - fSecsPerFrame * 2.25;
    end else
    begin
      fGravity := -9.8;
      p_Particle.m_fFade:= p_Particle^.m_fFade - fSecsPerFrame * 0.25;
    end;
    // p_Particle.m_vPos    := p_Particle.m_vVel0 * fT + p_Particle.m_vPos0;
    D3DXVec3Add(p_Particle.m_vPos, D3DXVec3Scale(v1, p_Particle.m_vVel0, fT)^, p_Particle.m_vPos0);
    p_Particle.m_vPos.y  := p_Particle.m_vPos.y + (0.5 * fGravity) * (fT * fT);
    p_Particle.m_vVel.y  := p_Particle.m_vVel0.y + fGravity * fT;

    if (p_Particle.m_fFade < 0.0) then p_Particle.m_fFade := 0.0;

    // Kill old particles
    if (p_Particle.m_vPos.y < m_fRadius) or
       p_Particle.m_bSpark and (p_Particle.m_fFade <= 0.0) then
    begin
      // Emit sparks
      if not p_Particle.m_bSpark then
      begin
        for i:= 0 to 3 do
        begin
          if Assigned(m_pParticlesFree) then
          begin
            pSpark:= m_pParticlesFree;
            m_pParticlesFree:= pSpark.m_pNext;
          end else
          begin
            try
              New(pSpark);
            except
              Result := E_OUTOFMEMORY;
              Exit;
            end;
          end;

          pSpark.m_pNext := p_Particle.m_pNext;
          p_Particle.m_pNext := pSpark;

          pSpark.m_bSpark  := TRUE;
          pSpark.m_vPos0   := p_Particle.m_vPos;
          pSpark.m_vPos0.y := m_fRadius;

          fRand1 := Random * D3DX_PI * 2.00;
          fRand2 := Random * D3DX_PI * 0.25;

          pSpark.m_vVel0.x  := p_Particle.m_vVel.x * 0.25 + Cos(fRand1) * Sin(fRand2);
          pSpark.m_vVel0.z  := p_Particle.m_vVel.z * 0.25 + Sin(fRand1) * Sin(fRand2);
          pSpark.m_vVel0.y  := Cos(fRand2);
          pSpark.m_vVel0.y  := pSpark.m_vVel0.y * Random * 1.5;

          pSpark.m_vPos := pSpark.m_vPos0;
          pSpark.m_vVel := pSpark.m_vVel0;

          D3DXColorLerp(pSpark.m_clrDiffuse, p_Particle.m_clrFade,
                        p_Particle.m_clrDiffuse, p_Particle.m_fFade);
          pSpark.m_clrFade := D3DXColor(0.0, 0.0, 0.0, 1.0);
          pSpark.m_fFade   := 1.0;
          pSpark.m_fTime0  := fTime;
        end;
      end;

      // Kill particle
      ppParticle^ := p_Particle.m_pNext;
      p_Particle.m_pNext := m_pParticlesFree;
      m_pParticlesFree := p_Particle;

      if (not p_Particle.m_bSpark) then Dec(m_dwParticles);
    end else
    begin
      ppParticle := @p_Particle.m_pNext;
    end;
  end;

  // Emit new particles
  dwParticlesEmit:= m_dwParticles + dwNumParticlesToEmit;
  while (m_dwParticles < m_dwParticlesLim) and (m_dwParticles < dwParticlesEmit) do
  begin
    if Assigned(m_pParticlesFree) then
    begin
      p_Particle := m_pParticlesFree;
      m_pParticlesFree := p_Particle.m_pNext;
    end else
    begin
      try
        New(p_Particle);
      except
        Result := E_OUTOFMEMORY;
        Exit;
      end;
    end;

    p_Particle.m_pNext := m_pParticles;
    m_pParticles := p_Particle;
    Inc(m_dwParticles);

    // Emit new particle
    fRand1 := Random * D3DX_PI * 2.0;
    fRand2 := Random * D3DX_PI * 0.25;

    p_Particle.m_bSpark := FALSE;

    // pParticle.m_vPos0 := vPosition + D3DXVECTOR3(0.0, m_fRadius, 0.0);
    D3DXVec3Add(p_Particle.m_vPos0, vPosition, D3DXVector3(0.0, m_fRadius, 0.0));

    p_Particle.m_vVel0.x  := Cos(fRand1) * Sin(fRand2) * 2.5;
    p_Particle.m_vVel0.z  := Sin(fRand1) * Sin(fRand2) * 2.5;
    p_Particle.m_vVel0.y  := Cos(fRand2);
    p_Particle.m_vVel0.y  := p_Particle.m_vVel0.y * Random * fEmitVel;

    p_Particle.m_vPos := p_Particle.m_vPos0;
    p_Particle.m_vVel := p_Particle.m_vVel0;

    p_Particle.m_clrDiffuse := clrEmitColor;
    p_Particle.m_clrFade    := clrFadeColor;
    p_Particle.m_fFade      := 1.0;
    p_Particle.m_fTime0     := fTime;
  end;

  Result := S_OK;
end;

//-----------------------------------------------------------------------------
// Name: Render()
// Desc: Renders the particle system using either pointsprites (if supported)
//       or using 4 vertices per particle
//-----------------------------------------------------------------------------
function CParticleSystem.Render(pd3dDevice: IDirect3DDevice8): HResult;
var
  p_Particle: PParticle;
  pVertices: PPointVertex;
  dwNumParticlesToRender: DWord;
  Flags: DWord;
  vPos, vVel: TD3DXVector3;
  fLengthSq: Single;
  dwSteps: DWord;
  clrDiffuse: TD3DXColor;
  dwDiffuse: DWORD;
  i: Integer;
begin
  // Set the render states for using point sprites
  pd3dDevice.SetRenderState(D3DRS_POINTSPRITEENABLE, iTRUE);
  pd3dDevice.SetRenderState(D3DRS_POINTSCALEENABLE,  iTRUE);
  pd3dDevice.SetRenderState(D3DRS_POINTSIZE,     FtoDW(0.08));
  pd3dDevice.SetRenderState(D3DRS_POINTSIZE_MIN, FtoDW(0.00));
  pd3dDevice.SetRenderState(D3DRS_POINTSCALE_A,  FtoDW(0.00));
  pd3dDevice.SetRenderState(D3DRS_POINTSCALE_B,  FtoDW(0.00));
  pd3dDevice.SetRenderState(D3DRS_POINTSCALE_C,  FtoDW(1.00));

  // Set up the vertex buffer to be rendered
  pd3dDevice.SetStreamSource(0, m_pVB, SizeOf(TPointVertex));
  pd3dDevice.SetVertexShader(D3DFVF_POINTVERTEX);

  p_Particle := m_pParticles;
  dwNumParticlesToRender := 0;
  // Lock the vertex buffer.  We fill the vertex buffer in small
  // chunks, using D3DLOCK_NOOVERWRITE.  When we are done filling
  // each chunk, we call DrawPrim, and lock the next chunk.  When
  // we run out of space in the vertex buffer, we start over at
  // the beginning, using D3DLOCK_DISCARD.
  Inc(m_dwBase, m_dwFlush);
  if (m_dwBase >= m_dwDiscard) then m_dwBase:= 0;
  if m_dwBase = 0 then Flags:= D3DLOCK_DISCARD else Flags:= D3DLOCK_NOOVERWRITE;
  m_pVB.Lock(m_dwBase * SizeOf(TPointVertex), m_dwFlush * SizeOf(TPointVertex),
   PByte(pVertices), Flags);
  // Render each particle
  while Assigned(p_Particle) do
  begin
    vPos:= p_Particle.m_vPos;
    vVel:= p_Particle.m_vVel;
    fLengthSq := D3DXVec3LengthSq(vVel);
    if (fLengthSq < 1.0)        then dwSteps := 2
    else if (fLengthSq <  4.00) then dwSteps := 3
    else if (fLengthSq <  9.00) then dwSteps := 4
    else if (fLengthSq < 12.25) then dwSteps := 5
    else if (fLengthSq < 16.00) then dwSteps := 6
    else if (fLengthSq < 20.25) then dwSteps := 7
    else                             dwSteps := 8;
    D3DXVec3Scale(vVel, vVel, -0.04 / dwSteps);
    D3DXColorLerp(clrDiffuse, p_Particle.m_clrFade, p_Particle.m_clrDiffuse, p_Particle.m_fFade);
    dwDiffuse:= D3DXColorToDWord(clrDiffuse);
    // Render each particle a bunch of times to get a blurring effect
    for i:= 0 to dwSteps - 1 do
    begin
      pVertices.v     := vPos;
      pVertices.color := dwDiffuse;
      Inc(pVertices);
      Inc(dwNumParticlesToRender);
      if (dwNumParticlesToRender = m_dwFlush) then
      begin
        // Done filling this chunk of the vertex buffer.  Lets unlock and
        // draw this portion so we can begin filling the next chunk.
        m_pVB.Unlock;
        Result:= pd3dDevice.DrawPrimitive(D3DPT_POINTLIST, m_dwBase, dwNumParticlesToRender);
        if FAILED(Result) then Exit;
        // Lock the next chunk of the vertex buffer.  If we are at the
        // end of the vertex buffer, DISCARD the vertex buffer and start
        // at the beginning.  Otherwise, specify NOOVERWRITE, so we can
        // continue filling the VB while the previous chunk is drawing.
        Inc(m_dwBase, m_dwFlush);
        if (m_dwBase >= m_dwDiscard) then m_dwBase:= 0;
        if m_dwBase = 0 then Flags:= D3DLOCK_DISCARD else Flags:= D3DLOCK_NOOVERWRITE;
        m_pVB.Lock(m_dwBase * SizeOf(TPointVertex), m_dwFlush * SizeOf(TPointVertex),
         PByte(pVertices), Flags);
        dwNumParticlesToRender := 0;
      end;
      D3DXVec3Add(vPos, vPos, vVel);
    end;
    p_Particle := p_Particle.m_pNext;
  end;
  // Unlock the vertex buffer
  m_pVB.Unlock;
  // Render any remaining particles
  if (dwNumParticlesToRender <> 0) then
  begin
    Result:= pd3dDevice.DrawPrimitive(D3DPT_POINTLIST, m_dwBase, dwNumParticlesToRender);
    if FAILED(Result) then Exit;
  end;
  // Reset render states
  pd3dDevice.SetRenderState(D3DRS_POINTSPRITEENABLE, iFALSE);
  pd3dDevice.SetRenderState(D3DRS_POINTSCALEENABLE,  iFALSE);
  Result := S_OK;
end;

end.

