// Desc: Example code showing how to do text in a Direct3D scene.
// Copyright (c) 1997-2001 Microsoft Corporation. All rights reserved.
unit Unit1;

interface
uses
  Windows, Messages, SysUtils, Classes, Graphics,
  Controls, Forms, CommDlg,
  OmegaInput, OmegaBitmapFont, OmegaTimer,
  Dialogs, OmegaImageList, OmegaScreen,
  DXUtil, D3DUtil, D3DFont, OmegaUtils, D3DX8, {$I UseD3D8.inc};

type
  TForm1 = class(TForm)
    OmegaScreen1: TOmegaScreen;
    OmegaTimer1: TOmegaTimer;
    OmegaBitmapFont1: TOmegaBitmapFont;
    OmegaInput1: TOmegaInput;
    procedure FormCreate(Sender: TObject);
    procedure OmegaTimer1Timer(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormDestroy(Sender: TObject);
   private
    m_pFont: CD3DFont;
    m_pMesh3DText: ID3DXMesh;
    m_strFont: array[0..99] of Char;
    m_dwFontSize: DWORD;
    m_matObj1: TD3DXMatrix;
    m_matObj2: TD3DXMatrix;
    procedure ChangeFont;
    procedure ResetData;
    function CreateD3DXTextMesh: HRESULT;
  end;

var Form1: TForm1;

implementation
{$R *.DFM}

procedure TForm1.FormCreate(Sender: TObject);
begin
 self.ClientWidth := 800;
 self.ClientHeight := 600;
 OmegaScreen1.Width := self.ClientWidth;
 OmegaScreen1.Height := self.ClientHeight;
 OmegaScreen1.Init;
 m_strFont    := 'Arial';
 m_dwFontSize := 18;
 m_pFont      := CD3DFont.Create(m_strFont, m_dwFontSize);
 DXUtil_Timer(TIMER_START);
 m_pFont.InitDeviceObjects(OmegaScreen1.Device);
 m_pFont.RestoreDeviceObjects;
 ResetData;
 OmegaTimer1.Enabled := True;
end;

function TForm1.CreateD3DXTextMesh: HRESULT;
var
  hdc_: HDC;
  hFont_: HFONT;
  hFontOld: HFONT;
  lf: TLogFont;
begin
  hdc_ := CreateCompatibleDC(0);
  m_pMesh3DText := nil;
  ZeroMemory(@lf, SizeOf(lf));
  lf.lfHeight := m_dwFontSize;
  lf.lfStrikeOut := 1;
  lstrcpy(lf.lfFaceName, m_strFont);
  hFont_ := CreateFontIndirect(lf);
  hFontOld := SelectObject(hdc_, hFont_);
  Result := D3DXCreateText(OmegaScreen1.Device, hdc_, 'D3DXCreateText', 0.001, 0.4,
                           m_pMesh3DText, nil, nil);
  SelectObject(hdc_, hFontOld);
  if FAILED(Result) then Exit;
  Result:= S_OK;
end;

procedure TForm1.ResetData;
var
  vEyePt: TD3DXVector3;
  vLookatPt: TD3DXVector3;
  vUpVec: TD3DXVector3;
  matWorld, matView, matProj: TD3DXMatrix;
  fAspect: Single;
  light: TD3DLight8;
begin
  // Restore the textures
  with OmegaScreen1.Device do
  begin
   SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
   SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_DIFFUSE);
   SetTextureStageState(0, D3DTSS_COLOROP,   D3DTOP_MODULATE);
   SetTextureStageState(0, D3DTSS_MINFILTER, D3DTEXF_LINEAR);
   SetTextureStageState(0, D3DTSS_MAGFILTER, D3DTEXF_LINEAR);
   SetTextureStageState(0, D3DTSS_MAGFILTER, D3DTEXF_LINEAR);
   SetRenderState(D3DRS_DITHERENABLE, 1);
   SetRenderState(D3DRS_SPECULARENABLE, 1);
   SetRenderState(D3DRS_LIGHTING, iTRUE);
   SetRenderState(D3DRS_AMBIENT, $80808080);
   D3DUtil_InitLight(light, D3DLIGHT_DIRECTIONAL, 10.0, -10.0, 10.0);
   SetLight(0, light);
   LightEnable(0, True);
   // Set the transform matrices
   vEyePt    := D3DXVector3(0.0,-5.0,-10.0);
   vLookatPt := D3DXVector3(0.0, 0.0,  0.0);
   vUpVec    := D3DXVector3(0.0, 1.0,  0.0);
   D3DXMatrixIdentity(matWorld);
   D3DXMatrixLookAtLH(matView, vEyePt, vLookatPt, vUpVec);
   fAspect := ClientWidth / ClientHeight;
   D3DXMatrixPerspectiveFovLH(matProj, D3DX_PI/4, fAspect, 1.0, 100.0);
   SetTransform(D3DTS_WORLD,      matWorld);
   SetTransform(D3DTS_VIEW,       matView);
   SetTransform(D3DTS_PROJECTION, matProj);
   CreateD3DXTextMesh;
  end;
end;

procedure TForm1.OmegaTimer1Timer(Sender: TObject);
var mtrl: TD3DMaterial8;
    m_fTime: single;
begin
 m_fTime := DXUtil_Timer(TIMER_GETAPPTIME);
 if not OmegaScreen1.CanDraw then exit;
 OmegaInput1.Update;
 if oisbutton1 in OmegaInput1.keyboard.statesclicked then
 begin
   m_pFont.InvalidateDeviceObjects;
   m_pFont.DeleteDeviceObjects;
   OmegaScreen1.ToggleFullScreen;
   m_pFont.InitDeviceObjects(OmegaScreen1.Device);
   m_pFont.RestoreDeviceObjects;
   ResetData;
 end;
 OmegaScreen1.ClearScreen(0, 0, 0);
 // Setup five rotation matrices (for rotating text strings)
 D3DXMatrixRotationAxis(m_matObj1, D3DXVector3(1.0,2.0,0.0), m_fTime / 2.0);
 D3DXMatrixRotationAxis(m_matObj2, D3DXVector3(2.0,1.0,0.0), m_fTime / 2.0);
 // Add some translational values to the matrices
 m_matObj1._41 := 1.0;   m_matObj1._42 := 6.0;   m_matObj1._43 := 20.0;
 m_matObj2._41 := -4.0;  m_matObj2._42 := -1.0;  m_matObj2._43 := 0.0;

 OmegaScreen1.BeginRender;
 OmegaScreen1.Device.SetTexture(0, nil);

 // Draw CD3DFont in 2D (red)
 m_pFont.DrawText(60, 100, D3DCOLOR_ARGB(255,255,0,0), 'CD3DFont::DrawText');
 // Draw CD3DFont in 3D (green)
 D3DUtil_InitMaterial(mtrl, 0.0, 1.0, 0.0);
 OmegaScreen1.Device.SetMaterial(mtrl);
 OmegaScreen1.Device.SetTransform(D3DTS_WORLD, m_matObj1);
 m_pFont.Render3DText('CD3DFont.Render3DText', D3DFONT_CENTERED or D3DFONT_TWOSIDED or D3DFONT_FILTERED );
 // Draw D3DXFont mesh in 3D (blue)
 if (m_pMesh3DText <> nil) then
 begin
  D3DUtil_InitMaterial(mtrl, 0.0, 0.0, 1.0);
  OmegaScreen1.Device.SetMaterial(mtrl);
  OmegaScreen1.Device.SetTransform(D3DTS_WORLD, m_matObj2);
  m_pMesh3DText.DrawSubset(0);
 end;
 OmegaBitmapFont1.Print(44, 510, 'Press "space" to toggle fullscreen');
 OmegaBitmapFont1.Print(44, 530, 'Press "F" to change font');
 OmegaScreen1.EndRender;
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
 OmegaTimer1.Enabled := False;
end;

procedure TForm1.FormKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
 if key = 27 then close;
 if key = ord('F') then ChangeFont;
end;

procedure TForm1.ChangeFont;
var
  hdc_: HDC;
  lHeight: Longint;
  lf: TLogFont;
  cf: TChooseFont;
begin
  hdc_ := GetDC(handle);
  lHeight := -MulDiv(m_dwFontSize, GetDeviceCaps(hdc_, LOGPIXELSY), 72);
  ReleaseDC(handle, hdc_);
  // hdc_ := 0;
  lstrcpy(lf.lfFaceName, m_strFont);
  lf.lfHeight := lHeight;
  ZeroMemory(@cf, SizeOf(cf));
  cf.lStructSize := SizeOf(cf);
  cf.hwndOwner   := handle;
  cf.lpLogFont   := @lf;
  cf.Flags       := CF_SCREENFONTS or CF_INITTOLOGFONTSTRUCT;
  if ChooseFont(cf) then
  begin
    SAFE_DELETE(m_pFont);
    lstrcpy(m_strFont, lf.lfFaceName);
    m_dwFontSize := Trunc(cf.iPointSize / 10);
    m_pFont := CD3DFont.Create(m_strFont, m_dwFontSize);
    m_pFont.InitDeviceObjects(OmegaScreen1.Device);
    m_pFont.RestoreDeviceObjects;
    CreateD3DXTextMesh;
  end;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
 m_pFont.InvalidateDeviceObjects;
 m_pFont.DeleteDeviceObjects;
 m_pFont := nil;
 m_pMesh3DText := nil;
end;

end.





