///////////////////////////////////////////////
//Catch the Omega v1.00
// coding by Eric Behme
// Grafix by Oliver Mller, Eric Behme
// Homepage: http://www.ericbehme.de
//
// This is a small freeware game made for
// Omega components (beta 07), feel free to do
// with it what you like to, except
// using the Magic Storm Logo
//
// Omega Homepage http://www.delphisanctuary.com
//
// Bug reports, comments, question or suggestions
// please into the forums of the above homepages
//
// Have fun with it!
//
// In a later version perhaps Music, better Sounds
// and maybe an Online-Highscore and switching fullscreen
///////////////////////////////////////////////

unit CatchtheUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, OmegaInput, OmegaGUI, OmegaTimer, OmegaSound3D, OmegaFont,
  OmegaImageList, OmegaSprite, OmegaScreen, pngimage, Menus, DirectInput,
  OmegaBitmapFont;

type
  TLevel=array[0..8,0..8] of byte;
  TScene=(Gmenu,Glevel,GDone,Gover,GHints);
  TDirection= (goingup,goingdown,goingleft,goingright,goingnowhere);

  //My Player (Pacman)
  TPlayer = class(TSprite)
              Dir:      TDirection;
              score:    integer;
              procedure draw; override;
              procedure oncollision(const sprite:tsprite; const colX, colY: integer); override;
            end;

  //Monsters and red Omegas
  TMonster= class(TSprite)
              Dir:      TDirection;
              typ:      byte;
              procedure draw; override;
              procedure Move(const movecount: single); override;
              procedure oncollision(const sprite:tsprite; const colX, colY: integer); override;
              constructor Create(const AParent: TOmegaSprite); override;
            end;
  
  TOmegaForm = class(TForm)
    OmegaScreen1: TOmegaScreen;
    OmegaSprite1: TOmegaSprite;
    imagelistmenu: TOmegaImageList;
    OmegaSound3D1: TOmegaSound3D;
    OmegaTimer1: TOmegaTimer;
    OmegaGUI1: TOmegaGUI;
    OmegaInput1: TOmegaInput;
    imagelistwall: TOmegaImageList;
    imagelistsprites: TOmegaImageList;
    ImageListInterface: TOmegaImageList;
    OmegaFont1: TOmegaBitmapFont;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure OmegaTimer1Timer(Sender: TObject);
  private
    { Private declarations }
    procedure create_gui;
    procedure ButtonStartClick(sender:tobject);
    procedure ButtonQuitClick(sender:tobject);
    procedure ButtonHintsClick(sender:tobject);
    procedure do_level;
    procedure do_menu;
    procedure start_level;
    procedure start_game;
    procedure draw_level;
    procedure move_player;
    procedure create_player;
    procedure do_done;
    procedure draw_interface;
    procedure draw_sprites;
    procedure free_sprites;
    procedure do_over;
    procedure do_hints;
  public
    { Public declarations }
  end;

var
  OmegaForm:          TOmegaForm;
  OmegaGuilist:       TOmegaGuiList;
  Scene:              TScene;
  Level:              TLevel;
  Player:             TPlayer;
  Omegabutton:        array [0..2] of TOmegaButton;
  sprogdir:           String; //Directory of the program
  itimer,ileveldone,
  iomegacount,
  iresttimer,
  imonstercount,
  ieattime:           integer;
  boolomegared:       boolean;
  imtyp:              byte;

implementation

{$R *.dfm}

procedure TOmegaForm.ButtonHintsClick(sender: tobject);
begin
  iresttimer:=600;
  scene:=ghints;
end;

procedure TOmegaForm.ButtonQuitClick(sender: tobject);
begin
  close;
end;

procedure TOmegaForm.ButtonStartClick(sender: tobject);
begin
  //Start a new game
  start_game;
  start_level;
  scene:=glevel;
end;

procedure TOmegaForm.create_gui;
var i: integer;
begin
  //Create the OmegaGUI for Main Menu
  OmegaGuiList:=TOmegaGUIList.Create(OmegaGUI1);
  For i:=0 to 2 do
  begin
    //Create 3 Buttons
    OmegaButton[i]:=TOmegaButton.Create(OmegaGuiList);
    OmegaButton[i].Width:=240;
    OmegaButton[i].Height:=80;
    OmegaButton[i].X:=280;
    OmegaButton[i].Y:=i*100+100;
    OmegaButton[i].group:=0;
    OmegaButton[i].Image[0]:=ImageListMenu.ImageList.Items[0];  //default image
    OmegaButton[i].ImageIndex[0]:=0;
    OmegaButton[i].Image[1]:=ImageListMenu.ImageList.Items[1];  //when mouse is over
    OmegaButton[i].ImageIndex[1]:=0;
  end;
  //The Button's onclick method, called by Mouseclick or return
  OmegaButton[0].OnClick:=ButtonStartClick;
  OmegaButton[1].OnClick:=ButtonHintsClick;
  OmegaButton[2].OnClick:=ButtonQuitClick;
  //Set a caption
  OmegaButton[0].Caption:='Start';
  OmegaButton[1].Caption:='Hints';
  OmegaButton[2].Caption:='Quit';
  //  omegagui1.init;
end;

procedure TOmegaForm.create_player;
begin
 //create our player as a sprite
 Player:=TPlayer.Create(omegaform.omegaSprite1);
 with Player do begin
    dir:=goingright;
    Name := 'player'; // set the sprite's name
    dopixelcheck:=true;
    DoCollision := True; // if DoCollision is true, the sprite can be collided with
    Red := 255; // rgb value for Red coloring
    Green := 255; // rgb value for Green coloring    255,255,255 is white
    Blue := 255;  // rgb value for Blue coloring
    Alpha :=  255; // alpha setting, the lower the number the more transparent your sprite is
    ScaleX := 0; // if you set this to 2, your sprite will be twice as wide as your image
    ScaleY := 0; // if you set this to 2, your sprite will be twice as tall as your image
    DoAdditiveBlend := False;  //No Blending
    image:=imagelistsprites.ImageList.Items[0]; //pacman image
    score:=0;
  end;
end;

procedure TOmegaForm.do_done;
begin
  //Level is finished
  dec(iresttimer);
  omegaScreen1.BeginRender;
  omegaScreen1.ClearScreen(0,0,0,255);
  draw_level;
  draw_sprites;
  omegasprite1.Move(0);
  omegasprite1.dead;
  draw_interface;
  omegaFont1.Print(550,10,'FPS: '+floattostrf(omegatimer1.fps,fffixed,1,1));
  omegaFont1.Print(100,300,'Level done');
  omegascreen1.EndRender;
  if iresttimer=0 then begin
    inc(ileveldone);
    start_level;
    scene:=glevel;
  end;
end;

procedure TOmegaForm.do_hints;
begin
  //Show the hints
  dec(iresttimer);
  omegainput1.Update;
  if oisbutton1 in omegainput1.Keyboard.States then scene:=gmenu;
  omegaScreen1.BeginRender;
  omegaScreen1.ClearScreen(0,0,0,255);
  omegaFont1.Print(550,10,'FPS: '+floattostrf(omegatimer1.fps,fffixed,1,1));
  omegaFont1.Print(20,100,'Move Pacman with the Cursor keys.');
  omegaFont1.Print(20,120,'DoubleClick on the Input Component');
  omegaFont1.Print(20,140,'on the Form to change the keys.');
  omegaFont1.Print(20,160,'Press F1 to make a screnshot.');
  omegaFont1.Print(20,180,'Collect all blue Omegas to finish the level.');
  omegaFont1.Print(20,200,'Catch the red Omega and eat Ghosts while');
  omegaFont1.Print(20,220,'they are alphablended.');
  omegaFont1.Print(20,260,'If you hit a Ghost, the game is over.');
  omegaFont1.Print(20,280,'Press ESC to go to the main menu.');
  omegaFont1.Print(20,320,'In tne menu you can use the mouse,');
  omegaFont1.Print(20,340,'or cursor keys and enter.');
  omegaFont1.Print(20,420,'Coding by Eric Behme.');
  omegaFont1.Print(20,440,'Grafix by Oliver Mueller, Eric Behme.');
  omegaFont1.Print(20,460,'This is a free sample for the Omega Components for Delphi.');
  omegaFont1.Print(20,500,'All comments, suggestions or questions go to the forum of');
  omegaFont1.Print(20,540,'Omega Homepage http://www.delphisanctuary.com');
  omegaFont1.Print(20,560,'Eric''s Domain http://www.ericbehme.de');
  omegascreen1.EndRender;
  if iresttimer=0 then begin
    scene:=gmenu;
  end;
end;

procedure TOmegaForm.do_level;
begin
  //Level in progress
  if ieattime>0 then begin
    dec(ieattime);
    if ieattime=0 then boolomegared:=false;
  end;  
  omegainput1.Update; //get the current keys
  if oisbutton2 in omegainput1.Keyboard.States //making screenshot via F1
  then omegascreen1.ScreenShot('CatchtheOmega.bmp');
  omegaScreen1.BeginRender;
  omegaScreen1.ClearScreen(0,0,0,255); //clear the screen
  draw_level;     //draw my level
  draw_sprites;   //draw the sprites, check collision and erase sprites
  omegasprite1.Move(0); //move sprites
  move_player;    //move my pacman via cursor keys
  draw_interface; //draw the interface
  //create a monster
  if (itimer mod 100=0) and (imonstercount<=ileveldone) then begin
    imtyp:=0;
    tmonster.Create(omegasprite1);
    inc(imonstercount);
  end;
  //create a red omega
  if (itimer mod 100=0) and (random(5)=0) and (boolomegared=false) then begin
    imtyp:=1;
    tmonster.Create(omegasprite1);
    boolomegared:=true;
  end;
  //some text
  omegaFont1.Print(550,10,'FPS: '+floattostrf(omegatimer1.fps,fffixed,1,1));
  omegaFont1.Print(550,50,'Score: '+inttostr(player.score));
  if ieattime>0 then omegaFont1.Print(550,80,'Eattime: '+inttostr(ieattime));
  omegascreen1.EndRender;
end;

procedure TOmegaForm.do_menu;
var i:integer;
begin
  omegainput1.Update;
  omegainput1.Keyboard.KeyAssigns[oisbutton12] := DIK_F11;
  if oisbutton12 in omegainput1.Keyboard.StatesClicked then
   OmegaScreen1.ToggleFullScreen;
  if not OmegaScreen1.CanDraw then exit;

  //check down key for the buttons
  if oisdown in omegainput1.Keyboard.StatesClicked then
  begin
   if omegagui1.FocusID=omegabutton[0].id then i:=1 else
   if omegagui1.FocusID=omegabutton[1].id then i:=2 else i:=0;
   setcursorpos(omegabutton[i].X+omegaform.Left,omegabutton[i].y+omegaform.Top);
   omegagui1.FocusID:=omegabutton[i].id;
  end;
  //check up key for the buttons
  if oisup in omegainput1.Keyboard.StatesClicked then begin
   if omegagui1.FocusID=omegabutton[0].id then i:=2 else
   if omegagui1.FocusID=omegabutton[1].id then i:=0 else i:=1;
   setcursorpos(omegabutton[i].X+omegaform.Left,omegabutton[i].y+omegaform.Top);
   omegagui1.FocusID:=omegabutton[i].id;
  end;
  
  omegaScreen1.BeginRender;
  omegaScreen1.ClearScreen(0,0,0,255);
  omegaFont1.Print(550,10,'FPS: '+floattostrf(omegatimer1.fps,fffixed,1,1));
  omegaFont1.Print(250,500,'Press F11 to toggle fullscreen');
  omegaguilist.Draw;
  omegascreen1.EndRender;
end;

procedure TOmegaForm.do_over;
begin
  dec(iresttimer);
  omegaScreen1.BeginRender;
  omegaScreen1.ClearScreen(0,0,0,255);
  draw_level;
  draw_sprites;
  omegasprite1.Move(0);
  omegasprite1.dead;
  draw_interface;
  omegaFont1.Print(550,10,'FPS: '+floattostrf(omegatimer1.fps,fffixed,1,1));
  omegaFont1.Print(200,300,'Game over');
  omegaFont1.Print(200,350,'Your score: '+inttostr(player.score));
  omegascreen1.EndRender;
  if iresttimer=0 then begin
    omegaguilist.Enabled:=true;
    free_sprites;
    scene:=gmenu;
  end;
end;

procedure TOmegaForm.draw_interface;
begin
  imagelistinterface.ImageList.Items[0].Draw(0,540,0);
  imagelistinterface.ImageList.Items[1].Draw(540,0,0);
  imagelistinterface.ImageList.Items[2].Draw(510,400,0);
end;

procedure TOmegaForm.draw_level;
var i,j: integer;
begin
  imagelistwall.begindraw;
  //draw back
  for i:=0 to 8 do
  for j:=0 to 8 do begin
    imagelistwall.ImageList.Items[2].Draw(i*60,j*60,0);
  end;
  //draw shadows
  for i:=0 to 8 do
  for j:=0 to 8 do begin
    if level[i,j]=1 then imagelistwall.ImageList.Items[0].Draw(i*60-6,j*60-6,0,0,0,0,0,0,0,0,110,false,0);
    if level[i,j]=2 then imagelistwall.ImageList.Items[1].Draw(i*60-6,j*60-6,0,0,0,0,0,0,0,0,110,false,0);
  end;
  //draw walls and omegas
  for i:=0 to 8 do
  for j:=0 to 8 do begin
    if level[i,j]=1 then imagelistwall.ImageList.Items[0].Draw(i*60,j*60,0);
    if level[i,j]=2 then imagelistwall.ImageList.Items[1].Draw(i*60,j*60,0);
  end;
  imagelistwall.enddraw;
end;

procedure TOmegaForm.draw_sprites;
begin
  imagelistsprites.BeginDraw;
  omegaSprite1.Draw;
  imagelistsprites.EndDraw;
  omegaSprite1.Collision;
  omegasprite1.dead;
end;

procedure TOmegaForm.FormCreate(Sender: TObject);
begin
  sprogdir:=extractfilepath(application.exename);
  if OmegaScreen1.FullScreen = true then Omegaform.BorderStyle := bsNone;
end;
    
procedure TOmegaForm.FormShow(Sender: TObject);
var i: integer;
begin
//  OmegaScreen1.SethWnd(OmegaForm.Handle);
  OmegaScreen1.Width:=self.width;
  OmegaScreen1.Height:=self.height;
  OmegaScreen1.Init; // initialize our omega screen - important
  //Load and initialize imagelists
  imagelistmenu.ImageList.LoadFromFile('grafix\omegalistmenu.oil');
  Imagelistmenu.init;
  imagelistinterface.ImageList.LoadFromFile('grafix\omegalistinterface.oil');
  Imagelistinterface.init;
  imagelistwall.ImageList.LoadFromFile('grafix\omegalistwall.oil');
  ImagelistWall.Init;
  imagelistsprites.ImageList.LoadFromFile('grafix\omegalistsprites.oil');
  ImageListsprites.Init;

  //init the sound
  Omegasound3d1.Init;
  //preload the sounds
  for i:=0 to omegasound3d1.SoundList.Count-1 do
  omegasound3d1.SoundList.Items[i].Load;
  //create the gui
  create_gui;
  //setting focus on top button
  omegagui1.FocusID:=omegabutton[0].id;
  //set mousepos on top button
  setcursorpos(omegabutton[0].X+omegaform.Left,omegabutton[0].y+omegaform.Top);
  //setting the starting scene, the main menu
  scene:=gmenu;
  //all done, starting timer

  omegaTimer1.Enabled := True;
end;

procedure TOmegaForm.free_sprites;
var i: integer;
begin
  //clear all sprites
  for i:=0 to omegasprite1.Count-1 do begin
    omegasprite1.Items[i].Dead;
  end;
  omegasprite1.Dead;
end;

procedure TOmegaForm.move_player;

function isfree(x,y:integer):boolean;
begin
 if x<0 then x:=x+9 else if x>8 then x:=x-9;
 if y<0 then y:=y+9 else if y>8 then y:=y-9;
 result:=level[x,y]<>1;
end;

begin
 omegainput1.Update;
  //quitting to main menu via pressing escape
  if oisbutton1 in omegainput1.Keyboard.States then begin
    iresttimer:=150;
    scene:=gover;
  end;  
  //setting a direction via user input
  if (round(player.x) mod 60=0) and (round(player.y) mod 60=0) then begin
    if (oisup in omegainput1.Keyboard.States)
    and isfree(round(player.X) div 60,round(player.y) div 60-1) then begin
      omegasound3d1.SoundList.Items[0].Play;
      player.y:=player.Y-6;
      player.Dir:=goingup;
    end else
    if (oisdown in omegainput1.Keyboard.States)
    and isfree(round(player.X) div 60,round(player.y) div 60+1) then begin
      omegasound3d1.SoundList.Items[0].Play;
      player.y:=player.Y+6;
      player.dir:=goingdown;
    end else
    if (oisleft in omegainput1.Keyboard.States)
    and isfree(round(player.X) div 60-1,round(player.y) div 60) then begin
      omegasound3d1.SoundList.Items[0].Play;
      player.x:=player.x-6;
      player.dir:=goingleft;
    end else
    if (oisright in omegainput1.Keyboard.States)
    and isfree(round(player.X) div 60+1,round(player.y) div 60) then begin
      omegasound3d1.SoundList.Items[0].Play;
      player.x:=player.x+6;
      player.dir:=goingright;
    end;
  end
  else begin
    //step automatically to the next field
    if player.Dir=goingup then player.y:=player.Y-6;
    if player.Dir=goingdown then player.y:=player.Y+6;
    if player.Dir=goingleft then player.x:=player.x-6;
    if player.Dir=goingright then player.x:=player.x+6;
    //eat an omega
    if level[round(player.X) div 60,round(player.y) div 60]=2 then begin
      dec(iomegacount);
      inc(player.score);
      level[round(player.X) div 60,round(player.y) div 60]:=0;
      omegasound3d1.SoundList.Items[1].SoundPositionZ:=3;
      omegasound3d1.SoundList.Items[1].Play;
      //Level complete
      if iomegacount=0 then begin
        iresttimer:=100;
        scene:=gdone;
      end;
    end;  
  end;
  //going out of the screen
  if (player.x<-54) and (player.dir=goingleft) then player.X:=534;
  if (player.x>=540) and (player.dir=goingright) then player.X:=-54;
  if (player.y<-54) and (player.dir=goingup) then player.y:=534;
  if (player.y>=540) and (player.dir=goingdown) then player.y:=-54;
  //setting the image for the player for different directions
  if player.dir=goingup then player.Image:=imagelistsprites.ImageList.Items[2];
  if player.dir=goingdown then player.Image:=imagelistsprites.ImageList.Items[3];
  if player.dir=goingleft then player.Image:=imagelistsprites.ImageList.Items[0];
  if player.dir=goingright then player.Image:=imagelistsprites.ImageList.Items[1];
  //setting the imageindex to be used, i have 8 different pictures in each image
  player.ImageIndex:=player.ImageIndex+1;
  if player.ImageIndex>7 then player.imageindex:=0;
end;

procedure TOmegaForm.OmegaTimer1Timer(Sender: TObject);
begin
  if not OmegaScreen1.CanDraw then exit;
  inc(itimer);
  //starting the different scenes
  if scene=glevel then do_level else
  if scene=gmenu then do_menu else
  if scene=gdone then do_done else
  if scene=gover then do_over else
  if scene=ghints then do_hints;
end;

procedure TOmegaForm.start_game;
begin
   //start a new game
  ieattime:=0;
  boolomegared:=false;
  ileveldone:=0;
  //enable the gui
  omegaguilist.Enabled:=false;
  create_player;
end;

procedure TOmegaForm.start_level;
var i, j: integer;
begin
  boolomegared:=false;
  ieattime:=0;
  //take the monsters out of the spritelist
  for i:=0 to omegasprite1.Count-1 do
    if (omegasprite1.Items[i].Name='monster') or (omegasprite1.Items[i].Name='omegared') then omegasprite1.Items[i].Dead;
  iomegacount:=0;
  imonstercount:=0;
  //Clearing level
  for i:=0 to 8 do
  for j:=0 to 8 do level[i,j]:=0;
  //Put some walls into it
  for i:=0 to 8 do
  for j:=0 to 8 do if (i mod 2=1) and (j mod 2=1) then level[i,j]:=1;
  level[3,4]:=1;
  level[5,4]:=1;
  level[4,5]:=1;
  //Some walls for later level
  if ileveldone>0 then level[1,2]:=1;
  if ileveldone>1 then level[7,2]:=1;
  if ileveldone>2 then level[1,7]:=1;
  if ileveldone>3 then level[7,7]:=1;
  //Put Omegas into it
  for i:=0 to 8 do
  for j:=0 to 8 do if level[i,j]=0 then begin
    inc(iomegacount);
    level[i,j]:=2;
  end;
  //setting starting coordinates for player
  player.x:=0;
  player.y:=0;
end;

{ TMonster }

constructor TMonster.Create(const AParent: TOmegaSprite);
begin
  //create a new monster or red omega
  inherited;
  if imtyp=0 then name:='monster' //create a monster
  else name:='omegared';        //create a red omega
  dir:=goingup;
  z:=0;
  typ:=0;
  x:=240;
  y:=240;
  if imtyp=0 then image:=omegaform.imagelistsprites.ImageList.Items[random(4)+4]
  else image:=omegaform.imagelistsprites.ImageList.Items[8];
  imageindex:=0;
  dopixelcheck:=true; //colliding with pixelcheck
  DoCollision := True; // if DoCollision is true, the sprite can be collided with
  Red := 255; // rgb value for Red coloring
  Green := 255; // rgb value for Green coloring    255,255,255 is white
  Blue := 255;  // rgb value for Blue coloring
  Alpha :=  255; // alpha setting, the lower the number the more transparent your sprite is
  ScaleX := 1; // if you set this to 2, your sprite will be twice as wide as your image
  ScaleY := 1; // if you set this to 2, your sprite will be twice as tall as your image}
  Width := 60; // width of sprite - should be same as your tile's width
  Height := 60; // height of sprite - should be same as your tile's height
  Alpha:=255;
  DoAdditiveBlend := false;  //No Blending
end;

procedure TMonster.draw;
begin
  inherited;
  if ieattime=0 then //Shadow
   image.Draw(round(x)-6,round(y)-6,rotation,0,0,scalex,scaley,0,0,0,110,doadditiveblend,imageindex);
  //Sprite, Ghost or Omegared
  if ieattime=0 then alpha:=255
  else if ieattime>80 then alpha:=90-ieattime div 6
  else begin
    if itimer mod 7<3 then alpha:=50 else alpha:=255;
  end;
  image.Draw(round(x),round(y),rotation,0,0,scalex,scaley,red,green,blue,alpha,doadditiveblend,imageindex);
end;

procedure TMonster.Move(const movecount: single);
var idir: byte;
begin
  if (round(x) mod 60=0) and (round(y) mod 60=0) then begin
    //wall ahead
    if (dir=goingup) and (level[round(x) div 60, round(y) div 60-1]=1) then dir:=goingnowhere;
    if (dir=goingdown) and (level[round(x) div 60, round(y) div 60+1]=1) then dir:=goingnowhere;
    if (dir=goingleft) and (level[round(x) div 60-1, round(y) div 60]=1) then dir:=goingnowhere;
    if (dir=goingright) and (level[round(x) div 60+1, round(y) div 60]=1) then dir:=goingnowhere;
    //setting a new direction
    if (dir=goingnowhere) or (random(2)=0) then begin
      idir:=random(4);
      if (idir=0) and (level[round(x) div 60, round(y) div 60-1]<>1) then dir:=goingup;
      if (idir=1) and (level[round(x) div 60, round(y) div 60+1]<>1) then dir:=goingdown;
      if (idir=2) and (level[round(x) div 60-1, round(y) div 60]<>1) then dir:=goingleft;
      if (idir=3) and (level[round(x) div 60+1, round(y) div 60]<>1) then dir:=goingright;
    end;  
  end;
  begin
    if Dir=goingup then y:=y-6;
    if Dir=goingdown then y:=y+6;
    if Dir=goingleft then x:=x-6;
    if Dir=goingright then x:=x+6;
  end;
  //going out of the screen
  if (x<-54) and (dir=goingleft) then X:=534;
  if (x>=540) and (dir=goingright) then X:=-54;
  if (y<-54) and (dir=goingup) then y:=534;
  if (y>=540) and (dir=goingdown) then y:=-54;
end;

procedure TMonster.oncollision(const sprite: tsprite; const colX, colY: integer);
begin
  if sprite.Name='player' then begin
    if name='monster' then begin
      if ieattime=0 then begin
        omegaform.omegasound3d1.SoundList.Items[2].Play;
        if scene=glevel then begin
          //Game over, monster hit
          iresttimer:=50;
          scene:=gover;
        end;  
      end  
      else begin
        //eating the ghost
        omegaform.omegasound3d1.SoundList.Items[3].Play;
        dead;
        dec(imonstercount);
        player.score:=player.score+10;
      end;    
    end else begin
      //omega red hit, eating time!
      omegaform.omegasound3d1.SoundList.Items[3].Play;  
      ieattime:=400;
      dead; //killing omega red
    end;  
  end;
end;

{ TPlayer }

procedure TPlayer.draw;
begin
  inherited;
  //Shadow
  image.Draw(round(x)-6,round(y)-6,rotation,0,0,scalex,scaley,0,0,0,110,doadditiveblend,imageindex);
  //Pacman Sprite
  image.Draw(round(x),round(y),rotation,0,0,scalex,scaley,red,green,blue,alpha,doadditiveblend,imageindex);
end;

procedure TPlayer.oncollision(const sprite: tsprite; const colX, colY: integer);
begin
  inherited;
end;

end.
