unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, OmegaTimer, OmegaSprite, OmegaFont, OmegaImageList, OmegaScreen,
  OmegaParticleEngine;

type
  TFireParticle = class(TParticle)
  public
    procedure Reset; override;
    procedure Update(UpdateSpeed: Single); override;
  end;

  TMouseParticle = class(TParticle)
  public
    procedure Update(MoveCount: Single); override;
    procedure Reset; override;
  end;

type
  TForm1 = class(TForm)
    OmegaScreen: TOmegaScreen;
    OmegaImageList: TOmegaImageList;
    OmegaFont: TOmegaFont;
    OmegaSprite: TOmegaSprite;
    OmegaTimer: TOmegaTimer;
    OmegaParticleEngine: TOmegaParticleEngine;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure OmegaTimerTimer(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormResize(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;
  LastTick: DWORD;
  crs : tPoint;

implementation

{$R *.dfm}

procedure TFireParticle.Reset;
begin
  inherited;
  X := random(form1.ClientWidth);  // new xposition
  Xvel := (Random(1000)-500)/1000;   // new xvel
  Y := Form1.ClientHeight+random(50);// + random(Form1.ClientHeight div 2);
  alpha := random(255);
end;

procedure TFireParticle.Update(UpdateSpeed: Single);
begin
  inherited;
  if Alpha > 0 then Alpha := Alpha - 1
  else self.Reset;  // nice fading effect
end;

procedure TMouseParticle.Update;
begin
  inherited Update(MoveCount);
  Z := Z - 1;                  // this way the oldest particles are draw last
end;

procedure TMouseParticle.Reset;
begin
  inherited Reset;
  Xvel := (Random(2000)-1000)/1000;  // new xvel and yvel
  Yvel := (Random(2000)-1000)/1000;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  Self.Left := 0;
  Self.Top := 0;

  Randomize;
  self.ClientWidth := 800;
  self.ClientHeight := 600;
  OmegaScreen.Width := self.ClientWidth;
  OmegaScreen.Height := self.ClientHeight;
  OmegaScreen.Init;

  OmegaFont.CreateFont('verdana',10,[]);
  OmegaFont.Color := OmegaColor(255,255,255,255);
//  Form1.Cursor := crNone;
end;

procedure TForm1.FormShow(Sender: TObject);
var i : integer;
begin
  with OmegaParticleEngine.EngineList.Items[0] do
  begin
    NumOfParticles := 250;                                 // we want 50 particles to begin with
    UseCustomParticle(TFireParticle);                     // use our own particle
    ChangeAll := false;
                                                          // if we change a value we want all the sprites to get this value
    Sprite.Image := OmegaImageList.ImageList.Items[0];    // select image
    Particle.X := random(form1.ClientWidth);                                    // x position
    Particle.Y := Form1.ClientHeight+random(50);                                    // y position
    Particle.Yvel := -1;                                  // y velocity
    Particle.Decay := 0;                                  // the speed of the lifetime
    Sprite.BlendMode := bmSrcAdd;
    Particle.LifeTime := 1000;                             // how long will the particle exist
    Particle.RandomLocation := false;                      // when a new particle is created it will already have traveled a bit
    GetCursorPos(crs);
  end;

  with OmegaParticleEngine.EngineList.Items[1] do
  begin
    NumOfParticles := 0;
    UseCustomParticle(TMouseParticle);
    Sprite.Image := OmegaImageList.ImageList.Items[1];
    Sprite.ImageIndex := 0;
    Particle.Decay := 1;
    Particle.LifeTime := 80;
    Particle.RandomLocation := False;
    // note: When the Particle.Animate is on and the
    // particle is created it gets the ImageIndex
    // specified in Particle.AnimateIndex. When it
    // dies it will have the ImageIndex of
    // Particle.AnimateIndex + Particle.AnimateCount.
    // Unlike the normal Sprite.DoAnimate it will
    // only cycle through this once in its lifetime
    Particle.Animate := True;                            // we want the particles to be animated
    Particle.AnimateCount := 4;                          // the image have 4 frames that we are interested in
    Particle.AnimateIndex := 0;                          // we start at imageindex 0
    Particle.Loop := False;                              // don't loop this sprite
    Particle.RepeatCount := 0;                           // don't repeat the sprite cycles
    Particle.Z := 500;
  end;
  OmegaParticleEngine.Init;                              // must be called before we can draw anything
  LastTick := GetTickCount;
  OmegaTimer.Enabled := True;

  for i := 0 to OmegaParticleEngine.EngineList.Items[0].NumOfParticles -1 do
    begin
      OmegaParticleEngine.EngineList.Items[0].Items[i].y := random(1024);
      OmegaParticleEngine.EngineList.Items[0].Items[i].LifeTime := random(1000);
    end;
end;

procedure TForm1.OmegaTimerTimer(Sender: TObject);
var
  Tick: DWORD;
begin
  if not OmegaScreen.CanDraw then Exit;

  OmegaScreen.BeginRender;
  OmegaImageList.BeginDraw;
  OmegaScreen.ClearScreen(0,0,0,0);

  Tick := GetTickCount - LastTick;
  LastTick := GetTickCount;

  OmegaParticleEngine.UpdateAll(Tick*0.1);
  OmegaSprite.Draw;
  OmegaSprite.Dead;

  //OmegaFont.Print(10, 400, 'Fire Engine Count = ' + IntToStr(OmegaParticleEngine.EngineList.Items[0].Count));
  //OmegaFont.Print(10, 420, 'Mouse Engine Count = ' + IntToStr(OmegaParticleEngine.EngineList.Items[1].Count));
  //OmegaFont.Print(10, 440, 'FPS = ' + IntToStr(OmegaTimer.FPS));

  OmegaImageList.EndDraw;
  OmegaScreen.EndRender;
end;

procedure TForm1.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = VK_ESCAPE then Close;

  with OmegaParticleEngine.EngineList.Items[0] do
  begin
    if Key = VK_RETURN then NumOfParticles := NumOfParticles + 5;
    if Key = VK_SPACE  then NumOfParticles := NumOfParticles - 5;
  end;
end;

procedure TForm1.FormMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  with OmegaParticleEngine.EngineList.Items[1] do
  begin
    Particle.X := X - 16;            
    Particle.Y := Y - 16;
    NumOfParticles := NumOfParticles + 10;
  end;
end;

procedure TForm1.FormResize(Sender: TObject);
begin
  OmegaScreen.Width := Form1.ClientWidth;
  OmegaScreen.Height := Form1.ClientHeight;
  OmegaScreen.ReInit;
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
 Form1.Cursor := crDefault;
end;

end.
