unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, OmegaTimer, OmegaSprite, OmegaFont, OmegaImageList, OmegaScreen,
  OmegaParticleEngine;

type
  TFireParticle = class(TParticle)
  public
    procedure Reset; override;
    procedure Update(UpdateSpeed: Single); override;
  end;

  TMouseParticle = class(TParticle)
  public
    procedure Update(MoveCount: Single); override;
    procedure Reset; override;
  end;

type
  TForm1 = class(TForm)
    OmegaScreen: TOmegaScreen;
    OmegaImageList: TOmegaImageList;
    OmegaFont: TOmegaFont;
    OmegaSprite: TOmegaSprite;
    OmegaTimer: TOmegaTimer;
    OmegaParticleEngine: TOmegaParticleEngine;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure OmegaTimerTimer(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;
  LastTick: DWORD;

implementation

{$R *.dfm}

procedure TFireParticle.Reset;
begin
  inherited;
  X := X + Random(50)-25;            // new xposition
  Xvel := (Random(1000)-500)/1000;   // new xvel
end;

procedure TFireParticle.Update(UpdateSpeed: Single);
begin
  inherited;
  Alpha := Trunc(LifeTime);  // nice fading effect
end;

procedure TMouseParticle.Update;
begin
  inherited Update(MoveCount);
  Z := Z - 1;                  // this way the oldest particles are draw last
end;

procedure TMouseParticle.Reset;
begin
  inherited Reset;
  Xvel := (Random(2000)-1000)/1000;  // new xvel and yvel
  Yvel := (Random(2000)-1000)/1000;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  Randomize;
  self.ClientWidth := 640;
  self.ClientHeight := 480;
  OmegaScreen.Width := self.ClientWidth;
  OmegaScreen.Height := self.ClientHeight;
  OmegaScreen.Init;

  OmegaFont.CreateFont('verdana',10,[]);
  OmegaFont.Color := OmegaColor(255,255,255,255);
end;

procedure TForm1.FormShow(Sender: TObject);
begin
  with OmegaParticleEngine.EngineList.Items[0] do
  begin
    NumOfParticles := 50;                                 // we want 50 particles to begin with
    UseCustomParticle(TFireParticle);                     // use our own particle
    ChangeAll := True;                                    // if we change a value we want all the sprites to get this value
    Sprite.Image := OmegaImageList.ImageList.Items[0];    // select image
    Particle.X := 320;                                    // x position
    Particle.Y := 400;                                    // y position
    Particle.Yvel := -1;                                  // y velocity
    Particle.Decay := 1;                                  // the speed of the lifetime
    Particle.LifeTime := 160;                             // how long will the particle exist
    Particle.RandomLocation := True;                      // when a new particle is created it will already have traveled a bit
  end;
  with OmegaParticleEngine.EngineList.Items[1] do
  begin
    NumOfParticles := 0;
    UseCustomParticle(TMouseParticle);
    Sprite.Image := OmegaImageList.ImageList.Items[1];
    Sprite.ImageIndex := 0;
    Particle.Decay := 1;
    Particle.LifeTime := 80;
    Particle.RandomLocation := False;
    // note: When the Particle.Animate is on and the
    // particle is created it gets the ImageIndex
    // specified in Particle.AnimateIndex. When it
    // dies it will have the ImageIndex of
    // Particle.AnimateIndex + Particle.AnimateCount.
    // Unlike the normal Sprite.DoAnimate it will
    // only cycle through this once in its lifetime
    Particle.Animate := True;                            // we want the particles to be animated
    Particle.AnimateCount := 4;                          // the image have 4 frames that we are interested in
    Particle.AnimateIndex := 0;                          // we start at imageindex 0
    Particle.Loop := False;                              // don't loop this sprite
    Particle.RepeatCount := 0;                           // don't repeat the sprite cycles
    Particle.Z := 500;
  end;
  OmegaParticleEngine.Init;                              // must be called before we can draw anything
  LastTick := GetTickCount;
  OmegaTimer.Enabled := True;
end;

procedure TForm1.OmegaTimerTimer(Sender: TObject);
var
  Tick: DWORD;
begin
  if not OmegaScreen.CanDraw then Exit;

  OmegaScreen.BeginRender;
  OmegaImageList.BeginDraw;
  OmegaScreen.ClearScreen(0,0,0,0);

  Tick := GetTickCount - LastTick;
  LastTick := GetTickCount;

  OmegaParticleEngine.UpdateAll(Tick*0.1);

  OmegaSprite.Move(0); 
  OmegaSprite.Draw;
  OmegaSprite.Dead;

  OmegaFont.Print(10, 400, 'Fire Engine Count = ' + IntToStr(OmegaParticleEngine.EngineList.Items[0].Count));
  OmegaFont.Print(10, 420, 'Mouse Engine Count = ' + IntToStr(OmegaParticleEngine.EngineList.Items[1].Count));
  OmegaFont.Print(10, 440, 'FPS = ' + IntToStr(OmegaTimer.FPS));

  OmegaImageList.EndDraw;
  OmegaScreen.EndRender;
end;

procedure TForm1.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  // "L"
  if Key = 76 then OmegaParticleEngine.EngineList.Items[0].LoadFromFile(ExtractFilePath(Application.ExeName)+'engine.ope');
  // "S"
  if Key = 83 then OmegaParticleEngine.EngineList.Items[0].SaveToFile(ExtractFilePath(Application.ExeName)+'engine.ope');

  if Key = VK_ESCAPE then Close;

  with OmegaParticleEngine.EngineList.Items[0] do
  begin
    if Key = VK_RETURN then NumOfParticles := NumOfParticles + 5;
    if Key = VK_SPACE  then NumOfParticles := NumOfParticles - 5;
  end;
end;

procedure TForm1.FormMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  with OmegaParticleEngine.EngineList.Items[1] do
  begin
    Particle.X := X - 16;            
    Particle.Y := Y - 16;
    NumOfParticles := NumOfParticles + 10;
  end;
end;

end.
