{ Unit: OmegaFont.pas

    Version Details:

    Beta 10:
      Modification by Tobias Fleischer (Tobybear) (11/06/2003)

      Added preservation of blending mode in BeginFont/EndFont

    Beta 09:
      Modification by Tobias Fleischer (Tobybear) (11/03/2003)

      Added color defaults

    Beta 08:
      Modification by Tobias Fleischer (Tobybear) (10/09/2003)

      Lots of code moved and modified to make it working even
      if the device changes (as in fullscreen toggling)

    Beta 07:
      Modification by Tobias Fleischer (Tobybear) (08/31/2003)

      I moved all of the OmegaColor stuff to OmegaUtils, as certainly
      other units might benefit from this as well. I momentarily
      disabled the function with the AlphaPercent parameter though,
      since it is conflicting with the r, g, b, a byte value function
      (and do we really need to set alpha either in 0..255 or in 0..100?)
      I also made OmegaColor a packed record that is now type-compatible
      with DirectX native color format TD3DCOLOR

    Beta 06:
      Modification by David Senabre (Patroclus) (05/23/2003)

      Add Print method allowing to specify RGBA values.
      Add PrintInvert methods, that automaticaly inverts colour attibutes

    Beta 04:
      Modifications by Mickey Petersen (dough) (10/24/2002)

      Instead of forcing users to use the Red/Green/Blue/Alpha properties,
      they now have the option of using the new Color property,
      which will, then, split it up into the Red, Green, Blue and Alpha property.
      The Color property takes one parameter, TOmegaColor, and can be
      assembled using one of the three OmegaColor functions:

        - Generic function for creating a TOmegaColor
        OmegaColor(Red, Green, Blue: Byte; Alpha: Byte = 255): TOmegaColor;
        - Same as above, but requires a percent point(0-100) instead of a
          whole Byte(0-255).
        OmegaColor(Red, Green, Blue: Byte; AlphaPercent: TAlphaPercent): TOmegaColor;
        - This function takes a TColor and an Alpha value and generates
          a TOmegaColor based on it.
        OmegaColor(Color: TColor; Alpha: Byte = 255): TOmegaColor;

      Also, i've changed the CreateFont method so it takes a TFontStyles
      instead of a string of Boolean parameters. This'll will keep compatibility
      with the TFont object's Style property.
                                                                              }
unit OmegaFont;

interface
uses Dialogs, D3DX8, OmegaScreen, Classes, Graphics,
  Windows, SysUtils, OmegaUtils, {$I UseD3D8.inc};

type
  TOmegaFont = class(TComponent)
  private
    bm: integer;
    FFont: ID3DXFont;
    sname: string;
    ssize: integer;
    sstyle: TFontStyles;
    FOmegaScreen: TOmegaScreen;
    FRed: Byte;
    FBlue: Byte;
    FGreen: Byte;
    FAlpha: Byte;
    FColor: TOmegaColor;
    FCanvas: TCanvas;
    FTheFont: TFont;
    procedure SetOmegaScreen(value: TOmegaScreen);
    procedure SetColor(const Value: TOmegaColor);
  public
    destructor Destroy; override;
    function CreateFont(sFontName: string; iSize: integer; Style: TFontStyles): Boolean; overload;
    function CreateFont(const Font: TFont): Boolean; overload;
    function TextHeight(const Text: string): Integer;
    function TextWidth(const Text: string): integer;
    procedure BeginFont;
    procedure EndFont;
    procedure Init;
    procedure UnInit;
    procedure PrintInvert(XPos, YPos: Integer; sString: string);
    procedure Print(XPos, YPos: Integer; sString: string; R, G, B, A: Byte); overload;
    procedure Print(XPos, YPos: Integer; sString: string); overload;
    procedure Print(Pos: TPoint; sString: string); overload;
    property Color: TOmegaColor read FColor write SetColor;
    property Red: Byte read FRed write FRed;
    property Blue: Byte read FBlue write FBlue;
    property Green: Byte read FGreen write FGreen;
    property Alpha: Byte read FAlpha write FAlpha;
  published
    property OmegaScreen: TOmegaScreen read FOmegaScreen write setOmegaScreen;
  end;

{ Quick functions for creating a TOmegaColor record }
{
function OmegaColor(Red, Green, Blue: Byte; Alpha: Byte = 255): TOmegaColor; overload;
// function OmegaColor(Red, Green, Blue: Byte; AlphaPercent: TAlphaPercent): TOmegaColor; overload;
function OmegaColor(Color: TColor; Alpha: Byte = 255): TOmegaColor; overload;
}
implementation
{
function OmegaColor(Color: TColor; Alpha: Byte = 255): TOmegaColor; overload;
var
  RGB: Longint;
begin
  // Convert the color to RGB so we can process it.
  RGB := ColorToRGB(Color);
  Result.Alpha := Alpha;

  // Extract the Red, Green and blue part from the RGB value.
  Result.Red := Byte(RGB);
  Result.Green := RGB shr 8;
  Result.Blue := RGB shr 16;
end;

function OmegaColor(Red, Green, Blue: Byte; Alpha: Byte = 255): TOmegaColor; overload;
begin
  Result.Alpha := Alpha;
  Result.Red := Red;
  Result.Green := Green;
  Result.Blue := Blue;
end;

function OmegaColor(Red, Green, Blue: Byte; AlphaPercent: TAlphaPercent): TOmegaColor; overload;
begin
  Result.Alpha := 255 div 100 * AlphaPercent;
  Result.Red := Red;
  Result.Green := Green;
  Result.Blue := Blue;
end;
}

function TOmegaFont.CreateFont(sFontName: string; iSize: integer; Style: TFontStyles): Boolean;
begin
  sname := sFontName;
  ssize := isize;
  sstyle := style;
  Color := OmegaColor($FFFFFFFF);
  result := true;
  UnInit;
  Init;
end;

function TOmegaFont.CreateFont(const Font: TFont): Boolean;
begin
  if not Assigned(Font) then
   raise Exception.Create('CreateFont() had unassigned Font param.');
  result := CreateFont(Font.Name, Font.Size, Font.Style);
end;

procedure TOmegaFont.BeginFont;
begin
  if not Assigned(FFont) then Exit;
  if not Assigned(FOmegaScreen.Device) then Exit;
  bm := FOmegaScreen.BlendMode;
  FFont._Begin;
end;

procedure TOmegaFont.EndFont;
begin
  if not Assigned(FFont) then Exit;
  if not Assigned(FOmegaScreen.Device) then Exit;
  FFont._End;
  FOmegaScreen.BlendMode := bm;
end;

procedure TOmegaFont.SetOmegaScreen(Value: TOmegaScreen);
begin
  FOmegaScreen := Value;
end;

procedure TOmegaFont.SetColor(const Value: TOmegaColor);
begin
  FColor := Value;
  { Update the Red, Green, Blue and Alpha properties
    to reflect the new values entered }
  Red := FColor.Red;
  Blue := FColor.Blue;
  Green := FColor.Green;
  Alpha := FColor.Alpha;
end;

procedure TOmegaFont.Print(XPos, YPos: Integer; sString: string; R, G, B, A: Byte);
var
  Rect: TRect;
begin
  if not Assigned(FFont) then Exit;
  if not Assigned(FOmegaScreen.Device) then Exit;

  Rect.left := XPos;
  Rect.top := YPos;
  Rect.bottom := 0;
  Rect.right := 0;

  FFont.DrawTextA(PChar(sString), -1, Rect, DT_NOCLIP, D3dColor_RGBA(R, G, B, A));
end;

procedure TOmegaFont.Print(XPos, YPos: Integer; sString: string);
begin
  Print(XPos, YPos, sString, FRed, FGreen, FBlue, FAlpha);
end;

procedure TOmegaFont.Print(Pos: TPoint; sString: string);
begin
  Print(Pos.X, Pos.Y, sString);
end;

procedure TOmegaFont.PrintInvert(XPos, YPos: Integer; sString: string);
begin
  Print(XPos, YPos, sString, (255 - FRed), (255 - FGreen), (255 - FBlue), FAlpha);
end;

destructor TOmegaFont.Destroy;
begin
  UnInit;
  inherited destroy;
end;

function TOmegaFont.TextHeight(const Text: string): Integer;
begin
  FCanvas.Font := FTheFont;
  Result := FCanvas.TextHeight(Text);
end;

function TOmegaFont.TextWidth(const Text: string): Integer;
begin
  FCanvas.Font := FTheFont;
  Result := FCanvas.TextWidth(Text);
end;

procedure TOmegaFont.Init;
var
  oFont: TFont;
begin
  { Create the font and assign the properties. }
  oFont := TFont.Create;
  try
    oFont.Name := sname;
    oFont.Size := ssize;
    oFont.Style := sstyle;
    D3DXCreateFont(FOmegaScreen.Device, oFont.Handle, FFont);
  finally { wrap up }
    oFont.Free;
  end; { try/finally }
  
  FCanvas := TCanvas.Create;
  FCanvas.Handle := GetWindowDC(OmegaScreen.ThehWnd);
  FTheFont := TFont.Create;
  FTheFont.Name := sname;
  FTheFont.Size := ssize;
  FTheFont.Style := sstyle;
end;

procedure TOmegaFont.UnInit;
begin
 if assigned(FFont) then FFont := nil;
 if assigned(FCanvas) then
 begin
  FCanvas.Free;
  FCanvas := nil;
 end;
 if assigned(FTheFont) then
 begin
  FTheFont.Free;
  FTheFont := nil;
 end;
end;

end.
