{
//****************************************************************
// OmegaImageList unit
// author: Tobias Fleischer
// contact: tobybear@web.de
// comments are welcome!
//****************************************************************

version history:
06/06/04: DrawRect/DrawInRect now also support blendmode, mirroring 
          and imageindex
03/10/04: adapted whole unit to latest DX8/DX9 headers
01/26/04: renamed DrawRect function to DrawInRect functions,
          added new function DrawRect that works like the
          old one, so OmegaGUI bars should now work again
01/20/04: fixed crash with unassigned pictures
2003: various updates and fixes

"todo":
- add proper documentation
}
unit OmegaImageList;
interface
uses
  Windows, Messages, SysUtils, Classes, D3DX8,
  Dialogs, OmegaScreen, Forms, Graphics, OmegaRenderer,
  OmegaUtils, pngimage, {$I UseD3D8.inc};

type
  TCryptFunction = function(s: TStream): TStream;

  TImageListCollection = class;
  TOmegaImageList = class;
  TImageListItem = class(TCollectionItem)
  private
    FNumOfRows, FNumOfColumns: integer;
    FName: string;
    FRenderer: TOmegaRenderer;
    FDevice: IDirect3DDevice8;
    FImage: IDirect3DTexture8;
    FPicture: TPicture;
    FHeight: Integer;
    FWidth: Integer;
    FTileHeight: Integer;
    FTileWidth: Integer;
    FRealHeight: Integer;
    FRealWidth: Integer;
    FFilepath: TFileName;
    FTransparentColor: TColor;
    FTransparent: Boolean;
    function GetRealHeight: Integer;
    function GetRealWidth: Integer;
    procedure SetTileHeight(Value: Integer);
    procedure SetTileWidth(Value: integer);
  protected
    function GetDisplayName: string; override;
  public
    property Renderer: TOmegaRenderer read FRenderer write FRenderer;
    constructor Create(Collection: TCollection); override;
    destructor Destroy; override;
    property ImageHeight: Integer read FHeight write FHeight;
    property ImageWidth: Integer read FWidth write FWidth;
    property Image: IDirect3DTexture8 read FImage write FImage;
    property RealHeight: Integer read GetRealHeight;
    property RealWidth: Integer read GetRealWidth;
    property NumOfRows: Integer read FNumOfRows;
    property NumOfColumns: Integer read FNumOfColumns;
    procedure SetTransparentColor(Value: TColor);
    procedure SetPicture(Value: TPicture);
    procedure Init;
    procedure UnInit(deleteimages: boolean = false);

    procedure Draw(X, Y: integer; ImgIndex: integer); overload;
    procedure Draw(X, Y: integer; Rotation: single; CenterX, CenterY: single; ScaleX, ScaleY: single; red, green, blue, alpha: integer; AdditiveBlend: Boolean; ImgIndex: integer; BlendMode: integer = bmNormal); overload;
    procedure Draw(X, Y: integer; Rotation: single; CenterX, CenterY: single; ScaleX, ScaleY: single; red, green, blue, alpha: integer; ImgIndex: integer; BlendMode: integer = bmNormal); overload;
    procedure Draw(X, Y: integer; Rotation: single; CenterX, CenterY: single; ScaleX, ScaleY: single; col: TOmegaColor; ImgIndex: integer; BlendMode: integer = bmNormal); overload;
    procedure Draw(X, Y: integer; Rotation: single; CenterX, CenterY: single; ScaleX, ScaleY: single; col: TOmegaColor; MirrorHorizontally, MirrorVertically: boolean; ImgIndex: integer; BlendMode: integer = bmNormal); overload;
    procedure Draw4Col(X, Y: integer; Rotation: single; CenterX, CenterY: single; ScaleX, ScaleY: single; col1, col2, col3, col4: TOmegaColor; ImgIndex: integer; BlendMode: integer = bmNormal); overload;
    procedure Draw4Col(x1, y1, x2, y2, x3, y3, x4, y4: integer; Rotation: single; CenterX, CenterY: single; ScaleX, ScaleY: single; col1, col2, col3, col4: TOmegaColor; ImgIndex: integer; BlendMode: integer = bmNormal); overload;
    procedure Draw4Col(X, Y: integer; Rotation: single; CenterX, CenterY: single; ScaleX, ScaleY: single; col1, col2, col3, col4: TOmegaColor; MirrorHorizontally, MirrorVertically: boolean; ImgIndex: integer; BlendMode: integer = bmNormal); overload;
    procedure Draw4Col(x1, y1, x2, y2, x3, y3, x4, y4: integer; Rotation: single; CenterX, CenterY: single; ScaleX, ScaleY: single; col1, col2, col3, col4: TOmegaColor; MirrorHorizontally, MirrorVertically: boolean; ImgIndex: integer; BlendMode: integer = bmNormal); overload;

    procedure DrawInRect(Rect: TRect; X, Y: integer;
     Rotation: single; CenterX, CenterY: single;
     ScaleX, ScaleY: single;
     red, green, blue, alpha: integer;
     mirrorh, mirrorv: boolean;
     imageindex: integer;
     blendmode: integer); overload;
    procedure DrawInRect(Rect: TRect; X, Y: integer); overload;

    procedure DrawRect(Rec: TRect; X, Y: integer;
     Rotation: single; RotationCenterX, RotationcenterY: single;
     ScaleX, ScaleY: single;
     red, green, blue, alpha: integer;
     mirrorh, mirrorv: boolean;
     imageindex: integer;
     blendmode: integer); overload;
    procedure DrawRect(Rec: TRect; X, Y: integer); overload;

  published
    property Name: string read FName write FName;
    property Picture: TPicture read FPicture write SetPicture;
    property TileHeight: Integer read FTileHeight write SetTileHeight;
    property TileWidth: Integer read FTileWidth write SetTileWidth;
    property TransparentColor: TColor read FTransparentColor write SetTransparentColor;
    property Transparent: Boolean read FTransparent write FTransparent;
    property Filepath: TFilename read FFilepath write FFilepath;
  end;

  TImageListCollection = class(TOwnedCollection)
  private
    FEncode, FDecode: TCryptFunction;
  protected
    function GetItem(Index: Integer): TImageListItem;
    procedure SetItem(Index: Integer; Value: TImageListItem);
    procedure Update(Item: TCollectionItem); override;
  public
    constructor Create(AOwner: TComponent);
    property DecodeFunction: TCryptFunction read FDecode write FDecode;
    property EncodeFunction: TCryptFunction read FEncode write FEncode;
    procedure SaveToFile(const Filename: string);
    procedure SaveToStream(var Stream: TStream);
    procedure LoadFromFile(const Filename: string);
    procedure LoadFromStream(Stream: TStream);
    function Add: TImageListItem;
    function Insert(Index: Integer): TImageListItem;
    function Find(const SearchName: string): TImageListItem;
    property Items[Index: Integer]: TImageListItem read GetItem write SetItem;
  end;

  TOmegaImageList = class(TComponent)
  private
    FImageList: TImageListCollection;
    FOmegaScreen: TOmegaScreen;
    FRenderer: TOmegaRenderer;
    procedure SetImageList(const Value: TImageListCollection);
  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    TexScaleX, TexScaleY, TexOfsX, TexOfsY: single; // only for debugging!
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetOmegaScreen(value: TOmegaScreen);
    procedure Init;
    procedure UnInit(deleteimages: boolean = false);
    procedure BeginDraw;
    procedure EndDraw;
  published
    property ImageList: TImageListCollection read FImageList write SetImageList;
    property OmegaScreen: TOmegaScreen read FOmegaScreen write SetOmegaScreen;
  end;

implementation

{
function GetD3DXVector2(_x, _y: single): TD3DXVector2;
begin
  Result.x := _x;
  Result.y := _y;
end;
 }
//----------------------------------------------------------------------------//
// ******************  Beginning of ImageListItem Component ******************//
//----------------------------------------------------------------------------//

constructor TImageListItem.Create(Collection: TCollection);
begin
  inherited Create(Collection);
  FPicture := TPicture.Create;
  TileWidth := 1;
  TileHeight := 1;
end;

destructor TImageListItem.Destroy;
begin
  FImage := nil;
  if FDevice <> nil then FDevice := nil;
  if FRenderer <> nil then FRenderer := nil;
  FPicture.Free;
  inherited Destroy;
end;

procedure TImageListItem.Init;
var FD3DSurfaceDesc: TD3DSurface_DESC;
    Stream: TMemoryStream;
    sRealFilePath: string;
begin
  //Load picture from filepath if present
  FFilePath := Trim(FFilePath);
  if Length(FFilepath) > 0 then
  begin
    //Make relative paths absolute
    if (Pos(':', FFilepath) = 0) and (Pos('\\', FFilepath) = 0) then
    begin
      ChDir(ExtractFilePath(Application.ExeName));
      sRealFilepath := ExpandFileName(FFilepath);
    end
    else
      sRealFilepath := FFilepath;
    if FileExists(sRealFilePath) then FPicture.LoadFromFile(sRealFilepath);
  end;

  if (FPicture.Graphic = nil) then exit;

  FDevice := TOmegaImageList(TImageListCollection(Collection).GetOwner).FOmegaScreen.Device;
  FRenderer := TOmegaImageList(TImageListCollection(Collection).GetOwner).FRenderer;
  {
  If (FDevice = nil) then
  begin
   MessageBox(0, pChar('Error: no device assigned!'), 'Error', MB_ICONERROR);
   Application.Terminate;
  end;
  If (FRenderer = nil) then
  begin
   MessageBox(0, pChar('Error: no renderer assigned!'), 'Error', MB_ICONERROR);
   Application.Terminate;
  end;
  }
  Stream := TMemoryStream.Create;
  try
    FPicture.Graphic.SaveToStream(Stream);
    if FTileHeight <= 0 then FTileHeight := FPicture.Graphic.Height;
    if FTileWidth <= 0 then FTileWidth := FPicture.Graphic.Width;
    if not assigned(self.FImage) then
    if Self.Transparent = true then
    begin
      if failed(D3DXCreateTextureFromFileInMemoryEx(
       FDevice, Stream.Memory^, Stream.Size,
        0, 0, 0, 0, D3DFMT_UNKNOWN, D3DPOOL_MANAGED,
        D3DX_FILTER_NONE, D3DX_FILTER_NONE,
        $FF000000 or TColor(OmegaColor(self.TransparentColor,0)),
        //Self.TransparentColor,
        nil, nil, Self.FImage)) then
      begin
        MessageBox(0, pChar('Error creating texture ' + Self.FName), 'Error', MB_ICONERROR);
        Stream.Free;
        Application.Terminate;
      end;
    end else
    begin
      if failed(D3DXCreateTextureFromFileInMemoryEx(
       FDevice, Stream.Memory^, Stream.Size,
        0, 0, 0, 0, D3DFMT_UNKNOWN, D3DPOOL_MANAGED,
        D3DX_FILTER_NONE, D3DX_FILTER_NONE,
        0, nil, nil, Self.FImage)) then
      begin
        MessageBox(0, pChar('Error creating texture ' + Self.FName), 'Error', MB_ICONERROR);
        Stream.Free;
        Application.Terminate;
      end;
    end;
    Self.FImage.GetLevelDesc(0, FD3DSurfaceDesc);
    FRealWidth   := FD3DSurfaceDesc.Width;
    FRealHeight  := FD3DSurfaceDesc.Height;
    if FRealWidth < 2 then FRealWidth := 2;
    if FRealHeight < 2 then FRealHeight := 2;
    FHeight := FPicture.Height;
    if FHeight = 0 then FHeight := 1;
    FWidth := FPicture.Width;
    if FWidth = 0 then FWidth := 1;
    FNumOfRows := FHeight div FTileHeight;
    FNumOfColumns := FWidth div FTileWidth;
  finally
   Stream.Free;
   if Length(FFilePath) > 0 then
   begin
    FPicture.Free;
    FPicture := TPicture.Create;
   end;
  end;
end;

procedure TImageListItem.UnInit(deleteimages: boolean = false);
begin
 if deleteimages then FImage := nil;
 FRenderer := nil;
 FDevice := nil;
end;

function TImageListItem.GetDisplayName: string;
begin
 Result := FName;
end;

procedure TImageListItem.SetPicture(value: TPicture);
begin
 FPicture.Assign(Value);
 if value <> nil then
 begin
  FWidth := value.Graphic.Width;
  FHeight := value.Graphic.Height;
  TileWidth := value.Graphic.Width;
  TileHeight := value.Graphic.Height;
  GetRealWidth; // to update FRealWidth and FRealHeight
 end;
end;

procedure TImageListItem.SetTransparentColor(value: TColor);
begin
  FTransparentColor := value;
end;

procedure TImageListItem.SetTileHeight(Value: Integer);
begin
 FTileHeight := value;
 if FTileHeight = 0 then FTileHeight := FHeight;
 FNumOfRows := FHeight div FTileHeight;
end;

procedure TImageListItem.SetTileWidth(Value: integer);
begin
 FTileWidth := value;
 if FTileWidth = 0 then FTileWidth := FWidth;
 FNumOfColumns := FWidth  div FTileWidth;
end;

{
function TImageListItem.GetRect(index, width, height: integer): TRect;
var
  aWidth, aHeight, rowcount, colcount: integer;
  rect: TRect;
begin
  rect.Left := 0;
  rect.Right := 0;
  rect.Top := 0;
  rect.Bottom := 0;

  aWidth := Self.ImageWidth;
  aHeight := Self.ImageHeight;
  ColCount := aWidth div Width;
  RowCount := aHeight div Height;

  if Index > RowCount * ColCount then
    Index := RowCount * ColCount + 1;

  if ColCount > 0 then
  begin
    rect.Left := (index mod colcount) * Width;
    rect.Top := (index div colcount) * Height;
  end;
  rect.Right := Rect.Left + Width;
  rect.Bottom := Rect.Top + Height;

  result := rect;
end;
}

procedure TImageListItem.Draw4Col(x1, y1, x2, y2, x3, y3, x4, y4: integer;
  Rotation, CenterX, CenterY, ScaleX, ScaleY: single; col1, col2, col3,
  col4: TOmegaColor; MirrorHorizontally, MirrorVertically: boolean;
  ImgIndex: integer; BlendMode: integer = bmNormal);

procedure DrawIndex;
var //a1, a2, a3, a4,
    f1, f2, l, r, t, b: double;
begin
{ //only for debugging!
 a1 := TOmegaImageList(TImageListCollection(Collection).GetOwner).TexScaleX;
 a2 := TOmegaImageList(TImageListCollection(Collection).GetOwner).TexScaleY;
 a3 := TOmegaImageList(TImageListCollection(Collection).GetOwner).TexOfsX;
 a4 := TOmegaImageList(TImageListCollection(Collection).GetOwner).TexOfsY;
}
// a1:=0; a2:=0;
//  a3:=0; a4:=0;

 f1 := 1 / (FRealWidth);
 f2 := 1 / (FRealHeight);

 if ImgIndex >= FNumOfRows * FNumOfColumns then
  ImgIndex := FNumOfRows * FNumOfColumns - 1;

 l := (ImgIndex mod FNumOfColumns) * (TileWidth);
 t := (ImgIndex div FNumOfColumns) * (TileHeight);
 r := (l + TileWidth)  * f1;
 b := (t + TileHeight) * f2;

 l := l * f1;
 t := t * f2;

// l:=0;t:=0;r:=1;b:=1;
{ ax1:=l;
 ax2:=t;
 ax3:=r;
 ax4:=b;}
         
 FRenderer.SetTexturePositions(l, t, r, t, r, b, l, b);
 FDevice.SetTexture(0, image);
 FRenderer.SetPositions(x1-0.5, y1-0.5, x2-0.5, y2-0.5, x3-0.5,
  y3-0.5, x4-0.5, y4-0.5);
// FRenderer.SetPositions(x1, y1, x2-1, y2, x3-1, y3-1, x4, y4-1);
// FRenderer.SetPositions(x1, y1, x2, y2, x3, y3, x4, y4);
 CenterX := CenterX * TileWidth + x1;
 CenterY := CenterY * TileHeight + y1;
{ bx1:=FRenderer.x1;
 by1:=FRenderer.y1;
 bx2:=FRenderer.x2;
 by2:=FRenderer.y2;
 bx3:=FRenderer.x3;
 by3:=FRenderer.y3;
 bx4:=FRenderer.x4;
 by4:=FRenderer.y4;}
// FRenderer.Rotate(CenterX, CenterY, Rotation);
 FRenderer.RotateAndScale(CenterX, CenterY, Rotation, ScaleX, ScaleY);
 FRenderer.Draw;
end;

procedure DrawFull;
var r, b: single;
begin
 //only for debugging!
 {
 a1 := TOmegaImageList(TImageListCollection(Collection).GetOwner).TexScaleX;
 a2 := TOmegaImageList(TImageListCollection(Collection).GetOwner).TexScaleY;
 }
// a1:=0; a2:=0;
 r := (FWidth)  / (FRealWidth);
 b := (FHeight) / (FRealHeight);
 FRenderer.SetTexturePositions(0, 0, r, 0, r, b, 0, b);

 FDevice.SetTexture(0, image);
// FRenderer.SetPositions(x1, y1, x2, y2, x3, y3, x4, y4);
 FRenderer.SetPositions(x1-0.5, y1-0.5, x2-0.5, y2-0.5, x3-0.5,
  y3-0.5, x4-0.5, y4-0.5);
 CenterX := CenterX * ImageWidth + x1;
 CenterY := CenterY * ImageHeight + y1;

 FRenderer.RotateAndScale(CenterX, CenterY, Rotation, ScaleX, ScaleY);
 FRenderer.Draw;
end;

begin
  if (ScaleX = 0) then ScaleX := 1;
  if (ScaleY = 0) then ScaleY := 1;
  if (BlendMode <> bmIgnore) then FRenderer.OmegaScreen.SetBlendMode(BlendMode);
  FRenderer.SetColors(col1, col2, col3, col4);
  FRenderer.MirrorHorizontally := MirrorHorizontally;
  FRenderer.MirrorVertically := MirrorVertically;
  if (ImgIndex <> -1) then DrawIndex else DrawFull;
end;

procedure TImageListItem.Draw(X, Y: integer; Rotation, CenterX, CenterY,
  ScaleX, ScaleY: single; col: TOmegaColor; MirrorHorizontally,
  MirrorVertically: boolean; ImgIndex: integer; BlendMode: integer = bmNormal);
begin
 Draw4Col(x, y, x + TileWidth, y, x + TileWidth, y + TileHeight,
  x, y + TileHeight, Rotation, CenterX, CenterY,
  ScaleX, ScaleY, col, col, col, col, MirrorHorizontally,
  MirrorVertically, ImgIndex, BlendMode);
end;

procedure TImageListItem.Draw(X, Y: integer; ImgIndex: integer);
begin
 Draw4Col(x, y, x + TileWidth, y, x + TileWidth, y + TileHeight,
  x, y + TileHeight, 0, 0, 0,
  1, 1, OmegaColor($FFFFFFFF), OmegaColor($FFFFFFFF),
  OmegaColor($FFFFFFFF), OmegaColor($FFFFFFFF), false,
  false, ImgIndex)
end;

procedure TImageListItem.Draw(X, Y: integer; Rotation: single; CenterX,
 CenterY: single; ScaleX, ScaleY: single; Red, Green, Blue, Alpha: integer;
 AdditiveBlend: Boolean; ImgIndex: integer; BlendMode: integer = bmNormal);
var c: TOmegaColor;
begin
 if AdditiveBlend then FDevice.SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ONE)
 else FDevice.SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
 c := OmegaColor(red, green, blue, alpha);
 Draw4Col(x, y, x + TileWidth, y, x + TileWidth, y + TileHeight,
  x, y + TileHeight, Rotation, CenterX, CenterY,
  ScaleX, ScaleY, c, c, c, c, false, false, ImgIndex, BlendMode);
end;

procedure TImageListItem.Draw(X, Y: integer; Rotation, CenterX,
  CenterY, ScaleX, ScaleY: single; col: TOmegaColor;
  ImgIndex: integer; BlendMode: integer = bmNormal);
begin
 Draw4Col(x, y, x + TileWidth, y, x + TileWidth, y + TileHeight,
  x, y + TileHeight, Rotation, CenterX, CenterY,
  ScaleX, ScaleY, col, col, col, col, false, false, ImgIndex, BlendMode);
end;

procedure TImageListItem.Draw(X, Y: integer; Rotation, CenterX,
  CenterY, ScaleX, ScaleY: single; red, green, blue, alpha,
  ImgIndex: integer; BlendMode: integer = bmNormal);
var c: TOmegaColor;
begin
 c := OmegaColor(red, green, blue, alpha);
 Draw4Col(x, y, x + TileWidth, y, x + TileWidth, y + TileHeight,
  x, y + TileHeight, Rotation, CenterX, CenterY,
  ScaleX, ScaleY, c, c, c, c, false, false, ImgIndex, BlendMode);
end;

procedure TImageListItem.Draw4Col(x1, y1, x2, y2, x3, y3, x4, y4: integer;
  Rotation, CenterX, CenterY, ScaleX, ScaleY: single; col1,
  col2, col3, col4: TOmegaColor; ImgIndex: integer; BlendMode: integer = bmNormal);
begin
 Draw4Col(x1, y1, x2, y2, x3, y3, x4, y4,
  Rotation, CenterX, CenterY, ScaleX, ScaleY,
  col1, col2, col3, col4, false, false, ImgIndex, BlendMode);
end;

procedure TImageListItem.Draw4Col(X, Y: integer; Rotation, CenterX,
  CenterY, ScaleX, ScaleY: single; col1, col2, col3,
  col4: TOmegaColor; ImgIndex: integer; BlendMode: integer = bmNormal);
begin
 Draw4Col(x, y, x + TileWidth, y, x + TileWidth, y + TileHeight,
  x, y + TileHeight, Rotation, CenterX, CenterY,
  ScaleX, ScaleY, col1, col2, col3, col4, false, false, ImgIndex, BlendMode);
end;

procedure TImageListItem.Draw4Col(X, Y: integer; Rotation, CenterX,
  CenterY, ScaleX, ScaleY: single; col1, col2, col3, col4: TOmegaColor;
  MirrorHorizontally, MirrorVertically: boolean; ImgIndex: integer;
  BlendMode: integer = bmNormal);
begin
 Draw4Col(x, y, x + TileWidth, y, x + TileWidth, y + TileHeight,
  x, y + TileHeight, Rotation, CenterX, CenterY,
  ScaleX, ScaleY, col1, col2, col3, col4,
  MirrorHorizontally, MirrorVertically, ImgIndex, BlendMode);
end;

procedure TImageListItem.DrawInRect(Rect: TRect;
 X, Y: integer;
 Rotation, CenterX, CenterY, ScaleX, ScaleY: single;
 red, green, blue, alpha: integer;
 mirrorh, mirrorv: boolean;
 imageindex: integer;
 blendmode: integer);
var c: TOmegaColor;
begin
 c := OmegaColor(red, green, blue, alpha);
 Draw4Col(x + Rect.Left, y + Rect.Top, x + Rect.Right,
 y + Rect.Top, x + Rect.Right, y + Rect.Bottom,
  x + Rect.Left, y + Rect.Bottom, Rotation, CenterX, CenterY,
  ScaleX, ScaleY, c, c, c, c, mirrorh, mirrorv, imageindex, blendmode);
end;

procedure TImageListItem.DrawInRect(Rect: TRect; X, Y: integer);
begin
 DrawInRect(Rect, X, Y, 0, 0, 0, 0, 0, 255, 255, 255, 255, false, false, -1, bmNormal);
end;

function TImageListItem.GetRealHeight: Integer;
begin
 if FRealHeight < 2 then
 begin
  FRealWidth   := 2;
  FRealHeight  := 2;
  while (FRealWidth < FWidth) do FRealWidth := FRealWidth shl 1;
  while (FRealHeight < FHeight) do FRealHeight := FRealHeight shl 1;
 end;
 result := FRealHeight;
end;

function TImageListItem.GetRealWidth: Integer;
begin
 if FRealWidth < 2 then
 begin
  FRealWidth   := 2;
  FRealHeight  := 2;
  while (FRealWidth < FWidth) do FRealWidth := FRealWidth shl 1;
  while (FRealHeight < FHeight) do FRealHeight := FRealHeight shl 1;
 end;
 result := FRealWidth;
end;

//----------------------------------------------------------------------------//
// ********************  End of ImageListItem ********************************//
//----------------------------------------------------------------------------//

//----------------------------------------------------------------------------//
// ******************  Beginning of ImageListCollection **********************//
//----------------------------------------------------------------------------//

type
  TImageListCollectionComponent = class(TComponent)
  private
    FList: TImageListCollection;
  published
    property List: TImageListCollection read FList write FList;
  end;

constructor TImageListCollection.Create(AOwner: TComponent);
begin
 inherited Create(AOwner, TImageListItem);
 FEncode := nil;
 FDecode := nil;
end;

procedure TImageListCollection.SaveToFile(const Filename: string);
var Stream: TMemoryStream;
begin
 Stream := TMemoryStream.Create;
 SaveToStream(TStream(Stream));
 Stream.SaveToFile(Filename);
 Stream.Free;
end;

procedure TImageListCollection.SaveToStream(var Stream: TStream);
var Component: TImageListCollectionComponent;
begin
 Component := TImageListCollectionComponent.Create(nil);
 try
  Component.FList := self;
  Stream.Position := 0;
  Stream.WriteComponentRes('OmegaImageListCollection', Component);
  Stream.Position := 0;
  if assigned(FEncode) then Stream := FEncode(Stream);
 finally
  Component.Free;
 end;
end;

procedure TImageListCollection.LoadFromFile(const Filename: string);
var Stream: TMemoryStream;
begin
  try
  Stream := TMemoryStream.Create;
  Stream.LoadFromFile(Filename);
  self.LoadFromStream(Stream);
  Stream.Free;
  except
  showmessage('Cannot be loaded - file error or file crypted!');
  end;
end;

procedure TImageListCollection.LoadFromStream(Stream: TStream);
var Component: TImageListCollectionComponent;
    ss: string;
begin
 Clear;
 Component := TImageListCollectionComponent.Create(nil);
 try
  Component.FList := self;
  Stream.Position := 0;
  if assigned(FDecode) then Stream := FDecode(Stream);
  Stream.Position := 0;
  Stream.ReadComponentRes(Component);
 except
  ss := 'The stream could not be loaded, either it is corrupt or encrypted!';
  messagebox(0, pchar(ss), 'Error!', 1);
 end;
 Component.Free;
end;

function TImageListCollection.GetItem(Index: Integer): TImageListItem;
begin
 Result := TImageListItem(inherited GetItem(Index));
end;

function TImageListCollection.Add: TImageListItem;
begin
 Result := TImageListItem(inherited Add);
end;

function TImageListCollection.Insert(Index: Integer): TImageListItem;
begin
 Result := TImageListItem(inherited Insert(Index));
end;

function TImageListCollection.Find(const SearchName: string): TImageListItem;
var i: integer;
begin
  Result := nil;
  for i := 0 to Count - 1 do
    if Items[i].Name = SearchName then
    begin
      Result := Items[i];
      Exit;
    end;
end;

procedure TImageListCollection.SetItem(Index: Integer; Value: TImageListItem);
begin
 inherited SetItem(Index, Value);
end;

procedure TImageListCollection.Update(Item: TCollectionItem);
begin
 inherited Update(Item);
end;

//----------------------------------------------------------------------------//
// ******************  End of ImageListCollection ****************************//
//----------------------------------------------------------------------------//

//----------------------------------------------------------------------------//
// ******************  Beginning of ImageList ********************************//
//----------------------------------------------------------------------------//

procedure TOmegaImageList.SetImageList(const Value: TImageListCollection);
begin
 FImageList.Assign(Value);
end;

constructor TOmegaImageList.Create(AOwner: TComponent);
begin
 inherited;
 FImageList := TImageListCollection.Create(Self);
 FRenderer := TOmegaRenderer.Create;
 TexScaleX := 0.1;//0.1;
 TexScaleY := 1;//0.1;
 TexOfsX := 0.0;
 TexOfsY := -0.5;
end;

destructor TOmegaImageList.Destroy;
begin
 FImageList.Free;
 if FRenderer <> nil then FRenderer := nil;
 inherited Destroy;
end;

procedure TOmegaImageList.Init;
var i: integer;
begin
 if FOmegaScreen = nil then exit;
// if FRenderer = nil then exit;
 FRenderer.OmegaScreen := FOmegaScreen;
 FRenderer.Init;
 if FOmegaScreen.Device = nil then exit;
 for i := 0 to ImageList.Count - 1 do ImageList.Items[i].Init;
end;

procedure TOmegaImageList.UnInit(deleteimages: boolean = false);
var i: integer;
begin
 if FRenderer <> nil then FRenderer.UnInit;
 for i := 0 to ImageList.Count - 1 do
  ImageList.Items[i].UnInit(deleteimages);
end;

procedure TOmegaImageList.BeginDraw;
begin
//  FRenderer._Begin;
end;

procedure TOmegaImageList.EndDraw;
begin
//  FRenderer._End;
end;

procedure TOmegaImageList.SetOmegaScreen(value: TOmegaScreen);
begin
 FOmegaScreen := Value;
end;

procedure TOmegaImageList.Notification(AComponent: TComponent;
 Operation: TOperation);
begin
 inherited;
 case Operation of
  opInsert: if (AComponent is TOmegaScreen)and(not Assigned(OmegaScreen)) then
   SetOmegaScreen(TOmegaScreen(AComponent));
  opRemove: if (AComponent = OmegaScreen) then OmegaScreen:= nil;
 end;
end;

//----------------------------------------------------------------------------//
// ******************  End of ImageList **************************************//
//----------------------------------------------------------------------------//

procedure TImageListItem.DrawRect(Rec: TRect; X, Y: integer; Rotation,
  RotationCenterX, RotationCenterY, ScaleX, ScaleY: single;
  red, green, blue, alpha: integer;
  mirrorh, mirrorv: boolean;
  imageindex: integer;
  blendmode: integer);
var
  l, r, t, b: double;
  w, h: integer;
  c: TOmegaColor;
begin
 l := Rec.Left / FRealWidth;
 t := Rec.Top / FRealHeight;
 r := Rec.Right / FRealWidth;
 b := Rec.Bottom / FRealHeight;
 w := Rec.Right - Rec.Left;
 h := Rec.Bottom - Rec.Top;

 if (ScaleX = 0) then ScaleX := 1;
 if (ScaleY = 0) then ScaleY := 1;
 if (BlendMode <> bmIgnore) then FRenderer.OmegaScreen.SetBlendMode(BlendMode);
 c := OmegaColor(red, green, blue, alpha);
 FRenderer.SetColors(c, c, c, c);
 FRenderer.MirrorHorizontally := MirrorH;
 FRenderer.MirrorVertically := MirrorV;
         
 FRenderer.SetTexturePositions(l, t, r, t, r, b, l, b);
 FDevice.SetTexture(0, image);

 FRenderer.SetPositions(x - 0.5, y - 0.5,
                        x + w - 0.5, y - 0.5,
                        x + w - 0.5, y + h - 0.5,
                        x - 0.5, y + h - 0.5);

 RotationCenterX := RotationCenterX * w + X;
 RotationCenterY := RotationCenterY * h + Y;

 FRenderer.RotateAndScale(RotationCenterX, RotationCenterY, Rotation,
                          ScaleX, ScaleY);
 FRenderer.Draw;
end;

procedure TImageListItem.DrawRect(Rec: TRect; X, Y: integer);
begin
 FRenderer.OmegaScreen.SetViewPort(rect(x,y,
  x+rec.right-rec.left,y+rec.bottom-rec.top));
 Draw(x-rec.left,y-rec.top,-1);
 FRenderer.OmegaScreen.SetPreviousViewPort;
end;

end.




