{
//****************************************************************
// OmegaInput unit
//****************************************************************

version history:

07/26/04, Tobias Fleischer (Tobybear):
 - now uses latest headers in DirectInput8.pas 

07/25/04, baRman_ru:
 - defined default return value for function VKToDIK

04/08/04, Jannik Niemer (Ztar):
 - made States and StatesClicked public
 - added support for Gamepads (Steering Wheels etc. should also work)
 - added Support for multiple Joysticks / Gamepads
 - added AxesCount, ButtonCount and HasForceFeedback properties (to support
                                                                 FF later on)
11/06/03, Tobias Fleischer (Tobybear):
 - made States and StatesClicked property writeable
 - added events

Original version by Jannik Niemer (Ztar)
}
{$DEFINE DIRECTX8}

unit OmegaInput;

interface
uses DirectInput8, Classes, Windows, OmegaClass, Messages,
 Forms, Controls;
type
  TOmegaInputState = (oisUp, oisDown, oisLeft, oisRight, oisButton1, oisButton2,
                      oisButton3, oisButton4, oisButton5, oisButton6, oisButton7,
                      oisButton8, oisButton9, oisButton10, oisButton11, oisButton12,
                      oisButton13, oisButton14, oisButton15, oisButton16, oisButton17,
                      oisButton18, oisButton19, oisButton20, oisButton21, oisButton22,
                      oisButton23, oisButton24, oisButton25, oisButton26, oisButton27,
                      oisButton28);

  TOmegaInputStates = set of TOmegaInputState;

  TOmegaInput = class;

  PKeyAssign = ^TKeyAssign;
  TKeyAssign = integer;
  TKeyAssigns = array[TOmegaInputState] of TKeyAssign;

  TCustomInput = class(TPersistent)
  private
    FStates: TOmegaInputStates;
    FStatesClicked: TOmegaInputStates;
    FStatesClickedBusy: TOmegaInputStates;
    FInitialized: boolean;
    FOmegaInput: TOmegaInput;
    FDevice: IDirectInputDevice8;
    FEnabled: boolean;
    FForegroundOnly: boolean;
    FExclusive: boolean;
    Procedure SetEnabled(value: boolean);
    Procedure SetForegroundOnly(Value: boolean);
    PRocedure SetExclusive(Value: boolean);
    Procedure Initialize;virtual;
    Procedure Finalize;
  public
    Constructor Create(OmegaInput: TOmegaInput);virtual;
    Procedure Acquire;
    property Device: IDirectINputDevice8 read FDevice;
    property States: TOmegaInputStates read FStates write FStates;
    property StatesClicked: TOmegaInputStates read FStatesClicked write FStatesClicked;
  published
    property Enabled: boolean read FEnabled write SetEnabled;
    property Initialized: boolean read FInitialized;
    property ForegroundOnly: boolean read FForegroundOnly write SetForegroundOnly;
    property Exclusive: boolean read FExclusive write SetExclusive;
  end;

  TKeyboard = class(TCustominput)
  private
    Procedure Initialize;override;
  protected
    Procedure DefineProperties(Filer: TFiler); override;
  public
    KeyAssigns: TKeyAssigns;
    Constructor Create(OmegaInput: TOmegaInput);override;
    Procedure Update;
    Procedure ReadAssigns(Stream: TStream);
    Procedure WriteAssigns(Stream: TStream);
  end;

  TMouse = class(TCustomInput)
  private
    FMouseState: TDIMouseState;
    Function GetX: integer;
    Function GetY: integer;
    Function GetZ: integer;
    Procedure Initialize;override;
  public
    Procedure Update;
  published
    property X: integer read GetX;
    property Y: integer read GetY;
    property Z: integer read GetZ;
  end;

  TJoystick = class(TCustomInput)
  private
    FAutoCenter: boolean;
    FHasForceFeedback: boolean;
    FJoyState: TDIJoyState;
    FButtonCount: integer;
    FAxesCount: integer;
    FID: integer;
    FGUID: TGUID;
    FName: string;
    FEnumIndex: integer;
    FEnumFlag: boolean;
    FRanges: array[0..Sizeof(TDiJoyState) - 1] of integer;
    FDeadZones: array[0..Sizeof(TDiJoyState) - 1] of integer;
    Function GetX: integer;
    Function GetY: integer;
    Function GetZ: integer;
    Procedure SetID(Value: integer);
    Procedure SetAutocenter(Value: boolean);
    Procedure SetRange(Obj: integer; Value: integer);
    Procedure SetDeadZone(Obj: integer; Value: integer);
    Function GetRange(Obj: integer): integer;
    Function GetDeadZone(obj: integer): integer;
    Procedure Initialize;override;
  public
    Constructor Create(OmegaInput: TOmegaInput);override;
    Destructor Destroy; override;
    Procedure Update;
    property HasForceFeedback: boolean read FHasForceFeedback;
    property JoyState: TDIJoyState read FJoyState;
    property DeadZone[Obj: integer]: integer read GetDeadzone write SetDeadzone;
    property Range[Obj: integer]: integer read GetRange write SetRange;
    property X: integer read GetX;
    property Y: integer read GetY;
    property Z: integer read GetZ;
    property Name: string read FName;
    property ID: integer read FID write SetID;
  published
    property Autocenter: boolean read FAutocenter write SetAutoCenter;
    property DeadZoneX: integer index DIJOFS_X read GetDeadZone write SetDeadZone;
    property DeadZoneY: integer index DIJOFS_Y read GetDeadZone write SetDeadZone;
    property DeadZoneZ: integer index DIJOFS_Z read GetDeadZone write SetDeadZone;
    property AxesCount: integer read FAxesCount;
    property ButtonCount: integer read FButtonCount;
    property RangeX: integer index DIJOFS_X read GetRange write SetRange;
    property RangeY: integer index DIJOFS_Y read GetRange write SetRange;
    property RangeZ: integer index DIJOFS_Z read GetRange write SetRange;
  end;

  TOmegaInput = class(TComponent)
  private
    FOnKeyboard: TNotifyEvent;
    FOnMouse: TNotifyEvent;
    FOnJoystick: TNotifyEvent;
    FForm: TCustomForm;
    FInitialized: boolean;
    FWndProcControl: TWndProcControlClass;
    FDInput: IDirectInput8;
    FKeyboard: TKeyboard;
    FMouse: TMouse;
    FJoystick: TJoystick;
    FJoysticks: TList;
    Procedure Finalize;
    Procedure SetKeyboard(Value: TKeyboard);
    Procedure SetMouse(Value: TMouse);
    Procedure SetJoystick(Value: TJoystick);
    Function GetJoystick(index: integer): TJoystick;
    Function GetJoyCount: integer;
    Procedure FormWndProc(var Message: TMessage; DefWindowProc: TWndMethod);
  public
    Constructor Create(AOwner: TComponent); override;
    Destructor Destroy; override;
    Procedure Update;
    Procedure Initialize;
    property Joysticks[index: integer]: TJoystick read GetJoystick;
    property JoystickCount: integer read GetJoyCount;
  published
    property Keyboard: TKeyboard read FKeyboard write SetKeyboard;
    property Mouse: TMouse read FMouse write SetMouse;
    property Joystick: TJoystick read FJoystick write SetJoystick;
    property Initialized: boolean read FInitialized;
    property OnKeyboard: TNotifyEvent read FOnKeyboard write FOnKeyboard;
    property OnMouse: TNotifyEvent read FOnMouse write FOnMouse;
    property OnJoystick: TNotifyEvent read FOnJoystick write FOnJoystick;
  end;

  Function Def_KeyAssigns: TKeyAssigns;

  Function VKToDIK(key: integer): integer;
  function DIKToStr(key: integer): string;

implementation

uses SysUtils;

Function DIKToStr(key: integer): string;
begin
  Result := '';
  case key of
    0             : Result := 'None';
    DIK_0         : Result := '0';
    DIK_1..DIK_9  : Result := char(key - DIK_1 + 1 + $30);
    DIK_A         : Result := 'A';
    DIK_ADD       : Result := 'NUM +';
    DIK_APOSTROPHE: Result := 'Apostrophe';
    DIK_B         : Result := 'B';
    DIK_BACK      : Result := 'Backspace';
    DIK_BACKSLASH : Result := 'Backslash';
    DIK_C         : Result := 'C';
    DIK_CALCULATOR: Result := 'Calculator';
    DIK_CAPITAL   : Result := 'Caps Lock';
    DIK_COMMA     : Result := ',';
    DIK_D         : Result := 'D';
    DIK_DECIMAL   : Result := 'NUM ,';
    DIK_DELETE    : Result := 'Delete';
    DIK_DIVIDE    : Result := 'NUM /';
    DIK_DOWN      : Result := 'Down Arrow';
    DIK_E         : Result := 'E';
    DIK_END       : Result := 'End';
    DIK_EQUALS    : Result := '=';
    DIK_ESCAPE    : Result := 'Esc';
    DIK_F         : Result := 'F';
    DIK_F1        : Result := 'F1';
    DIK_F2        : Result := 'F2';
    DIK_F3        : Result := 'F3';
    DIK_F4        : Result := 'F4';
    DIK_F5        : Result := 'F5';
    DIK_F6        : Result := 'F6';
    DIK_F7        : Result := 'F7';
    DIK_F8        : Result := 'F8';
    DIK_F9        : Result := 'F9';
    DIK_F10       : Result := 'F10';
    DIK_F11       : Result := 'F11';
    DIK_F12       : Result := 'F12';
    DIK_F13       : Result := 'F13';
    DIK_F14       : Result := 'F14';
    DIK_F15       : Result := 'F15';
    DIK_G         : Result := 'G';
    DIK_GRAVE     : Result := 'Grave';
    DIK_H         : Result := 'H';
    DIK_HOME      : Result := 'Home';
    DIK_I         : Result := 'I';
    DIK_INSERT    : Result := 'Insert';
    DIK_J         : Result := 'J';
    DIK_K         : Result := 'K';
    DIK_L         : Result := 'L';
    DIK_LBRACKET  : Result := '[';
    DIK_LCONTROL  : Result := 'Left Ctrl';
    DIK_LEFT      : Result := 'Left Arrow';
    DIK_LMENU     : Result := 'Left Alt';
    DIK_LSHIFT    : Result := 'Left Shift';
    DIK_M         : Result := 'M';
    DIK_MINUS     : Result := '-';
    DIK_MULTIPLY  : Result := 'NUM *';
    DIK_N         : Result := 'N';
    DIK_NEXT      : Result := 'Page Down';
    DIK_NUMPAD0   : Result := 'NUM 0';
    DIK_NUMPAD1   : Result := 'NUM 1';
    DIK_NUMPAD2   : Result := 'NUM 2';
    DIK_NUMPAD3   : Result := 'NUM 3';
    DIK_NUMPAD4   : Result := 'NUM 4';
    DIK_NUMPAD5   : Result := 'NUM 5';
    DIK_NUMPAD6   : Result := 'NUM 6';
    DIK_NUMPAD7   : Result := 'NUM 7';
    DIK_NUMPAD8   : Result := 'NUM 8';
    DIK_NUMPAD9   : Result := 'NUM 9';
    DIK_NUMPADENTER: Result := 'NUM Enter';
    DIK_O : Result := 'O';
    DIK_OEM_102     : Result := 'OEM_102';
    DIK_P : Result := 'P';
    DIK_PERIOD : Result := '.';
    DIK_PRIOR  : Result := 'Page Up';
    DIK_Q : Result := 'Q';
    DIK_R : Result := 'R';
    DIK_RBRACKET : Result := ']';
    DIK_RCONTROL  : Result := 'Right Ctrl';
    DIK_RETURN  : Result := 'Enter';
    DIK_RIGHT  : Result := 'Right Arrow';
    DIK_RMENU  : Result := 'Right Alt';
    DIK_RSHIFT : Result := 'Right Shift';
    DIK_S  : Result := 'S';
    DIK_SEMICOLON : Result := ';';
    DIK_SLASH  : Result := '/';
    DIK_SPACE  : Result := 'Space';
    DIK_SUBTRACT : Result := 'NUM -';
    DIK_SYSRQ : Result := 'SysRQ';
    DIK_T        : Result := 'T';
    DIK_TAB     : Result := 'Tab';
    DIK_U      : Result := 'U';
    DIK_UP      : Result := 'Up Arrow';
    DIK_V      : Result := 'V';
    DIK_W      : Result := 'W';
    DIK_X     : Result := 'X';
    DIK_Y      : Result := 'Y';
    DIK_Z     : Result := 'Z';
  end;
end;

Function VKToDIK(key: integer): integer;
begin
  case key of
    VK_BACK: Result := DIK_BACK;
    VK_TAB: Result := DIK_TAB;
    VK_CLEAR: Result := DIK_NUMPAD5;      {Num Lock off}
    VK_RETURN: Result := DIK_RETURN;
    VK_SHIFT, VK_LSHIFT: Result := DIK_LSHIFT;
    VK_RSHIFT: Result := DIK_RSHIFT;
    VK_CONTROL, VK_LCONTROL: Result := DIK_LCONTROL;
    VK_RCONTROL: Result := DIK_RCONTROL;
    VK_MENU, VK_LMENU: Result := DIK_LMENU;
    VK_RMENU: Result := DIK_RMENU;
    VK_CAPITAL: Result := DIK_CAPITAL;
    VK_ESCAPE: Result := DIK_ESCAPE;
    VK_SPACE: Result := DIK_SPACE;
    VK_PRIOR: Result := DIK_PRIOR;
    VK_NEXT: Result := DIK_NEXT;
    VK_END: Result := DIK_END;
    VK_HOME: Result := DIK_HOME;
    VK_LEFT: Result := DIK_LEFT;
    VK_UP: Result := DIK_UP;
    VK_RIGHT: Result := DIK_RIGHT;
    VK_DOWN: Result := DIK_DOWN;
    VK_INSERT: Result := DIK_INSERT;
    VK_DELETE: Result := DIK_DELETE;
    $30: Result := DIK_0;
    $31..$39: Result := DIK_1 - $31 + key;
    $41: Result := DIK_A;
    $42: Result := DIK_B;
    $43: Result := DIK_C;
    $44: Result := DIK_D;
    $45: Result := DIK_E;
    $46: Result := DIK_F;
    $47: Result := DIK_G;
    $48: Result := DIK_H;
    $49: Result := DIK_I;
    $4A: Result := DIK_J;
    $4B: Result := DIK_K;
    $4C: Result := DIK_L;
    $4D: Result := DIK_M;
    $4E: Result := DIK_N;
    $4F: Result := DIK_O;
    $50: Result := DIK_P;
    $51: Result := DIK_Q;
    $52: Result := DIK_R;
    $53: Result := DIK_S;
    $54: Result := DIK_T;
    $55: Result := DIK_U;
    $56: Result := DIK_V;
    $57: Result := DIK_W;
    $58: Result := DIK_X;
    $59: Result := DIK_Y;
    $5A: Result := DIK_Z;
    VK_LWIN: Result := DIK_LWIN;
    VK_RWIN: Result := DIK_RWIN;
    VK_APPS: Result := DIK_APPS;
    VK_NUMPAD0: Result := DIK_NUMPAD0;
    VK_NUMPAD1: Result := DIK_NUMPAD1;
    VK_NUMPAD2: Result := DIK_NUMPAD2;
    VK_NUMPAD3: Result := DIK_NUMPAD3;
    VK_NUMPAD4: Result := DIK_NUMPAD4;
    VK_NUMPAD5: Result := DIK_NUMPAD5;      {Num Lock on}
    VK_NUMPAD6: Result := DIK_NUMPAD6;
    VK_NUMPAD7: Result := DIK_NUMPAD7;
    VK_NUMPAD8: Result := DIK_NUMPAD8;
    VK_NUMPAD9: Result := DIK_NUMPAD9;
    VK_MULTIPLY: Result := DIK_MULTIPLY;
    VK_ADD: Result := DIK_ADD;
    VK_SUBTRACT: Result := DIK_SUBTRACT;
    VK_DECIMAL: Result := DIK_DECIMAL;
    VK_DIVIDE: Result := DIK_DIVIDE;
    VK_F1: Result := DIK_F1;
    VK_F2: Result := DIK_F2;
    VK_F3: Result := DIK_F3;
    VK_F4: Result := DIK_F4;
    VK_F5: Result := DIK_F5;
    VK_F6: Result := DIK_F6;
    VK_F7: Result := DIK_F7;
    VK_F8: Result := DIK_F8;
    VK_F9: Result := DIK_F9;
    VK_F10: Result := DIK_F10;
    VK_F11: Result := DIK_F11;
    VK_F12: Result := DIK_F12;
    VK_NUMLOCK: Result := DIK_NUMLOCK;
    VK_SCROLL: Result := DIK_SCROLL;

    192: Result := DIK_SEMICOLON;      //
    221: Result := DIK_EQUALS;         //
    188: Result := DIK_COMMA;
    219: Result := DIK_MINUS;
    190: Result := DIK_PERIOD;
    189: Result := DIK_SLASH;
    220: Result := DIK_GRAVE;
    186: Result := DIK_LBRACKET;
    191: Result := DIK_BACKSLASH;
    187: Result := DIK_RBRACKET;
    222: Result := DIK_APOSTROPHE;
    else Result:= 0;
  end;
end;

Function Def_KeyAssigns: TKeyAssigns;
var
  temp: TKeyAssigns;
begin
  FillChar(temp, Sizeof(TKeyAssigns), 0);
  temp[oisUp] := DIK_UP;
  temp[oisDown] := DIK_DOWN;
  temp[oisLeft] := DIK_LEFT;
  temp[oisRight] := DIK_RIGHT;
  temp[oisButton1] := DIK_SPACE;
  temp[oisButton2] := DIK_LCONTROL;
  temp[oisButton3] := DIK_RETURN;
  temp[oisButton27] := DIK_ESCAPE;
  temp[oisButton28] := DIK_LMENU;
  Result := temp;
end;

Constructor TCustomInput.Create(OmegaInput: TOmegaInput);
begin
  inherited Create;
  FDevice := nil;
  FOmegaInput := OmegaInput;
  FEnabled := false;
  FExclusive := false;
  FForegroundOnly := true;
end;

Procedure TCustomInput.Initialize;
begin
end;

Procedure TCustomInput.Finalize;
begin
  if FDevice <> nil then FDevice.Unacquire;
  FDevice := nil;
  FInitialized := false;
end;

procedure TCustomInput.SetEnabled(Value: boolean);
begin
  if not Value then
  begin
    if FInitialized then Finalize;
    FEnabled := false;
  end else
  begin
    FEnabled := true;
    if (not FInitialized) and (FOmegaInput.Initialized) then
    begin
      Initialize;
      Acquire;
    end;
  end;
end;

procedure TCustomInput.SetForegroundOnly(Value: boolean);
begin
  if FForegroundOnly <> Value then
  begin
    FForeGroundOnly := value;
    if Initialized then
    begin
      Initialize; //Reinit if already initialized
      Acquire;
    end;
  end;
end;

Procedure TCustomInput.SetExclusive(Value: boolean);
begin
  if FExclusive <> Value then
  begin
    FExclusive := Value;
    if Initialized then
    begin
      Initialize; //Reinit if already initialized
      Acquire;
    end;
  end;
end;

procedure TCustomInput.Acquire;
begin
  if FDevice <> nil then FDevice.Acquire;
end;

Constructor TKeyboard.Create(OmegaInput: TOmegaInput);
begin
  inherited Create(OmegaInput);
  FEnabled := true;
  KeyAssigns := Def_KeyAssigns;
end;

Procedure TKeyboard.Update;
var
  buffer: TDIKeyboardState;
  hr: HRESULT;
  i: TOmegaInputState;
begin
  FStates := [];
  FStatesClicked := [];
  if FInitialized and (FDevice <> nil) then
  begin
    FillChar(buffer, Sizeof(buffer), 0);
    hr := FDevice.GetDeviceState(Sizeof(Buffer), @buffer);
    if Failed(hr) then
    begin
      if hr = DIERR_INPUTLOST then
        FDevice.Acquire;
      Exit;
    end;
    for i := Low(TOmegaInputState) to High(TOmegaInputState) do
    begin
      if buffer[KeyAssigns[i]] and $80 <> 0 then
      begin
        //Down
        FStates := FStates + [i];
        if not (i in FStatesClickedBusy) then
        begin
          FStatesClicked := FStatesClicked + [i];
          FStatesClickedBusy := FStatesClickedBusy + [i];
        end;
      end
      else
        //Up
        FStatesClickedBusy := FStatesClickedBusy - [i];
    end;
  end;
end;

Procedure TKeyboard.Initialize;
var
  Format: TDIDataFormat;
  flags: cardinal;
begin
  Finalize;
  if (csDesigning in FOmegaInput.ComponentState) or (not Enabled) then Exit;
  if FOmegaInput.FDInput <> nil then
  begin
    Format := c_dfDIKeyboard;
    Flags := 0;
    if not Exclusive then Flags := DISCL_NONEXCLUSIVE else Flags := DISCL_EXCLUSIVE;
    if not ForegroundOnly then Flags := Flags or DISCL_BACKGROUND
                          else Flags := Flags or DISCL_FOREGROUND;
    if Succeeded(FOmegaInput.FDInput.CreateDevice(GUID_SysKeyboard, FDevice, nil)) then
      if Succeeded(FDevice.SetDataFormat(@Format)) then
        if Succeeded(FDevice.SetCooperativeLevel(FOmegaInput.FForm.Handle, Flags)) then FInitialized := true;
  end;
end;

Procedure TKeyboard.ReadAssigns(Stream: TStream);
begin
  Stream.ReadBuffer(KeyAssigns, Sizeof(KeyAssigns));
end;

Procedure TKeyboard.WriteAssigns(Stream: TStream);
begin
  Stream.WriteBuffer(KeyAssigns, Sizeof(KeyAssigns));
end;

Procedure TKeyboard.DefineProperties(Filer: TFiler);
begin
  inherited DefineProperties(Filer);
  Filer.DefineBinaryProperty('Assigns', ReadAssigns, WriteAssigns, true);
  Filer.DefineBinaryProperty('Assigns2', ReadAssigns, WriteAssigns, false);
end;

Procedure TMouse.Update;
var
  hr: HRESULT;
  i: TOmegaInputState;
begin
  FStates := [];
  FStatesClicked := [];
  if FInitialized and (FDevice <> nil) then
  begin
    FillChar(FMouseState, Sizeof(FMouseState), 0);
    hr := FDevice.GetDeviceState(Sizeof(FMouseState), @FMouseState);
    if Failed(hr) then
    begin
      if hr = DIERR_INPUTLOST then
        FDevice.Acquire;
      Exit;
    end;
    if FMouseState.lX > 0 then FStates := FStates + [oisRight];
    if FMouseState.lX < 0 then FStates := FStates + [oisLeft];
    if FMouseState.lY > 0 then FStates := FStates + [oisDown];
    if FMouseState.lY < 0 then FStates := FStates + [oisUp];
    if FMouseState.rgbButtons[0] and $80 <> 0 then FStates := FStates + [oisButton1];
    if FMouseState.rgbButtons[1] and $80 <> 0 then FStates := FStates + [oisButton2];
    if FMouseState.rgbButtons[2] and $80 <> 0 then FStates := FStates + [oisButton3];
    if FMouseState.rgbButtons[3] and $80 <> 0 then FStates := FStates + [oisButton4];
    for i := Low(TOmegaInputState) to High(TOmegaInputState) do
    begin
      if i in FStates then
      begin
        //Down
        if not (i in FStatesClickedBusy) then
        begin
          FStatesClicked := FStatesClicked + [i];
          FStatesClickedBusy := FStatesClickedBusy + [i];
        end;
      end
      else
        //Up
        FStatesClickedBusy := FStatesClickedBusy - [i];
    end;
  end;
end;

Procedure TMouse.Initialize;
var
  Format: TDIDataFormat;
  Flags: Cardinal;
begin
  Finalize;
  if (csDesigning in FOmegaInput.ComponentState) or (not Enabled) then Exit;
  if FOmegaInput.FDInput <> nil then
  begin
    Format := c_dfDIMouse;
    Flags := 0;
    if not Exclusive then Flags := DISCL_NONEXCLUSIVE else Flags := DISCL_EXCLUSIVE;
    if not ForegroundOnly then Flags := Flags or DISCL_BACKGROUND
                          else Flags := Flags or DISCL_FOREGROUND;
    if Succeeded(FOmegaInput.FDInput.CreateDevice(GUID_SysMouse, FDevice, nil)) then
      if Succeeded(FDevice.SetDataFormat(@Format)) then
        if Succeeded(FDevice.SetCooperativeLevel(FOmegaInput.FForm.Handle, Flags)) then
          FInitialized := true;
  end;
end;

Function TMouse.GetX: integer;
begin
  Result := FMouseState.lX;
end;                       

Function TMouse.GetY: integer;
begin
  Result := FMouseState.lY;
end;

Function TMouse.GetZ: integer;
begin
  Result := FMouseState.lZ;
end;

Constructor TJoystick.Create(OmegaInput: TOmegaInput);
var
  i: integer;
begin
  inherited Create(OmegaInput);
  OmegaInput.FJoysticks.Add(Self);
  for i := Low(FDeadZones) to High(FDeadZones) do
    FDeadZones[i] := 20;
  for i := Low(FRanges) to High(FRanges) do
    FRanges[i] := 1000;

  DeadZoneX := 20;
  DeadZoneY := 20;
  DeadZoneZ := 20;

  RangeX := 1000;
  RangeY := 1000;
  RangeZ := 1000;

  FEnabled := false;
  FAutocenter := true;
  FID := 0;
  FButtonCount := 0;
end;

Destructor TJoystick.Destroy;
begin
  FOmegaInput.FJoysticks.Remove(self);
  inherited;
end;

Procedure TJoystick.SetID(Value: integer);
begin
  if FID <> Value then
  begin
    FID := Value;
    if Initialized then   //If the Joystick was initialized before,
      Initialize;         //reinit it
  end;
end;

Procedure TJoystick.SetAutocenter(Value: boolean);
var
  prop: TDIPropDWord;
begin
  if FAutoCenter <> Value then
  begin
    FAutoCenter := Value;
    prop.diph.dwSize := Sizeof(TDIPropDWord);
    prop.diph.dwHeaderSize := Sizeof(TDiPropHeader);
    prop.diph.dwObj := 0;
    prop.diph.dwHow := DIPH_DEVICE;
    prop.dwData := ord(Value);
    if FDevice <> nil then FDevice.SetProperty(DIPROP_AUTOCENTER, prop.diph);
  end;
end;

Procedure TJoystick.SetRange(Obj: integer; Value: integer);
var
  prop: TDIPropRange;
begin
  if (Obj >= Low(FRanges)) and (Obj <= High(FRanges)) then
  begin
    if Value < 0 then Value := 0;
    self.FRanges[Obj] := Value;
    if FDevice <> nil then
    begin
      prop.diph.dwSize := Sizeof(TDIPropRange);
      prop.diph.dwHeaderSize := Sizeof(TDIpropHeader);
      prop.diph.dwObj := Obj;
      prop.diph.dwHow := DIPH_BYOFFSET;
      prop.lMin := -Value;
      prop.lMax := Value;
      FDevice.SetProperty(DIPROP_RANGE, prop.diph);
    end;
  end;
end;

Procedure TJoystick.SetDeadZone(Obj: integer; Value: integer);
var
  prop: TDIPropDWord;
begin
  if (Obj >= Low(FDeadZones)) and (Obj <= High(FDeadZones)) then
  begin
    if Value < 0 then Value := 0;
    if Value > 100 then Value := 100;
    self.FDeadZones[Obj] := Value;
    if FDevice <> nil then
    begin
      prop.diph.dwSize := Sizeof(TDIPropDWord);
      prop.diph.dwHeaderSize := Sizeof(TDIPropHeader);
      prop.diph.dwObj := Obj;
      prop.diph.dwHow := DIPH_BYOFFSET;
      prop.dwData := Value * 100;
      FDevice.SetProperty(DIPROP_DEADZONE, prop.diph);
    end;
  end;
end;

Function TJoystick.GetRange(Obj: integer): integer;
begin
  if (Obj >= Low(FRanges)) and (Obj <= High(FRanges)) then
    Result := FRanges[Obj]
  else Result := 0;
end;

Function TJoystick.GetDeadZone(Obj: integer): integer;
begin
  if (Obj >= Low(FDeadZones)) and (Obj <= High(FDeadZones)) then
    Result := FDeadZones[Obj]
  else Result := 0;
end;

Procedure TJoystick.Update;
var
  hr: HRESULT;
  i: TOmegaInputState;
  j: integer;
begin
  FStates := [];
  FStatesClicked := [];
  if FInitialized and (FDevice <> nil) then
  begin
    FillChar(FJoyState, Sizeof(FJoyState), 0);
    hr := FDevice.Poll;
    if Failed(hr) then
    begin
      if hr = DIERR_INPUTLOST then
        FDevice.Acquire;
      Exit;
    end;
    hr := FDevice.GetDeviceState(Sizeof(TDiJoyState), @FJoyState);
    if FJoyState.lX > 0 then FStates := FStates + [oisRight];
    if FJoyState.lX < 0 then FStates := FStates + [oisLeft];
    if FJoyState.lY > 0 then FStates := FStates + [oisDown];
    if FJoyState.lY < 0 then FStates := FStates + [oisUp];
    for j := 0 to FButtonCount-1 do
    begin
      if FJoyState.rgbButtons[j] and $80 <> 0 then FStates := FStates + [TOmegaInputState(ord(oisButton1) + j)];
    end;
    for i := Low(TOmegaInputState) to High(TOmegaInputState) do
    begin
      if i in FStates then
      begin
        //Down
        if not (i in FStatesClickedBusy) then
        begin
          FStatesClicked := FStatesClicked + [i];
          FStatesClickedBusy := FStatesClickedBusy + [i];
        end;
      end
      else
        //Up
        FStatesClickedBusy := FStatesClickedBusy - [i];
    end;
  end;
end;

Procedure TJoystick.Initialize;
var
  Format: TDIDataFormat;
  Caps: TDIDevCaps;
  Flags: Cardinal;
  i: integer;

  Function JoystickEnumCallback(var lpdii: TDIDeviceInstanceA; pvref: pointer): HRESULT; stdcall;
  begin
    Result := DIENUM_CONTINUE;
    With TJoystick(pvref) do
    begin
      if FEnumIndex = FID then
      begin
        FGUID := lpdii.guidInstance;
        FName := lpdii.tszProductName;
        FEnumFlag := true;
        Result := DIENUM_STOP;
        Exit;
      end;
      inc(FEnumIndex);
    end;
  end;

begin
  Finalize;
  if (csDesigning in FOmegaInput.ComponentState) then Exit;

  if FOmegaInput.FDInput <> nil then
  begin
    FEnumFlag := false;
    FEnumIndex := 0;
    FOmegaInput.FDInput.EnumDevices(DI8DEVCLASS_GAMECTRL, @JoystickEnumCallback, self, DIEDFL_ATTACHEDONLY);

    if (not FEnumFlag) or (not Enabled) then Exit;

    Format := c_dfDIJoystick;
    Caps.dwSize := Sizeof(TDIDevCaps);
    if Succeeded(FOmegaInput.FDInput.CreateDevice(FGUID, FDevice, nil)) then
      if Succeeded(FDevice.GetCapabilities(Caps)) then
          begin
            FButtonCount := Caps.dwButtons;
            FAxesCount := Caps.dwAxes;
            FHasForceFeedback := Boolean(Caps.dwFlags and DIDC_FORCEFEEDBACK);
            Flags := 0;
            if not Exclusive then Flags := DISCL_NONEXCLUSIVE else Flags := DISCL_EXCLUSIVE;
            if not ForegroundOnly then Flags := Flags or DISCL_BACKGROUND
                                  else Flags := Flags or DISCL_FOREGROUND;
              if Succeeded(FDevice.SetDataFormat(@Format)) then
                if Succeeded(FDevice.SetCooperativeLevel(FOmegaInput.FForm.Handle, DISCL_NONEXCLUSIVE or DISCL_FOREGROUND)) then
                begin
                  SetAutocenter(FAutocenter);
                  for i := Low(FRanges) to High(FRanges) do
                    SetRange(i, FRanges[i]);
                  for i := Low(FDeadZones) to High(FDeadZones) do
                    SetDeadZone(i, FDeadZones[i]);
                  FInitialized := true;
                end;
          end;
  end;
end;

Function TJoystick.GetX: integer;
begin
  Result := FJoyState.lX;
end;                       

Function TJoystick.GetY: integer;
begin
  Result := FJoyState.lY;
end;

Function TJoystick.GetZ: integer;
begin
  Result := FJoyState.lZ;
end;

Constructor TOmegaInput.Create(AOwner: TComponent);
var
  Component: TComponent;
begin
  inherited Create(AOwner);
  FDinput := nil;
  FInitialized := false;               
  Component := Owner;
  while (Component <> nil) and (not (Component is TCustomForm)) do
    Component := Component.Owner;
  if Component = nil then
    raise Exception.Create('OmegaInput: No Form present!');
  FForm := TCustomForm(Component);
  FJoysticks := TList.Create;
  FKeyboard := TKeyboard.Create(self);
  FMouse := TMouse.Create(self);
  FJoystick := TJoystick.Create(self);
  FWndProcControl := TWndProcControlClass.Create(FForm, FormWndProc);
end;

Destructor TOmegaInput.Destroy;
begin
  Finalize;
  FKeyboard.Free;
  FMouse.Free;
  FJoystick.Free;
  while FJoysticks.Count > 0 do
    TJoystick(FJoysticks.Items[FJoysticks.Count - 1]).Free;
  FJoysticks.Free;
  FWndProcControl.Free;
  inherited Destroy;
end;

Procedure TOmegaInput.FormWndProc(var Message: TMessage; DefWindowProc: TWndMethod);
var
  i: integer;
begin
  case Message.Msg of
    WM_CREATE:
    begin
      DefWindowProc(Message);
      Initialize;
      Exit;
    end;
    WM_ACTIVATEAPP:
    begin
      DefWindowProc(Message);
      if TWmActivateApp(Message).Active then
      begin
        Keyboard.Acquire;
        Mouse.Acquire;
        for i := 0 to FJoysticks.Count - 1 do
          TJoystick(FJoysticks[i]).Acquire;
        //Joystick.Acquire;
      end;
      Exit;
    end;
    WM_ACTIVATE:
    begin
      DefWindowProc(Message);
      if TWmActivate(Message).Active<>WA_INACTIVE then
      begin
        Keyboard.Acquire;
        Mouse.Acquire;
        for i := 0 to FJoysticks.Count - 1 do
          TJoystick(FJoysticks[i]).Acquire;
        //Joystick.Acquire;
      end;
      Exit;
    end;
  end;
  DefWindowProc(Message);
end;

Procedure TOmegaInput.Initialize;
var
  i: integer;
  j: integer;

  Function JoystickEnumCallback(var lpdii: TDIDeviceInstanceA; pvref: pointer): HRESULT; stdcall;
  begin
    TOmegaInput(pvref).Tag := TOmegaInput(pvref).Tag + 1;
    Result := DIENUM_CONTINUE;
  end;

begin
  Finalize;
  if not (csDesigning in ComponentState) then
  begin
    DirectInput8Create(hInstance, DIRECTINPUT_VERSION, IID_IDirectInput8A, FDInput, nil);
    FInitialized := true;
    FKeyboard.Initialize;
    FMouse.Initialize;

    Tag := 0;
    FDInput.EnumDevices(DI8DEVCLASS_GAMECTRL, @JoystickEnumCallback, self, DIEDFL_ATTACHEDONLY);
    //Joystick with ID 0 is already saved in Joysticks[0]
    for i := 1 to Tag - 1 do
    begin
      with TJoystick.Create(self) do
      begin
        ID := i;
        FAutoCenter := Joystick.FAutoCenter;
        for j := Low(FDeadZones) to High(FDeadZones) do
          FDeadZones[j] := Joystick.DeadZone[j];
        for j := Low(FRanges) to High(FRanges) do
          FRanges[j] := Joystick.Range[j];

        DeadZoneX := Joystick.DeadZoneX;
        DeadZoneY := Joystick.DeadZoneY;
        DeadZoneZ := Joystick.DeadZoneZ;

        RangeX := Joystick.RangeX;
        RangeY := Joystick.RangeY;
        RangeZ := Joystick.RangeZ;
        ForeGroundOnly := Joystick.ForegroundOnly;
        Exclusive := Joystick.Exclusive;
        Enabled := Joystick.Enabled;
      end;
    end;
    Tag := 0;
    
    for i := 0 to FJoysticks.Count - 1 do
      TJoystick(FJoysticks[i]).Initialize;
    //FJoystick.Initialize;
    FKeyboard.Acquire;
    FMouse.Acquire;
    for i := 0 to FJoysticks.Count - 1 do
          TJoystick(FJoysticks[i]).Acquire;
    //FJoystick.Acquire;
  end;
end;

Procedure TOmegaInput.Finalize;
var
  i: integer;
begin
  FInitialized := false;
  FKeyboard.Finalize;
  FMouse.Finalize;

  //delete all except Joystick 0
  while FJoysticks.Count > 1 do
  begin
    TJoystick(FJoysticks[FJoysticks.Count - 1]).Finalize;
    TJoystick(FJoysticks[FJoysticks.Count - 1]).Free;
  end;
  //FJoystick.Finalize;
  FDInput := nil;
end;

Procedure TOmegaInput.Update;
var
  i: integer;
begin
  Keyboard.Update;
  Mouse.Update;
  for i := 0 to FJoysticks.Count - 1 do
    TJoystick(FJoysticks[i]).Update;
  //Joystick.Update;
  if assigned(FOnKeyboard) and((Keyboard.States<>[]) or
   (Keyboard.StatesClicked<>[])) then FOnKeyboard(self);
  if assigned(FOnMouse) and((Mouse.States<>[]) or
   (Mouse.StatesClicked<>[])) then FOnMouse(self);
  if assigned(FOnJoystick) and((Joystick.States<>[]) or
   (Joystick.StatesClicked<>[])) then FOnJoystick(self);
end;

Procedure TOmegaInput.SetKeyboard(Value: TKeyboard);
begin
  FKeyboard.Assign(Value);
end;

Procedure TOmegaInput.SetMouse(Value: TMouse);
begin
  FMouse.Assign(Value);
end;

Procedure TOmegaInput.SetJoystick(Value: TJoystick);
begin
  Joystick.Assign(Value);
end;

Function TOmegaInput.GetJoystick(index: integer): TJoystick;
begin
  if (index >= 0) and (index < FJoysticks.Count) then
    Result := TJoystick(FJoysticks[index]);
end;

Function TOmegaInput.GetJoyCount: integer;
begin
  Result := FJoysticks.Count;
end;

end.
