unit OmegaInputEdit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, OmegaInput, DirectInput, ComCtrls, Spin;

type
  TOmegaInputEditForm = class(TForm)
    OKButton: TButton;
    CancelButton: TButton;
    PageControl1: TPageControl;
    tsKeyboard: TTabSheet;
    tsJoystick: TTabSheet;
    GroupBox1: TGroupBox;
    lStates: TLabel;
    lcKey1: TLabel;
    lbStates: TListBox;
    cbKey1: TComboBox;
    gbGeneral: TGroupBox;
    cbAutoCenter: TCheckBox;
    gbDeadZone: TGroupBox;
    gbRange: TGroupBox;
    edDeadZoneX: TSpinEdit;
    lDeadZoneX: TLabel;
    lDeadZoneY: TLabel;
    edDeadZoneY: TSpinEdit;
    lDeadZoneZ: TLabel;
    edDeadZoneZ: TSpinEdit;
    edRangeZ: TSpinEdit;
    lRangeZ: TLabel;
    lRangeY: TLabel;
    edRangeY: TSpinEdit;
    edRangeX: TSpinEdit;
    lRangeX: TLabel;
    Label1: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure lbStatesClick(Sender: TObject);
    procedure EditChange(Sender: TObject);
    procedure CancelButtonClick(Sender: TObject);
    procedure OKButtonClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    { Private declarations }
    FOldIndex: integer;
  public
    { Public declarations }
    FEnumIndex: integer;
    OmegaInput: TOmegaInput;
    KeyAssigns: TKeyAssigns;
    FChanged: boolean;
  end;

var
  OmegaInputEditForm: TOmegaInputEditForm;

implementation

{$R *.dfm}

Function KeyToStr(key: integer): string;
begin
  Result := '';
  case key of
    0             : Result := 'None';
    DIK_0         : Result := '0';
    DIK_1..DIK_9  : Result := char(key - DIK_1 + 1 + $30);
    DIK_A         : Result := 'A';
    DIK_ADD       : Result := 'NUM +';
    DIK_B         : Result := 'B';
    DIK_BACK      : Result := 'Backspace';
    DIK_C         : Result := 'C';
    DIK_CAPITAL   : Result := 'Caps Lock';
    DIK_COMMA     : Result := ',';
    DIK_D         : Result := 'D';
    DIK_DECIMAL   : Result := 'NUM *';
    DIK_DELETE    : Result := 'Delete';
    DIK_DIVIDE    : Result := 'NUM /';
    DIK_DOWN      : Result := 'Down Arrow';
    DIK_E         : Result := 'E';
    DIK_END       : Result := 'End';
    DIK_ESCAPE    : Result := 'Esc';
    DIK_F         : Result := 'F';
    DIK_F1        : Result := 'F1';
    DIK_F2        : Result := 'F2';
    DIK_F3        : Result := 'F3';
    DIK_F4        : Result := 'F4';
    DIK_F5        : Result := 'F5';
    DIK_F6        : Result := 'F6';
    DIK_F7        : Result := 'F7';
    DIK_F8        : Result := 'F8';
    DIK_F9        : Result := 'F9';
    DIK_F10       : Result := 'F10';
    DIK_F11       : Result := 'F11';
    DIK_F12       : Result := 'F12';
    DIK_F13       : Result := 'F13';
    DIK_F14       : Result := 'F14';
    DIK_F15       : Result := 'F15';
    DIK_G         : Result := 'G';
    DIK_H         : Result := 'H';
    DIK_HOME      : Result := 'Home';
    DIK_I         : Result := 'I';
    DIK_INSERT    : Result := 'Insert';
    DIK_J         : Result := 'J';
    DIK_K         : Result := 'K';
    DIK_L         : Result := 'L';
    DIK_LCONTROL  : Result := 'Left Ctrl';
    DIK_LEFT      : Result := 'Left Arrow';
    DIK_LMENU     : Result := 'Left Alt';
    DIK_LSHIFT    : Result := 'Left Shift';
    DIK_M         : Result := 'M';
    DIK_MINUS     : Result := '-';
    DIK_MULTIPLY  : Result := 'NUM *';
    DIK_N         : Result := 'N';
    DIK_NEXT      : Result := 'Page Down';
    DIK_NUMPAD0   : Result := 'NUM 0';    
    DIK_NUMPAD1   : Result := 'NUM 1';
    DIK_NUMPAD2   : Result := 'NUM 2';
    DIK_NUMPAD3   : Result := 'NUM 3';
    DIK_NUMPAD4   : Result := 'NUM 4';
    DIK_NUMPAD5   : Result := 'NUM 5';
    DIK_NUMPAD6   : Result := 'NUM 6';
    DIK_NUMPAD7   : Result := 'NUM 7';
    DIK_NUMPAD8   : Result := 'NUM 8';
    DIK_NUMPAD9   : Result := 'NUM 9';
    DIK_NUMPADENTER: Result := 'NUM Enter';
    DIK_O : Result := 'O';
    DIK_P : Result := 'P';
    DIK_PERIOD : Result := '.';
    DIK_PRIOR  : Result := 'Page Up';
    DIK_Q : Result := 'Q';
    DIK_R : Result := 'R';
    DIK_RCONTROL  : Result := 'Right Ctrl';
    DIK_RETURN  : Result := 'Enter';
    DIK_RIGHT  : Result := 'Right Arrow';
    DIK_RMENU  : Result := 'Right Alt';
    DIK_RSHIFT : Result := 'Right Shift';
    DIK_S  : Result := 'S';
    DIK_SLASH  : Result := '/';
    DIK_SPACE  : Result := 'Space';
    DIK_SUBTRACT : Result := 'NUM -';
    DIK_T        : Result := 'T';
    DIK_TAB     : Result := 'Tab';
    DIK_U      : Result := 'U';
    DIK_UP      : Result := 'Up Arrow';
    DIK_V      : Result := 'V';
    DIK_W      : Result := 'W';
    DIK_X     : Result := 'X';
    DIK_Y      : Result := 'Y';
    DIK_Z     : Result := 'Z';
  end;
end;

Procedure FillCB(cb: TComboBox);
begin
  cb.Clear;
  cb.Items.AddObject(KeyToStr(0), Pointer(0));
  cb.Items.AddObject(KeyToStr(DIK_UP), Pointer(DIK_UP));
  cb.Items.AddObject(KeyToStr(DIK_DOWN), Pointer(DIK_DOWN));
  cb.Items.AddObject(KeyToStr(DIK_LEFT), Pointer(DIK_LEFT));
  cb.Items.AddObject(KeyToStr(DIK_RIGHT), Pointer(DIK_RIGHT));
  cb.Items.AddObject(KeyToStr(DIK_LSHIFT), Pointer(DIK_LSHIFT));
  cb.Items.AddObject(KeyToStr(DIK_RSHIFT), Pointer(DIK_RSHIFT));
  cb.Items.AddObject(KeyToStr(DIK_LMENU), Pointer(DIK_LMENU));
  cb.Items.AddObject(KeyToStr(DIK_RMENU), Pointer(DIK_RMENU));
  cb.Items.AddObject(KeyToStr(DIK_LCONTROL), Pointer(DIK_LCONTROL));
  cb.Items.AddObject(KeyToStr(DIK_RCONTROL), Pointer(DIK_RCONTROL));
  cb.Items.AddObject(KeyToStr(DIK_A), Pointer(DIK_A));
  cb.Items.AddObject(KeyToStr(DIK_B), Pointer(DIK_B));
  cb.Items.AddObject(KeyToStr(DIK_C), Pointer(DIK_C));
  cb.Items.AddObject(KeyToStr(DIK_D), Pointer(DIK_D));
  cb.Items.AddObject(KeyToStr(DIK_E), Pointer(DIK_E));
  cb.Items.AddObject(KeyToStr(DIK_F), Pointer(DIK_F));
  cb.Items.AddObject(KeyToStr(DIK_G), Pointer(DIK_G));
  cb.Items.AddObject(KeyToStr(DIK_H), Pointer(DIK_H));
  cb.Items.AddObject(KeyToStr(DIK_I), Pointer(DIK_I));
  cb.Items.AddObject(KeyToStr(DIK_J), Pointer(DIK_J));
  cb.Items.AddObject(KeyToStr(DIK_K), Pointer(DIK_K));
  cb.Items.AddObject(KeyToStr(DIK_L), Pointer(DIK_L));
  cb.Items.AddObject(KeyToStr(DIK_M), Pointer(DIK_M));
  cb.Items.AddObject(KeyToStr(DIK_N), Pointer(DIK_N));
  cb.Items.AddObject(KeyToStr(DIK_O), Pointer(DIK_O));
  cb.Items.AddObject(KeyToStr(DIK_P), Pointer(DIK_P));
  cb.Items.AddObject(KeyToStr(DIK_Q), Pointer(DIK_Q));
  cb.Items.AddObject(KeyToStr(DIK_R), Pointer(DIK_R));
  cb.Items.AddObject(KeyToStr(DIK_S), Pointer(DIK_S));
  cb.Items.AddObject(KeyToStr(DIK_T), Pointer(DIK_T));
  cb.Items.AddObject(KeyToStr(DIK_U), Pointer(DIK_U));
  cb.Items.AddObject(KeyToStr(DIK_V), Pointer(DIK_V));
  cb.Items.AddObject(KeyToStr(DIK_W), Pointer(DIK_W));
  cb.Items.AddObject(KeyToStr(DIK_X), Pointer(DIK_X));
  cb.Items.AddObject(KeyToStr(DIK_Y), Pointer(DIK_Y));
  cb.Items.AddObject(KeyToStr(DIK_Z), Pointer(DIK_Z));
  cb.Items.AddObject(KeyToStr(DIK_NUMPAD0), Pointer(DIK_NUMPAD0));
  cb.Items.AddObject(KeyToStr(DIK_NUMPAD1), Pointer(DIK_NUMPAD1));
  cb.Items.AddObject(KeyToStr(DIK_NUMPAD2), Pointer(DIK_NUMPAD2));
  cb.Items.AddObject(KeyToStr(DIK_NUMPAD3), Pointer(DIK_NUMPAD3));
  cb.Items.AddObject(KeyToStr(DIK_NUMPAD4), Pointer(DIK_NUMPAD4));
  cb.Items.AddObject(KeyToStr(DIK_NUMPAD5), Pointer(DIK_NUMPAD5));
  cb.Items.AddObject(KeyToStr(DIK_NUMPAD6), Pointer(DIK_NUMPAD6));
  cb.Items.AddObject(KeyToStr(DIK_NUMPAD7), Pointer(DIK_NUMPAD7));
  cb.Items.AddObject(KeyToStr(DIK_NUMPAD8), Pointer(DIK_NUMPAD8));
  cb.Items.AddObject(KeyToStr(DIK_NUMPAD9), Pointer(DIK_NUMPAD9));
  cb.Items.AddObject(KeyToStr(DIK_SUBTRACT), Pointer(DIK_SUBTRACT));
  cb.Items.AddObject(KeyToStr(DIK_ADD), Pointer(DIK_ADD));
  cb.Items.AddObject(KeyToStr(DIK_DECIMAL), Pointer(DIK_DECIMAL));
  cb.Items.AddObject(KeyToStr(DIK_DIVIDE), Pointer(DIK_DIVIDE));
  cb.Items.AddObject(KeyToStr(DIK_NUMPADENTER), Pointer(DIK_NUMPADENTER));
end;

function StateToStr(state: TOmegaInputState): string;
begin
  Result := '';
  case State of
    oisUp: Result := 'Up';
    oisDown: Result := 'Down';
    oisRight: Result := 'Right';
    oisLeft: Result := 'Left';
    oisButton1..oisButton28: Result := 'Button ' + inttostr(ord(state)
                                                            - ord(oisButton1) + 1);
  end;
end;

procedure TOmegaInputEditForm.FormCreate(Sender: TObject);
var
  i: TOmegaInputState;
  i2: integer;
  i3: integer;
  s: string;
begin
  FOldIndex := -1;
  for i := Low(TOmegaInputState) to High(TOmegaInputState) do
    lbStates.Items.Add(StatetoStr(i));
  Fillcb(cbKey1);
  for i2 := 0 to 255 do
  begin
    s := KeyToStr(i2);
    if s <> '' then
    begin
      for i3 := 0 to cbKey1.Items.Count - 1 do
      begin
        if cbKey1.Items[i3] = s then Break;
      end;
      if i3 = cbKey1.Items.Count then cbKey1.Items.AddObject(s, Pointer(i2));
    end;
  end;
  FChanged := false;
end;

procedure TOmegaInputEditForm.lbStatesClick(Sender: TObject);
var
  KeyAssign: PKeyAssign;

Procedure SetIndex(cb: TComboBox; index: integer);
var
  i: integer;
begin
  for i := 0 to cb.Items.Count - 1 do
  begin
    if integer(cb.Items.Objects[i]) = index then
    begin
      cb.ItemIndex := i;
      Exit;
    end;
    cb.ItemIndex := 0;
  end;
end;

begin
  if FOldIndex > -1 then
  begin
    KeyAssign  := @KeyAssigns[TOmegaInputState(FOldIndex)];
    KeyAssign^ := Integer(cbKey1.Items.Objects[cbKey1.ItemIndex]);
  end;
  FOldIndex := lbStates.ItemIndex;

  KeyAssign := @KeyAssigns[TOmegaInputState(FOldIndex)];
  SetIndex(cbKey1, KeyAssign^);

end;

procedure TOmegaInputEditForm.EditChange(Sender: TObject);
begin
  FChanged := true;
end;

procedure TOmegaInputEditForm.CancelButtonClick(Sender: TObject);
begin
  Close;
end;

procedure TOmegaInputEditForm.OKButtonClick(Sender: TObject);
begin
  if FChanged then
  begin
    if lbStates.ItemIndex > -1 then
    begin
      FOldIndex := lbStates.ItemIndex;
      lbStatesClick(nil);
    end;
    OmegaInput.Keyboard.KeyAssigns := KeyAssigns;
    OmegaInput.Joystick.ID := 0;
    OmegaInput.Joystick.Autocenter := cbAutoCenter.Checked;
    OmegaInput.Joystick.RangeX := edRangeX.Value;
    OmegaInput.Joystick.RangeY := edRangeY.Value;
    OmegaInput.Joystick.RangeZ := edRangeZ.Value;
    OmegaInput.Joystick.DeadZoneX := edDeadZoneX.Value;
    OmegaInput.Joystick.DeadZoneY := edDeadZoneY.Value;
    OmegaInput.Joystick.DeadZoneZ := edDeadZoneZ.Value;
    Tag := 1;
  end;
  Close;
end;

procedure TOmegaInputEditForm.FormShow(Sender: TObject);
begin
  KeyAssigns := OmegaInput.Keyboard.KeyAssigns;
  cbAutoCenter.Checked := OmegaInput.Joystick.Autocenter;
  edRangeX.Value := OmegaInput.Joystick.RangeX;
  edRangeY.Value := OmegaInput.Joystick.RangeY;
  edRangeZ.Value := OmegaInput.Joystick.RangeZ;
  edDeadZoneX.Value := OmegaInput.Joystick.DeadZoneX;
  edDeadZoneY.Value := OmegaInput.Joystick.DeadZoneY;
  edDeadZoneZ.Value := OmegaInput.Joystick.DeadZoneZ;
end;

end.
