unit OmegaMultimedia;

interface

uses
  SysUtils,
  Classes,
  Forms,
  Dialogs,
  Math,
  Windows,
  Messages,
  ActiveX,
  DirectShow,    
  OmegaUtils;

type
  // not used
  TEventCode = (
    ecActivate,
    ecBufferingData,
    ecClockChanged,
    ecClockUnset,
    ecComplete,
    ecDeviceLost,
    ecDisplayChanged,
    ecEndOfSegment,
    ecErrorStillPlaying,
    ecErrorAbort,
    ecExtDeviceModeChange,
    ecFullscreenLost,
    ecGraphChanged,
    ecLengthChanged,
    ecNeedRestart,
    ecNotifyWindow,
    ecOLEEvent,
    ecOpeneningFile,
    ecPaletteChanged,
    ecPaused,
    ecQualityChange,
    ecRepaint,
    ecSegmentStarted,
    ecShuttingDown,
    ecSndDevInError,
    ecSndDevOutError,
    ecStarvation,
    ecStateChange,
    ecStepComplete,
    ecStreamControlStarted,
    ecStreamStrolStopped,
    ecStreamErrorStillPlaying,
    ecStreamErrorStopped,
    ecTimeCodeAvailable,
    ecUserAbort,
    ecVideoSizeChanged,
    ecWindowDestroyed,
    ecUnknown
  );

  TOnGraphEvent = procedure (Sender: TObject; const EvCode: Integer; const Param1, Param2: Integer) of object;

  TVideoItem = class;
  TAudioItem = class;
  TMediaItem = class;
  TMediaCollection = class;
  TOmegaMultimedia = class;

  TVideoItem = class(TPersistent)
  private
    FItem: TMediaItem;

    FWidth,
    FHeight: Integer;
    FLeft,
    FTop: Integer;
    FCaption: String;
    FVisible: Boolean;
    FFullscreen: Boolean; // not used
    FHideCursor: Boolean;

    FInitialized: Boolean;
    //FPlaying: Boolean;
  protected
    procedure SetWidth(Value: Integer);
    procedure SetHeight(Value: Integer);
    procedure SetLeft(Value: Integer);
    procedure SetTop(Value: Integer);
    procedure SetCaption(Value: String);
    procedure SetVisible(Value: Boolean);
    //procedure SetFullscreen(Value: Boolean);
    procedure SetHideCursor(Value: Boolean);

    function GetWidth: Integer;
    function GetHeight: Integer;
    function GetLeft: Integer;
    function GetTop: Integer;
    function GetCaption: String;
    function GetVisible: Boolean;
    //function GetFullscreen: Boolean;
    function GetHideCursor: Boolean;

    function GetBitRate: Integer;     // is it the same as rate? better keep it just to be safe ;)
    function GetAvgTimePerFrame: Double;
    function GetBitErrorRate: Integer;
    function GetOrgWidth: Integer;
    function GetOrgHeight: Integer;
  public
    constructor Create; overload;
    constructor Create(AItem: TMediaItem); overload;
    destructor Destroy; override;
    procedure Init;

    procedure StepFrames(const FramesToStep: Integer);

    property Initialized: Boolean read FInitialized;
    property MediaItem: TMediaItem read FItem write FItem;

    property OriginalWidth: Integer read GetOrgWidth;
    property OriginalHeight: Integer read GetOrgHeight;
    property BitRate: Integer read GetBitRate;
    property AvgTimePerFrame: Double read GetAvgTimePerFrame;
    property BitErrorRate: Integer read GetBitErrorRate;
  published
    property Width: Integer read GetWidth write SetWidth;
    property Height: Integer read GetHeight write SetHeight;
    property Left: Integer read GetLeft write SetLeft;
    property Top: Integer read GetTop write SetTop;
    property Caption: String read GetCaption write SetCaption;
    property Visible: Boolean read GetVisible write SetVisible default False;
    //property Fullscreen: Boolean read GetFullscreen write SetFullscreen default False;
    property HideCursor: Boolean read GetHideCursor write SetHideCursor;
  end;

  TAudioVolume = (avOmega, avDirectX);

  TAudioItem = class(TPersistent)
  private
    FItem: TMediaItem;

    FBalance: Integer;
    FOmegaVolume: Integer;
    FDirectXVolume: Integer;
    FInitialized: Boolean;

    FAudioVolume: TAudioVolume;
  protected
    procedure SetBalance(Value: Integer);
    procedure SetOmegaVolume(Value: Integer);
    procedure SetDirectXVolume(Value: Integer); 

    function GetBalance: Integer;
    function GetOmegaVolume: Integer;
    function GetDirectXVolume: Integer;

    procedure SetAudioVolume(Value: TAudioVolume);
    function GetAudioVolume: TAudioVolume;
  public
    constructor Create; overload;
    constructor Create(AItem: TMediaItem); overload;
    procedure Init;

    property Initialized: Boolean read FInitialized;
    property MediaItem: TMediaItem read FItem write FItem;
  published
    property OmegaVolume: Integer read GetOmegaVolume write SetOmegaVolume default 100;
    property DirectXVolume: Integer read GetDirectXVolume write SetDirectXVolume default 0;
    property Balance: Integer read FBalance write SetBalance default 0;
    property AudioVolume: TAudioVolume read GetAudioVolume write SetAudioVolume;
  end;

  TMediaItem = class(TCollectionItem)
  private
    FPosition: Double;
    FRate: Double;
    FInitialized: Boolean;
    FAudioOnly: Boolean;

    FLoaded: Boolean;
    FPaused: Boolean;
    FPlaying: Boolean;
    FCanStep: Boolean;

    FFileName: String;
    FName: String;
    FLoop: Boolean;

    FGraphBuilder: IGraphBuilder;
    FMediaPosition: IMediaPosition;
    FMediaControl: IMediaControl;
    FBasicAudio: IBasicAudio;
    FMediaEvent: IMediaEvent;
    FMediaSeeking: IMediaSeeking;
    FVideoWindow: IVideoWindow;
    FVideoFrameStep: IVideoFrameStep;
    FBasicVideo: IBasicVideo;
    //FBasicVideo2: IBasicVideo2;

    FVideoItem: TVideoItem;
    FAudioItem: TAudioItem;

    FOnComplete: TNotifyEvent;
    FOnGraphEvent: TOnGraphEvent;

    function ProtectedIsPlaying: Boolean;
  protected
    procedure CleanUpDirectShow;
    procedure InitDirectShow;

    procedure SetPosition(Value: Double);
    procedure SetRate(Value: Double);
    
    function GetPosition: Double;
    function GetDuration: Double;
    function GetRate: Double;

    procedure Render;
    function GetDisplayName: String; override;
  public
    property Position: Double read GetPosition write SetPosition;
    property Duration: Double read GetDuration;
    property Initialized: Boolean read FInitialized;
    property AudioOnly: Boolean read FAudioOnly;
    property Rate: Double read GetRate write SetRate;

    property GraphBuilder  : IGraphBuilder  read FGraphBuilder  write FGraphBuilder;
    property MediaPosition : IMediaPosition read FMediaPosition write FMediaPosition;
    property MediaControl  : IMediaControl  read FMediaControl  write FMediaControl;
    property BasicAudio    : IBasicAudio    read FBasicAudio    write FBasicAudio;
    property MediaEvent: IMediaEvent read FMediaEvent write FMediaEvent;
    property MediaSeeking: IMediaSeeking read FMediaSeeking write FMediaSeeking;
    property VideoWindow: IVideoWindow read FVideoWindow write FVideoWindow;
    property VideoFrameStep: IVideoFrameStep read FVideoFrameStep write FVideoFrameStep;
    property BasicVideo: IBasicVideo read FBasicVideo write FBasicVideo;
    //property BasicVideo2: IBasicVideo2 read FBasicVideo2 write FBasicVideo2;

    constructor Create(Collection: TCollection); override;
    destructor Destroy; override;
    procedure Load; overload;
    procedure Load(const FileName: TFileName); overload;
    function IsPlaying: Boolean;
    procedure Play;
    procedure Stop;
    procedure Rewind;
    procedure Pause;
    procedure Init;
    procedure SethWnd(h_Wnd: HWND);
  published
    property FileName: String read FFileName write FFilename;
    property Name: String read FName write FName;
    property Loop: Boolean read FLoop write FLoop;
    property Video: TVideoItem read FVideoItem write FVideoItem;
    property Audio: TAudioItem read FAudioItem write FAudioItem;

    property OnComplete: TNotifyEvent read FOnComplete write FOnComplete;
    property OnGraphEvent: TOnGraphEvent read FOnGraphEvent write FOnGraphEvent;
  end;

  TMediaCollection = class(TOwnedCollection)
  private
    procedure Render;
  protected
    function GetItem(Index: Integer): TMediaItem;
    procedure SetItem(Index: Integer; Value: TMediaItem);
    procedure Update(Item: TCollectionItem); override;
  public
    constructor Create(AOwner: TComponent);
    destructor Destroy; override;
    function Add: TMediaItem;
    function Insert(Index: Integer): TMediaItem;
    function Find(AName: String): TMediaItem;
    procedure LoadFromFile(const FileName: TFileName);
    procedure LoadFromStream(Stream: TStream);
    procedure SaveToFile(const FileName: TFileName);
    procedure SaveToStream(Stream: TStream);
    
    property Items[Index: Integer]: TMediaItem read GetItem write SetItem;
  published
  end;

  TOmegaMultimedia = class(TComponent)
  private
    FMediaList: TMediaCollection;
    procedure SetMediaList(const Value: TMediaCollection);
  protected
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Init;
    procedure Render;
  published
    property Media: TMediaCollection read FMediaList write SetMediaList;
  end;

  procedure Register;
  function EvCodeInfo(const EvCode: Integer): String;

implementation

//-----------------------------------------------------
// Common
//-----------------------------------------------------
procedure Register;
begin
  RegisterComponents('Omega', [TOmegaMultimedia]);
end;

//-----------------------------------------------------
function EvCodeInfo(const EvCode: Integer): String;
begin
  // These are directly from the DirectX 8.1 SDK
  case EvCode of
    EC_ACTIVATE: Result := 'A video window is being activated or deactivated.';
    EC_BUFFERING_DATA: Result := 'The graph is buffering data, or has stopped buffering data.';
    EC_CLOCK_CHANGED: Result := 'The reference clock has changed.';
    EC_CLOCK_UNSET: Result := 'The clock provider was disconnected.';
    EC_COMPLETE: Result := 'All data from a particular stream has been rendered.';
    EC_DEVICE_LOST: Result := 'A Plug and Play device was removed or has become available again.';
    EC_DISPLAY_CHANGED: Result := 'The display mode has changed.';
    EC_END_OF_SEGMENT: Result := 'The end of a segment has been reached.';
    EC_ERROR_STILLPLAYING: Result := 'An asynchronous command to run the graph has failed.';
    EC_ERRORABORT: Result := 'An operation was aborted because of an error.';
    EC_EXTDEVICE_MODE_CHANGE: Result := 'Not supported.';
    EC_FULLSCREEN_LOST: Result := 'The video renderer is switching out of full-screen mode.';
    EC_GRAPH_CHANGED: Result := 'The filter graph has changed.';
    EC_LENGTH_CHANGED: Result := 'The length of a source has changed.';
    EC_NEED_RESTART: Result := 'A filter is requesting that the graph be restarted.';
    EC_NOTIFY_WINDOW: Result := 'Notifies a filter of the video renderer''s window.';
    EC_OLE_EVENT: Result := 'A filter is passing a text string to the application.';
    EC_OPENING_FILE: Result := 'The graph is opening a file, or has finished opening a file.';
    EC_PALETTE_CHANGED: Result := 'The video palette has changed.';
    EC_PAUSED: Result := 'A pause request has completed.';
    EC_QUALITY_CHANGE: Result := 'The graph is dropping samples, for quality control.';
    EC_REPAINT: Result := 'A video renderer requires a repaint.';
    EC_SEGMENT_STARTED: Result := 'A new segment has started.';
    EC_SHUTTING_DOWN: Result := 'The filter graph is shutting down, prior to being destroyed.';
    EC_SNDDEV_IN_ERROR: Result := 'An audio device error has occurred on an input pin.';
    EC_SNDDEV_OUT_ERROR: Result := 'An audio device error has occurred on an output pin.';
    EC_STARVATION: Result := 'A filter is not receiving enough data.';
    EC_STATE_CHANGE: Result := 'The filter graph has changed state.';
    EC_STEP_COMPLETE: Result := 'A filter performing frame stepping has stepped the specified number of frames.';
    EC_STREAM_CONTROL_STARTED: Result := 'A stream-control start command has taken effect.';
    EC_STREAM_CONTROL_STOPPED: Result := 'A stream-control start command has taken effect.'; // shouldn't it be "stop command"?
    EC_STREAM_ERROR_STILLPLAYING: Result := 'An error has occurred in a stream. The stream is still playing.';
    EC_STREAM_ERROR_STOPPED: Result := 'A stream has stopped because of an error.';
    EC_TIMECODE_AVAILABLE: Result := 'Not supported.';
    EC_USERABORT: Result := 'The user has terminated playback.';
    EC_VIDEO_SIZE_CHANGED: Result := 'The native video size has changed.';
    EC_WINDOW_DESTROYED: Result := 'The video renderer was destroyed or removed from the graph.'
  else
    Result := 'Unknown Event Code';
  end;
end;

//-----------------------------------------------------
// TVideoItem
//-----------------------------------------------------
procedure TVideoItem.SetWidth(Value: Integer);
begin
  if not FFullscreen then
  begin
    FWidth := Value;
    if FItem.FVideoWindow = nil then Exit;
    if Failed(FItem.FVideoWindow.put_Width(FWidth)) then
       raise Exception.Create('Error setting Width');
  end;
end;

//-----------------------------------------------------
procedure TVideoItem.SetHeight(Value: Integer);
begin
  if not FFullscreen then
  begin
    FHeight := Value;
    if FItem.FVideoWindow = nil then Exit;
    if Failed(FItem.FVideoWindow.put_Height(FHeight)) then
       raise Exception.Create('Error setting Height');
  end;
end;

//-----------------------------------------------------
procedure TVideoItem.SetLeft(Value: Integer);
begin
  if not FFullscreen then
  begin
    FLeft := Value;
    if FItem.FVideoWindow = nil then Exit;
    if Failed(FItem.FVideoWindow.put_Left(FLeft)) then
       raise Exception.Create('Error setting Left');
  end;
end;

//-----------------------------------------------------
procedure TVideoItem.SetTop(Value: Integer);
begin
  if not FFullscreen then
  begin
    FTop := Value;
    if FItem.FVideoWindow = nil then Exit;
    if Failed(FItem.FVideoWindow.put_Top(FTop)) then
       raise Exception.Create('Error setting Top');
  end;
end;

//-----------------------------------------------------
procedure TVideoItem.SetCaption(Value: String);
begin
  if not FFullscreen then
  begin
    FCaption := Value;
    if FItem.FVideoWindow = nil then Exit;
    if Value = '' then Exit;
    if Failed(FItem.FVideoWindow.put_Caption(FCaption)) then
       raise Exception.Create('Error setting Caption');
  end;
end;

//-----------------------------------------------------
procedure TVideoItem.SetVisible(Value: Boolean);
begin
  if not FFullscreen then
  begin
    FVisible := Value;
    if FItem.FVideoWindow = nil then Exit;
    if Failed(FItem.FVideoWindow.put_Visible(FVisible)) then
       raise Exception.Create('Error setting Visible');
  end;
end;

{//-----------------------------------------------------
procedure TVideoItem.SetFullscreen(Value: Boolean);
begin
  FFullscreen := Value;
  if FItem.FVideoWindow = nil then Exit;
  if Failed(FItem.FVideoWindow.put_FullScreenMode(FFullscreen))then
     raise Exception.Create('Error setting Fullscreen');
end;  }

//-----------------------------------------------------
procedure TVideoItem.SetHideCursor(Value: Boolean);
begin
  FHideCursor := Value;
  if FItem.FVideoWindow = nil then Exit;
  if Failed(FItem.FVideoWindow.HideCursor(Value)) then
     raise Exception.Create('Error setting HideCursor');
end;

//-----------------------------------------------------
procedure TVideoItem.StepFrames(const FramesToStep: Integer);
begin
  if not FItem.FCanStep then Exit;
  if FItem.FVideoFrameStep = nil then Exit;
  if Succeeded(FItem.FVideoFrameStep.CanStep(FramesToStep, nil)) then
  begin
    // graph must be paused for frame stepping to work
    if not FItem.FPaused then FItem.Pause;
    if Failed(FItem.FVideoFrameStep.Step(FramesToStep, nil)) then
       raise Exception.Create('Error stepping ' + IntToStr(FramesToStep) + ' number of frames');
  end;
end;

//-----------------------------------------------------
function TVideoItem.GetWidth: Integer;
begin
  Result := FWidth;
  if FItem.FVideoWindow = nil then Exit;
  if Failed(FItem.FVideoWindow.get_Width(Result)) then
     raise Exception.Create('Error getting Width');
end;

//-----------------------------------------------------
function TVideoItem.GetHeight: Integer;
begin
  Result := FHeight;
  if FItem.FVideoWindow = nil then Exit;
  if Failed(FItem.FVideoWindow.get_Height(Result)) then
     raise Exception.Create('Error getting Height');
end;

//-----------------------------------------------------
function TVideoItem.GetLeft: Integer;
begin
  Result := FLeft;
  if FItem.FVideoWindow = nil then Exit;
  if Failed(FItem.FVideoWindow.get_Left(Result)) then
     raise Exception.Create('Error getting Left');
end;

//-----------------------------------------------------
function TVideoItem.GetTop: Integer;
begin
  Result := FTop;
  if FItem.FVideoWindow = nil then Exit;
  if Failed(FItem.FVideoWindow.get_Top(Result)) then
     raise Exception.Create('Error getting Top');
end;

//-----------------------------------------------------
function TVideoItem.GetCaption: String;
var
  temp: WideString;
begin
  Result := FCaption;
  if FItem.FVideoWindow = nil then Exit;
  if Failed(FItem.FVideoWindow.get_Caption(temp)) then
     raise Exception.Create('Error getting Caption');
  Result := temp;
end;

//-----------------------------------------------------
function TVideoItem.GetVisible: Boolean;
var
  temp: LongBool;
begin
  Result := FVisible;
  if FItem.FVideoWindow = nil then Exit;
  if Failed(FItem.FVideoWindow.get_Visible(temp)) then
     raise Exception.Create('Error getting Visible');
  Result := temp;
end;

//-----------------------------------------------------
function TVideoItem.GetHideCursor: Boolean;
var
  temp: LongBool;
begin
  Result := FHideCursor;
  if FItem.FVideoWindow = nil then Exit;
  if Failed(FItem.FVideoWindow.IsCursorHidden(temp)) then
     raise Exception.Create('Error getting IsCursorHidden');
  Result := temp;
end;

//-----------------------------------------------------
function TVideoItem.GetBitRate: Integer;
begin
  Result := 0;
  if FItem.FBasicVideo = nil then Exit;
  if Failed(Fitem.FBasicVideo.get_BitRate(Result)) then
     raise Exception.Create('Error getting BitRate');
end;

//-----------------------------------------------------
function TVideoItem.GetAvgTimePerFrame: Double;
begin
  Result := 0;
  if FItem.FBasicVideo = nil then Exit;
  if Failed(FItem.FBasicVideo.get_AvgTimePerFrame(Result)) then
     raise Exception.Create('Error getting average time per frame');
end;

//-----------------------------------------------------
function TVideoItem.GetBitErrorRate: Integer;
begin
  Result := 0;
  if Fitem.FBasicVideo = nil then Exit;
  if Failed(FItem.FBasicVideo.get_BitErrorRate(Result)) then
     raise Exception.Create('Error getting BitErrorRate');
end;

//-----------------------------------------------------
function TVideoItem.GetOrgWidth: Integer;
var
  temp: Integer;
begin
  Result := 0;
  if FItem.FBasicVideo = nil then Exit;
  if Failed(FItem.FBasicVideo.GetVideoSize(Result, temp)) then
     raise Exception.Create('Error getting Original Width');
end;

//-----------------------------------------------------
function TVideoItem.GetOrgHeight: Integer;
var
  temp: Integer;
begin
  Result := 0;
  if FItem.FBasicVideo = nil then Exit;
  if Failed(FItem.FBasicVideo.GetVideoSize(temp, Result)) then
     raise Exception.Create('Error getting Original Height');
end;

//-----------------------------------------------------
constructor TVideoItem.Create;
begin
  inherited;
  FInitialized := False;
  FFullscreen := False;
  FVisible := False;
end;

//-----------------------------------------------------
constructor TVideoItem.Create(AItem: TMediaItem);
begin
  inherited Create;
  FInitialized := False;
  FFullscreen := False;
  FVisible := False;
  FItem := AItem;
end;

//-----------------------------------------------------
destructor TVideoItem.Destroy;
begin
  inherited;
end;

//-----------------------------------------------------
procedure TVideoItem.Init;
begin
  SetLeft(FLeft);
  SetTop(FTop);
  SetWidth(FWidth);
  SetHeight(FHeight);
  SetCaption(FCaption);
  SetVisible(FVisible);
  //SetFullscreen(FFullscreen);
  SetHideCursor(FHideCursor);

  FInitialized := True;
end;

//-----------------------------------------------------
// TAudioItem
//-----------------------------------------------------

// this function handles the difference between the way Omega handles volume
// and the way DirectX handles volume
function OmegaVolumeToDirectXVolume(AVolume: Integer): Integer;
var
  t1,t2,t3: Single;
begin
  t1 := 0; 

  if AVolume <> 0 then
  begin
    t3 := Log10(AVolume);
    t1 := 20 * t3;
  end;

  t2 := t1 / 40;
  t2 := t2 * 10000;
  t2 := t2 - 10000;

  Result := RoundNormal(t2);
end;

// this function handles the difference between the way Omega handles volume
// and the way DirectX handles volume
// not used, eventhough I spent alot of time on it =(
function DirectXVolumeToOmegaVolume(AVolume: Integer): Integer;
var
  t1,t2,t3: Single;
begin
  t2 := AVolume;

  t2 := t2 + 10000;
  t2 := t2 / 10000;
  t1 := t2 * 40;

  t3 := t1 / 20;
  t3 := t3 * ln(10);
  t3 := exp(t3);
  Result := RoundNormal(t3);
end;

//-----------------------------------------------------
constructor TAudioItem.Create;
begin
  inherited;
  FBalance := 0;
  FOmegaVolume := 100;
  FDirectXVolume := 0;
  FInitialized := False;
end;

//-----------------------------------------------------
constructor TAudioItem.Create(AItem: TMediaItem);
begin
  inherited Create;
  FBalance := 0;
  FOmegaVolume := 100;
  FDirectXVolume := 0;
  FInitialized := False;
  FItem := AItem;
end;

//-----------------------------------------------------
procedure TAudioItem.SetBalance(Value: Integer);
begin
  if Value > 10000 then Value := 10000;
  if Value < -10000 then Value := -10000;
  FBalance := Value;

  if FItem.FBasicAudio <> nil then
     if Failed(FItem.FBasicAudio.put_Balance(Value)) then
        raise Exception.Create('Error setting Balace');
end;

//-----------------------------------------------------
procedure TAudioItem.SetOmegaVolume(Value: Integer);
begin
  if Value > 100 then Value := 100;
  if Value < 0 then Value := 0;
  FOmegaVolume := Value;

  if FAudioVolume = avOmega then
  begin
    if FItem.FBasicAudio <> nil then
       if Failed(FItem.FBasicAudio.put_Volume(OmegaVolumeToDirectXVolume(Value))) then
          raise Exception.Create('Error setting Volume');
  end;
end;

//-----------------------------------------------------
procedure TAudioItem.SetDirectXVolume(Value: Integer);
begin
  if Value > 0 then Value := 0;
  if Value < -10000 then Value := -10000;
  FDirectXVolume := Value;

  if FAudioVolume = avDirectX then
  begin
    if FItem.FBasicAudio <> nil then
       if Failed(FItem.FBasicAudio.put_Volume(Value)) then
          raise Exception.Create('Error setting Volume');
  end;
end;

//-----------------------------------------------------
function TAudioItem.GetBalance: Integer;
begin
  Result := FBalance;
  if FItem.FBasicAudio <> nil then
     if Failed(FItem.FBasicAudio.get_Balance(Result)) then
        raise Exception.Create('Error getting Balance');
end;

//-----------------------------------------------------
function TAudioItem.GetOmegaVolume: Integer;
begin
  Result := FOmegaVolume;
end;

//-----------------------------------------------------
function TAudioItem.GetDirectXVolume: Integer;
begin
  Result := FDirectXVolume;

{  if FItem.FBasicAudio <> nil then
     if Failed(FItem.FBasicAudio.get_Volume(Result)) then
        raise Exception.Create('Error getting Volume'); }
end;

//-----------------------------------------------------
procedure TAudioItem.Init;
begin
  if not FItem.FLoaded then
  begin
    SetOmegaVolume(FOmegaVolume);
    SetDirectXVolume(FDirectXVolume); 
    SetBalance(FBalance);
  end;
  FInitialized := True;
end;  

//-----------------------------------------------------
procedure TAudioItem.SetAudioVolume(Value: TAudioVolume);
begin
  if Value <> FAudioVolume then
  begin
    FAudioVolume := Value;

    {case FAudioVolume of
      avOmega:
      begin
        FVolume := DirectXVolumeToOmegaVolume(FVolume)
      end;

      avDirectX:
      begin
        FVolume := OmegaVolumeToDirectXVolume(FVolume);
      end;
    end;}
  end;
end;

//-----------------------------------------------------
function TAudioItem.GetAudioVolume: TAudioVolume;
begin
  Result := FAudioVolume;
end;

//-----------------------------------------------------
// TMediaItem
//-----------------------------------------------------
constructor TMediaItem.Create(Collection: TCollection);
begin
  inherited;

  FName := 'Media'+IntToStr(Collection.Count);
  FPosition := 0.0;
  FInitialized := False;
  FPlaying := False;
  FPaused := False;

  FVideoItem := TVideoItem.Create(self);
  FAudioItem := TAudioItem.Create(self);
end;

//-----------------------------------------------------
destructor TMediaItem.Destroy;
begin
  FAudioItem.Free;
  FVideoItem.Free;
  CleanUpDirectShow;
  inherited;
end;

//-----------------------------------------------------
procedure TMediaItem.CleanUpDirectShow;
begin
  if FBasicVideo <> nil then FBasicVideo := nil;
  //if FBasicVideo2 <> nil then FBasicVideo2 := nil;
  if FBasicAudio <> nil then FBasicAudio := nil;
  if FMediaEvent <> nil then FMediaEvent := nil;
  if FMediaSeeking <> nil then FMediaSeeking := nil;
  if FMediaPosition <> nil then FMediaPosition := nil;
  if FMediaControl <> nil then FMediaControl := nil;
  if FGraphBuilder <> nil then FGraphBuilder := nil;
end;

//-----------------------------------------------------
procedure TMediaItem.InitDirectShow;
begin
  if Failed(CoCreateInstance(CLSID_FilterGraph, nil, CLSCTX_INPROC, IID_IGraphBuilder, FGraphBuilder)) then
     raise Exception.Create('Failed to initialise DirectX Audio. DirectX 8 may not be installed');

  if Failed(CoCreateInstance(CLSID_FilterGraph, nil, CLSCTX_INPROC, IID_IGraphBuilder, FGraphBuilder)) then
     raise Exception.Create('Failed to initialise DirectShow. DirectX 8 may not be installed');
  if Failed(FGraphBuilder.QueryInterface(IID_IMediaControl, FMediaControl)) then
     raise Exception.Create('Failed to initialise DirectShow. DirectX 8 may not be installed');
  if Failed(FGraphBuilder.QueryInterface(IID_IMediaPosition, FMediaPosition)) then
     raise Exception.Create('Failed to initialise DirectShow. DirectX 8 may not be installed');
  if Failed(FGraphBuilder.QueryInterface(IID_IMediaSeeking, FMediaSeeking)) then
     raise Exception.Create('Failed to initialise DirectShow. DirectX 8 may not be installed');
  if Failed(FGraphBuilder.QueryInterface(IID_IMediaEvent, FMediaEvent)) then
     raise Exception.Create('Failed to initialise DirectShow. DirectX 8 may not be installed');
  if Failed(FGraphBuilder.QueryInterface(IID_IBasicAudio, FBasicAudio)) then
     raise Exception.Create('Failed to initialise DirectShow. DirectX 8 may not be installed');
  if Failed(FGraphBuilder.QueryInterface(IID_IBasicVideo, FBasicVideo)) then
     raise Exception.Create('Failed to initialise DirectShow. DirectX 8 may not be installed');

// Toby's addition - start

  {if Failed(FGraphBuilder.QueryInterface(IID_IBasicVideo2, FBasicVideo2)) then
     raise Exception.Create('Failed to initialise DirectShow. DirectX 8 may not be installed'}
  if Failed(FGraphBuilder.QueryInterface(IID_IVideoFrameStep, FVideoFrameStep)) then
   FCanStep := false
 else FCanStep := FVideoFrameStep.Step(0, nil) = S_OK;
//     raise Exception.Create('Failed to initialise DirectShow. DirectX 8 may not be installed');
  if Failed(FGraphBuilder.QueryInterface(IID_IVideoWindow, FVideoWindow)) then
     raise Exception.Create('Failed to initialise DirectShow. DirectX 8 may not be installed');

// Toby's addition - end

end;

//-----------------------------------------------------
procedure TMediaItem.SetPosition(Value: Double);
begin
  if FMediaPosition <> nil then
     if Failed(FMediaPosition.put_CurrentPosition(Value)) then
        raise Exception.Create('Error setting position');
end;

//-----------------------------------------------------
procedure TMediaItem.SetRate(Value: Double);
begin
  FRate := Value;
  if FMediaPosition = nil then Exit;
  if Failed(FMediaPosition.put_Rate(Value)) then
     raise Exception.Create('Error setting rate');
end;
//-----------------------------------------------------
function TMediaItem.GetPosition: Double;
begin
  Result := 0.0;
  if FMediaPosition <> nil then
     if Failed(FMediaPosition.get_CurrentPosition(Result)) then
        raise Exception.Create('Error getting position');
end;

//-----------------------------------------------------
function TMediaItem.GetDuration: Double;
begin
  Result := 0.0;
  if FMediaPosition <> nil then
     if Failed(FMediaPosition.get_Duration(Result)) then
        raise Exception.Create('Error getting duration');
end;

//-----------------------------------------------------
function TMediaItem.GetRate: Double;
begin
  Result := FRate;
  if FMediaPosition = nil then Exit;
  if Failed(FMediaPosition.get_Rate(Result)) then
     raise Exception.Create('Error getting Rate');
end;

//-----------------------------------------------------
function TMediaItem.ProtectedIsPlaying: Boolean;
var
  refCurrent, refDuration: TREFTIME;
begin
  Result := False;

  refCurrent := GetPosition;
  refDuration := GetDuration;
  if (refCurrent < refDuration) {and (refCurrent <> 0)} then Result := True;
end;

//-----------------------------------------------------
function TMediaItem.IsPlaying: Boolean;
begin
  Result := FPlaying;
end;

//-----------------------------------------------------
procedure TMediaItem.Render;
var
  EvCode, Param1, Param2: Integer;
  hr: HRESULT;
begin
  if not ProtectedIsPlaying then
  begin
    if Assigned(FOnComplete) then
       FOnComplete(self);
    if FLoop then
       Rewind;
  end;

  hr := FMediaEvent.GetEvent(EvCode, Param1, Param2, 0);
  if Succeeded(hr) then
  begin
    FMediaEvent.FreeEventParams(EvCode, Param1, Param2);
    if Assigned(FOnGraphEvent) then FOnGraphEvent(self, EvCode, Param1, Param2);
  end;
end;

//-----------------------------------------------------
function TMediaItem.GetDisplayName: String;
begin
  Result := FName;
end;

//-----------------------------------------------------
procedure TMediaItem.Play;
begin
  if (FMediaControl <> nil) and (FMediaPosition <> nil) then
  begin
    Rewind;
    if Failed(FMediaControl.Run) then
       raise Exception.Create('Error playing media');
    FPlaying := True;
  end;
end;

//-----------------------------------------------------
procedure TMediaItem.Stop;
begin
  if (FMediaControl <> nil) and (FMediaPosition <> nil) then
  begin
    if Failed(FMediaControl.Stop) then
       raise Exception.Create('Error stopping media');
    Rewind;
    FPlaying := False;
  end;
end;

//-----------------------------------------------------
procedure TMediaItem.Rewind;
begin
  SetPosition(0.0);
end;

//-----------------------------------------------------
procedure TMediaItem.Pause;
begin
  if FPaused then
  begin
    if Failed(FMediaControl.Run) then
       raise Exception.Create('Error playing media');
    FPaused := False;
    FPlaying := True;
  end
  else
  begin
    if Failed(FMediaControl.Pause) then
       raise Exception.Create('Error pausing media');
    FPaused := True;
    FPlaying := False;
  end;
end;

//-----------------------------------------------------
procedure TMediaItem.Load;
begin
  Load(FFileName);
end;

//-----------------------------------------------------
procedure TMediaItem.Load(const FileName: TFileName);
var
  hr: HRESULT;
  lVisible: LongBool;
begin
  if Filename = '' then Exit;

  CleanUpDirectShow;
  InitDirectShow;
  FAudioOnly := False;

  if Failed(FGraphBuilder.RenderFile(PWideChar(WideString(Filename)), nil)) then
  begin
    raise Exception.Create('Error loading file: ' + FileName);
    Exit;
  end;

  if (not Assigned(FVideoWindow)) or (not Assigned(BasicVideo)) then
     FAudioOnly := True
  else
  begin
    hr := FVideoWindow.get_Visible(lVisible);
    if Failed(hr) then
    begin
      if hr = E_NOINTERFACE then
         FAudioOnly := True
      else
        raise Exception.Create('Error getting Visible');
    end;
  end; 

  FLoaded := True;
// Toby's addition - start
  if not FAudioOnly then FVideoItem.Init;
  FAudioItem.Init;
// Toby's addition - end
end;

//-----------------------------------------------------
procedure TMediaItem.SethWnd(h_Wnd: HWND);
begin
  if FVideoWindow = nil then Exit;
  if Failed(FVideoWindow.put_Owner(h_Wnd))then
     raise Exception.Create('Error setting hWnd');
end;

//-----------------------------------------------------
procedure TMediaItem.Init;
begin
  if not FLoaded then
  begin     
    Load(FFileName);
// Toby's addition - start
   { if not FAudioOnly then
       FVideoItem.Init;
    FAudioItem.Init;}
// Toby's addition - end
  end;
  FInitialized := True;
end;

//-----------------------------------------------------
// TSongListCollection
//-----------------------------------------------------

type
  // just to make saveing/loading easier
  TMediaCollectionComponent = class(TComponent)
  private
    FList: TMediaCollection;
  public
    property List: TMediaCollection read FList write FList;
  end;

//-----------------------------------------------------
constructor TMediaCollection.Create(AOwner: TComponent);
begin
  inherited Create(AOwner, TMediaItem);
  if Failed(CoInitialize(nil)) then raise Exception.Create('Failed to initialise COM');
end;

//-----------------------------------------------------
destructor TMediaCollection.Destroy;
begin
  CoUninitialize;
  inherited;
end;

//-----------------------------------------------------
function TMediaCollection.GetItem(Index: Integer): TMediaItem;
begin
  Result := TMediaItem(inherited GetItem(Index));
end;

//-----------------------------------------------------
procedure TMediaCollection.SetItem(Index: Integer; Value: TMediaItem);
begin
  inherited SetItem(Index, Value);
end;

//-----------------------------------------------------
procedure TMediaCollection.Update(Item: TCollectionItem);
begin
  inherited Update(Item);
end;

//-----------------------------------------------------
function TMediaCollection.Add: TMediaItem;
begin
  Result := TMediaItem(inherited Add);
end;

//-----------------------------------------------------
function TMediaCollection.Insert(Index: Integer): TMediaItem;
begin
  Result := TMediaItem(inherited Insert(Index));
end;

//-----------------------------------------------------
function TMediaCollection.Find(AName: String): TMediaItem;
var
  i: Integer;
begin
  Result := nil;
  for i := 0 to Count - 1 do
  begin
    if AName = Items[i].FName then
    begin
      Result := Items[i];
      Exit;
    end;
  end;
end;

//-----------------------------------------------------
procedure TMediaCollection.Render;
var
  i: Integer;
begin
  for i := 0 to Count - 1 do
  begin
    //if Items[i].FLoop then
       Items[i].Render;
  end;
end;

//-----------------------------------------------------
procedure TMediaCollection.LoadFromFile(const FileName: TFileName);
var
  fs: TFileStream;
begin
  fs := TFileStream.Create(FileName, fmOpenRead or fmShareDenyWrite);
  try
    LoadFromStream(fs);
  finally
    fs.Free;
  end;
end;

//-----------------------------------------------------
procedure TMediaCollection.LoadFromStream(Stream: TStream);
var
  Component: TMediaCollectionComponent;
begin
  Clear;
  Component := TMediaCollectionComponent.Create(nil);
  try
    try
      Component.FList := self;
      Stream.ReadComponentRes(Component)
    finally
      Component.Free;
    end;
  except
    raise Exception.Create('Could not load TMediaCollection');
  end;
end;

//-----------------------------------------------------
procedure TMediaCollection.SaveToFile(const FileName: TFileName);
var
  fs: TFileStream;
begin
  fs := TFileStream.Create(FileName, fmCreate);
  try
    SaveToStream(fs);
  finally
    fs.Free;
  end;
end;

//-----------------------------------------------------
procedure TMediaCollection.SaveToStream(Stream: TStream);
var
  Component: TMediaCollectionComponent;
begin
  Component := TMediaCollectionComponent.Create(nil);
  try
    try
      Component.FList := self;
      Stream.WriteComponentRes('TMediaCollection', Component);
    finally
      Component.Free;
    end;
  except
    raise Exception.Create('Could not save TMediaCollection');
  end;
end;

//-----------------------------------------------------
// TOmegaMultimedia
//-----------------------------------------------------
constructor TOmegaMultimedia.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FMediaList := TMediaCollection.Create(self);
end;

//-----------------------------------------------------
destructor TOmegaMultimedia.Destroy;
begin
  FMediaList.Free;
  inherited;
end;

//-----------------------------------------------------
procedure TOmegaMultimedia.SetMediaList(const Value: TMediaCollection);
begin
  FMediaList.Assign(Value);
end;

//-----------------------------------------------------
procedure TOmegaMultimedia.Init;
var
  i: Integer;
begin
  for i := 0 to FMediaList.Count -1 do
  begin
    FMediaList.Items[i].Init;
  end;
end;

//-----------------------------------------------------
procedure TOmegaMultimedia.Render;
begin
  FMediaList.Render;
end;

end.
