{
//****************************************************************
// OmegaParticleEngine unit
//****************************************************************

07/25/04, baRman_ru:
 - added DoCollision property
}

unit OmegaParticleEngine;

interface

uses
  Classes, SysUtils, IniFiles, OmegaSprite, OmegaImageList, OmegaUtils;

type
  TEngineItem = class;

  // hold all sprite properties in the particle engine
  TPESprite = class(TPersistent)
  private
    FEngine       : TEngineItem;
    FImage        : TImageListItem;
    FRed,
    FGreen,
    FBlue,
    FAlpha        : Integer;
    FDoAdditiveBlend : Boolean;
    FBlendMode    : Integer;
    FImageIndex   : Integer;
    FDoCollision,
    FDoPixelCheck : Boolean;
    FRotation     : Single;
    FScaleX,
    FScaleY       : Single;
    FDoAnimate    : Boolean;
    FAnimCount,
    FAnimSpeed    : Integer;
    procedure SetBlendMode(const Value: Integer);
  protected
    procedure SetImage(Value: TImageListItem);
    procedure SetRed(Value: Integer);
    procedure SetGreen(Value: Integer);
    procedure SetBlue(Value: Integer);
    procedure SetAlpha(Value: Integer);
    procedure SetDoAdditiveBlend(Value: Boolean);
    procedure SetImageIndex(Value: Integer);
    procedure SetDoCollision(Value: Boolean);
    procedure SetPixelCheck(Value: Boolean);
    procedure SetRotation(Value: Single);
    procedure SetScaleX(Value: Single);
    procedure SetScaleY(Value: Single);
    procedure SetDoAnimate(Value: Boolean);
    procedure SetAnimCount(Value: Integer);
    procedure SetAnimSpeed(Value: Integer);
  public
    property Image: TImageListItem read FImage write SetImage;
    property Engine: TEngineItem read FEngine write FEngine;
    constructor Create;
  published
    property Red: Integer read FRed write SetRed default 255;
    property Green: Integer read FGreen write SetGreen default 255;
    property Blue: Integer read FBlue write SetBlue default 255;
    property Alpha: Integer read FAlpha write SetAlpha default 255;
    property DoAdditiveBlend: Boolean read FDoAdditiveBlend write SetDoAdditiveBlend;
    property BlendMode: Integer read FBlendMode write SetBlendMode;
    property ImageIndex: Integer read FImageIndex write SetImageIndex default -1;
    property DoCollision: Boolean read FDoCollision write SetDoCollision;
    property DoPixelCheck: Boolean read FDoPixelCheck write SetPixelCheck;
    property Rotation: Single read FRotation write SetRotation;
    property ScaleX: Single read FScaleX write SetScaleX ;
    property ScaleY: Single read FScaleY write SetScaleY ;
    property DoAnimate: Boolean read FDoAnimate write SetDoAnimate;
    property AnimCount: Integer read FAnimCount write SetAnimCount;
    property AnimSpeed: Integer read FAnimSpeed write SetAnimSpeed;
  end;

  // hold all particle properties in the particle engine
  TPEParticle = class(TPersistent)
  private
    FEngine : TEngineItem;

    //FUpdateSpeed : Single;
    FX,
    FY           : Single;
    FZ           : Integer;
    FXvel,
    FYvel,
    FXacc,
    FYacc        : Single;
    FLifeTime,
    FDecay       : Single;
    FLoop        : Boolean;
    FRepeatCount : Integer;
    FAnimate     : Boolean;
    FAnimateCount,
    FAnimateIndex: Integer;

    FRandomLocation : Boolean;
  protected
    procedure SetAnimate(Value: Boolean);
    procedure SetXvel(Value: Single);
    procedure SetYvel(Value: Single);
    procedure SetXacc(Value: Single);
    procedure SetYacc(Value: Single);
  public
    property Engine: TEngineItem read FEngine write FEngine;
    constructor Create;
  published
    //property UpdateSpeed: Single   read FUpdateSpeed  write FUpdateSpeed;
    property X: Single             read FX            write FX;
    property Y: Single             read FY            write FY;
    property Z: Integer            read FZ            write FZ;
    property Xvel: Single          read FXvel         write SetXvel;
    property Yvel: Single          read FYvel         write SetYvel;
    property Xacc: Single          read FXacc         write SetXacc;
    property Yacc: Single          read FYacc         write SetYacc;
    property LifeTime: Single      read FLifeTime     write FLifeTime;
    property Decay: Single         read FDecay        write FDecay;
    property Loop: boolean         read FLoop         write FLoop        default True;
    property RepeatCount: Integer  read FRepeatCount  write FRepeatCount;
    property Animate: boolean      read FAnimate      write SetAnimate;
    property AnimateCount: Integer read FAnimateCount write FAnimateCount;
    property AnimateIndex: Integer read FAnimateIndex write FAnimateIndex;

    property RandomLocation: Boolean read FRandomLocation write FRandomLocation default True;
  end;

  // the basic particle
  TParticle = class(TSimpleAnimSprite)
  private
    FEngine: TEngineItem;
    FXvel,
    FYvel,
    FXacc,
    FYacc        : Single;
    FLifeTime,
    FDecay       : Single;
    FLoop        : Boolean;
    FRepeatCount : Integer;
    FAnimate     : Boolean;
    FAnimateCount,
    FAnimateIndex: Integer;

    FAnimateSpeed,
    FAnimateFrame: Single;

    FLifeUnderZero: Boolean;
    FDrawIt: Boolean;
  protected
    procedure Reset; virtual;
    procedure Update(UpdateSpeed: Single); virtual;
  public
    procedure Draw; override;
    property Engine: TEngineItem   read FEngine       write FEngine;
    property Xvel: Single          read FXvel         write FXvel;
    property Yvel: Single          read FYvel         write FYvel;
    property Xacc: Single          read FXacc         write FXacc;
    property Yacc: Single          read FYacc         write FYacc;
    property LifeTime: Single      read FLifeTime     write FLifeTime;
    property Decay: Single         read FDecay        write FDecay;
    property Loop: boolean         read FLoop         write FLoop;
    property RepeatCount: Integer  read FRepeatCount  write FRepeatCount;
    property Animate: boolean      read FAnimate      write FAnimate;
    property AnimateCount: Integer read FAnimateCount write FAnimateCount;
    property AnimateIndex: Integer read FAnimateIndex write FAnimateIndex;     
  published
  end;      
  TParticleClass = class of TParticle;

  // our engine :)
  TEngineItem = class(TCollectionItem)
  private
    FPEParticle: TPEParticle;
    FPESprite: TPESprite;
    FParticles: TList;

    FName: String;
    FNumOfParticles: Integer;
    FChangeAll: Boolean;
    FDrawWithSpriteEngine: Boolean;

    FCustomParticle: TParticleClass;
    procedure NewParticle(Particle: TParticle);
  protected
    function GetCount: Integer;
    procedure SetParticle(Index: Integer; Value: TParticle);
    function GetParticle(Index: Integer): TParticle;
    procedure SetNumOfParticles(Value: Integer);
    function GetDisplayName: String; override;
  public
    property Count: Integer read GetCount;
    property Items[Index: Integer]: TParticle read GetParticle write SetParticle;

    constructor Create(Collection: TCollection); override;
    destructor Destroy; override;
    procedure Clear;
    procedure Draw;
    procedure Update(const UpdateSpeed: Single);
    procedure UseCustomParticle(Particle: TParticleClass);
    procedure SaveToFile(const FileName: TFileName);
    procedure LoadFromFile(const FileName: TFileName);
  published
    property Sprite: TPESprite read FPESprite write FPESprite;
    property Particle: TPEParticle read FPEParticle write FPEParticle;
    property NumOfParticles: Integer read FNumOfParticles write SetNumOfParticles default 0;
    property Name: String read FName write FName;
    property ChangeAll: Boolean read FChangeAll write FChangeAll default True;
    property DrawWithSpriteEngine: Boolean read FDrawWithSpriteEngine write FDrawWithSpriteEngine default True;
  end;

  TOmegaParticleEngine = class;

  // holds our engines
  TEngineCollection = class(TOwnedCollection)
  private
    FOmegaPE: TOmegaParticleEngine;
  protected
    function GetItem(Index: Integer): TEngineItem;
    procedure SetItem(Index: Integer; Value: TEngineItem);
    procedure Update(Item: TCollectionItem); override;
  public
    constructor Create(AOwner: TComponent);
    destructor Destroy; override;
    function Add: TEngineItem;
    function Insert(Index: Integer): TEngineItem;
    function Find(AName: String): TEngineItem;
    property Items[Index: Integer]: TEngineItem read GetItem write SetItem;
  published       
  end;

  // the component
  TOmegaParticleEngine = class(TComponent)
  private
    FOmegaSprite: TOmegaSprite;
    FEngineList : TEngineCollection;
    FInitialized: boolean;
    //FUpdateSpeed: Single;
  protected
    procedure SetEngineList(const Value: TEngineCollection);
    //procedure SetUpdateSpeed(Value: Single);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Init;
    procedure UpdateAll(const UpdateSpeed: Single);
    procedure DrawAll;
    property Initialized: Boolean read FInitialized;
    //property UpdateSpeed: Single read FUpdateSpeed write SetUpdateSpeed;
  published
    property OmegaSprite: TOmegaSprite read FOmegaSprite write FOmegaSprite;
    property EngineList: TEngineCollection read FEngineList write SetEngineList;
  end;

implementation

// TPESprite
//------------------------------------------------

constructor TPESprite.Create;
begin
  inherited;
  FREd := 255;
  FGreen := 255;
  FBlue := 255;
  FAlpha := 255;
  FImageIndex := -1;
  FScaleX := 1;
  FScaleY := 1;
end;

procedure TPESprite.SetImage(Value: TImageListItem);
var
  i: Integer;
begin
  FImage := Value;
  if FEngine.FChangeAll then
     for i := 0 to FEngine.FParticles.Count - 1 do
         TParticle(FEngine.FParticles.Items[i]).Image := Value;
end;

procedure TPESprite.SetRed(Value: Integer);
var
  i: Integer;
begin
  FRed := Value;
  if FEngine.FChangeAll then
     for i := 0 to FEngine.FParticles.Count - 1 do
         TParticle(FEngine.FParticles.Items[i]).Red := Value;
end;

procedure TPESprite.SetGreen(Value: Integer);
var
  i: Integer;
begin
  FGreen := Value;
  if FEngine.FChangeAll then
     for i := 0 to FEngine.FParticles.Count - 1 do
         TParticle(FEngine.FParticles.Items[i]).Green := Value;
end;

procedure TPESprite.SetBlue(Value: Integer);
var
  i: Integer;
begin
  FBlue := Value;
  if FEngine.FChangeAll then
     for i := 0 to FEngine.FParticles.Count - 1 do
         TParticle(FEngine.FParticles.Items[i]).Blue := Value;
end;

procedure TPESprite.SetAlpha(Value: Integer);
var
  i: Integer;
begin
  FAlpha := Value;
  if FEngine.FChangeAll then
     for i := 0 to FEngine.FParticles.Count - 1 do
         TParticle(FEngine.FParticles.Items[i]).Alpha := Value;
end;

procedure TPESprite.SetDoAdditiveBlend(Value: Boolean);
var
  i: Integer;
begin
  FDoAdditiveBlend := Value;
  if FEngine.FChangeAll then
     for i := 0 to FEngine.FParticles.Count - 1 do
         TParticle(FEngine.FParticles.Items[i]).DoAdditiveBlend := Value;
end;

procedure TPESprite.SetImageIndex(Value: Integer);
var
  i: Integer;
begin
  FImageIndex := Value;
  if FEngine.FChangeAll then
     for i := 0 to FEngine.FParticles.Count - 1 do
         TParticle(FEngine.FParticles.Items[i]).ImageIndex := Value;
end;

procedure TPESprite.SetDoCollision(Value: Boolean);
var
  i: Integer;
begin
  FDoCollision := Value;
  if FEngine.FChangeAll then
     for i := 0 to FEngine.FParticles.Count - 1 do
         TParticle(FEngine.FParticles.Items[i]).DoCollision := Value;
end;

procedure TPESprite.SetPixelCheck(Value: Boolean);
var
  i: Integer;
begin
  FDoPixelCheck := Value;
  if FEngine.FChangeAll then
     for i := 0 to FEngine.FParticles.Count - 1 do
         TParticle(FEngine.FParticles.Items[i]).DoPixelCheck := Value;
end;

procedure TPESprite.SetRotation(Value: Single);
var
  i: Integer;
begin
  FRotation := Value;
  if FEngine.FChangeAll then
     for i := 0 to FEngine.FParticles.Count - 1 do
         TParticle(FEngine.FParticles.Items[i]).Rotation := Value;
end;

procedure TPESprite.SetScaleX(Value: Single);
var
  i: Integer;
begin
  FScaleX := Value;
  if FEngine.FChangeAll then
     for i := 0 to FEngine.FParticles.Count - 1 do
         TParticle(FEngine.FParticles.Items[i]).ScaleX := Value;
end;

procedure TPESprite.SetScaleY(Value: Single);
var
  i: Integer;
begin
  FScaleY := Value;
  if FEngine.FChangeAll then
     for i := 0 to FEngine.FParticles.Count - 1 do
         TParticle(FEngine.FParticles.Items[i]).ScaleY := Value;
end;

procedure TPESprite.SetDoAnimate(Value: Boolean);
var
  i: Integer;
begin
  FDoAnimate := Value;
  if FEngine.FChangeAll then
     for i := 0 to FEngine.FParticles.Count - 1 do
         TParticle(FEngine.FParticles.Items[i]).DoAnimate := Value;
end;

procedure TPESprite.SetAnimCount(Value: Integer);
var
  i: Integer;
begin
  FAnimCount := Value;
  if FEngine.FChangeAll then
     for i := 0 to FEngine.FParticles.Count - 1 do
         TParticle(FEngine.FParticles.Items[i]).AnimCount := Value;
end;

procedure TPESprite.SetAnimSpeed(Value: Integer);
var
  i: Integer;
begin
  FAnimSpeed := Value;
  if FEngine.FChangeAll then
     for i := 0 to FEngine.FParticles.Count - 1 do
         TParticle(FEngine.FParticles.Items[i]).AnimSpeed := Value;
end;

// TPEParticle
//------------------------------------------------

constructor TPEParticle.Create;
begin
  inherited;
  //FUpdateSpeed := 1;
  FLoop := True;
  FRandomLocation := True;
  FDecay := 1.0;
end;

procedure TPEParticle.SetAnimate(Value: Boolean);
var
  i: Integer;
begin
  FAnimate := Value;
  if FEngine.FChangeAll then
     for i := 0 to FEngine.FParticles.Count - 1 do
         TParticle(FEngine.FParticles.Items[i]).FAnimate := Value;
end;

procedure TPEParticle.SetXvel(Value: Single);
var
  i: Integer;
begin
  FXvel := Value;
  if FEngine.FChangeAll then
     for i := 0 to FEngine.FParticles.Count - 1 do
         TParticle(FEngine.FParticles.Items[i]).FXvel := Value;
end;

procedure TPEParticle.SetYvel(Value: Single);
var
  i: Integer;
begin
  FYvel := Value;
  if FEngine.FChangeAll then
     for i := 0 to FEngine.FParticles.Count - 1 do
         TParticle(FEngine.FParticles.Items[i]).FYvel := Value;
end;

procedure TPEParticle.SetXacc(Value: Single);
var
  i: Integer;
begin
  FXacc := Value;
  if FEngine.FChangeAll then
     for i := 0 to FEngine.FParticles.Count - 1 do
         TParticle(FEngine.FParticles.Items[i]).FXacc := Value;
end;

procedure TPEParticle.SetYacc(Value: Single);
var
  i: Integer;
begin
  FYacc := Value;
  if FEngine.FChangeAll then
     for i := 0 to FEngine.FParticles.Count - 1 do
         TParticle(FEngine.FParticles.Items[i]).FYacc := Value;
end;

// TParticle
//------------------------------------------------

procedure TParticle.Reset;
var
  Temp{, Temp2}: Single;
begin
  X := FEngine.FPEParticle.FX;
  Y := FEngine.FPEParticle.FY;
  Z := FEngine.FPEParticle.FZ;

  FXvel := FEngine.FPEParticle.FXvel;
  FYvel := FEngine.FPEParticle.FYvel;

  FXacc := FEngine.FPEParticle.FXacc;
  FYacc := FEngine.FPEParticle.FYacc;

  Temp := FLifeTime;
  FLifeTime := FEngine.FPEParticle.FLifeTime;
  FDecay := FEngine.FPEParticle.FDecay;

  FAnimate := FEngine.FPEParticle.FAnimate;
  FAnimateCount := FEngine.FPEParticle.FAnimateCount;

  //if Temp < 0 then
  if Temp < -1 then
  begin
    //Temp2 := FLifeTime + Temp;
    //repeat
      //Update(FEngine.FPEParticle.FUpdateSpeed);
    //  Update(1);
    //until FLifeTime <= Temp2;

    FLifeUnderZero := True;
  end;

  if FAnimate then
  begin
    FAnimateSpeed := FLifeTime / FAnimateCount;
    ImageIndex := FEngine.FPEParticle.FAnimateIndex;
  end;
end;

//------------------------------------------------

procedure TParticle.Update(UpdateSpeed: Single);
var
  randTemp, i: Integer;
begin
  X := X + FXvel * UpdateSpeed;
  Y := Y + FYvel * UpdateSpeed;

  FXvel := FXvel + FXacc * UpdateSpeed;
  FYvel := FYvel + FYacc * UpdateSpeed;

  if FLifeunderZero then
  begin
    FLifeUnderZero := False;
    if (FEngine.FPEParticle.FRandomLocation) then
    begin
      randTemp := RoundNormal(FEngine.FPEParticle.FLifeTime);
      randTemp := Random(randTemp);
      for i := 0 to randTemp do
          Update(1);
    end;    
  end;

  if FAnimate then
  begin
    FAnimateFrame := FAnimateFrame + 1*UpdateSpeed;
    if FAnimateFrame >= FAnimateSpeed then
    begin
      FAnimateFrame := 0;
      ImageIndex := ImageIndex + 1;
    end;
  end;

  FLifeTime := FLifeTime - FDecay * UpdateSpeed;
  if FLifeTime <= 0 then
  begin
    if not FLoop then
    begin
      if FRepeatCount > 0 then
      begin
        Dec(FRepeatCount);
        Reset;
      end
      else
      begin
        Dead;
        FEngine.FParticles.Remove(self);
        FEngine.FNumOfParticles := FEngine.FNumOfParticles - 1;
      end;
    end
    else
    begin
      Reset;
    end;
  end;
end;

//------------------------------------------------

procedure TParticle.Draw;
begin
  if (not FDrawIt) and (not FEngine.FDrawWithSpriteEngine) then Exit;
  inherited;
  FDrawIt := False;
end;

// TEngineItem
//------------------------------------------------

constructor TEngineItem.Create(Collection: TCollection);
begin
  inherited Create(Collection);
  if not (Collection is TEngineCollection) then
     raise Exception.Create('Collection of TEngineItem must be of type TEngineCollection');

  FCustomParticle := TParticle;
  FNumOfPArticles := 0;
  FChangeAll := True;
  FDrawWithSpriteEngine := True;
  FParticles := TList.Create;
  FPEParticle := TPEParticle.Create;
  FPEParticle.FEngine := self;
  FPESprite := TPESprite.Create;
  FPESprite.FEngine := Self;
end;

//------------------------------------------------

destructor TEngineItem.Destroy;
var
  i: Integer;
begin
  for i := FParticles.Count - 1 downto 0 do
      FParticles.Delete(i);
  FParticles.Free;
  FPEParticle.Free;
  FPESprite.Free;
  inherited;
end;

//------------------------------------------------

function TEngineItem.GetCount: Integer;
begin
  Result := FParticles.Count;
end;

//------------------------------------------------

function TEngineItem.GetParticle(Index: Integer): TParticle;
begin
  Result := nil;

  if (Index >= 0) and (Index < FParticles.Count) then
     Result := TParticle(FParticles.Items[Index]);
end;

//------------------------------------------------

function TEngineItem.GetDisplayName: String;
begin
  Result := FName;
end;

//------------------------------------------------

procedure TEngineItem.SetParticle(Index: Integer; Value: TParticle);
begin
  if (Index >= 0) and (Index < 0) then
  begin
    with TParticle(FParticles.Items[Index]) do
    begin
      X            := Value.X;
      Y            := Value.Y;
      Z            := Value.Z;
      FXvel        := Value.FXvel;
      FYvel        := Value.FYvel;
      FXacc        := Value.FXacc;
      FYacc        := Value.FYacc;
      FLifeTime    := Value.FLifeTime;
      Red          := Value.Red;
      Green        := Value.Green;
      Blue         := Value.Blue;
      Alpha        := Value.Alpha;
      ImageIndex   := Value.ImageIndex;
      DoCollision  := Value.DoCollision;
      DoPixelCheck := Value.DoPixelCheck;
      Rotation     := Value.Rotation;
      ScaleX       := Value.ScaleX;
      ScaleY       := Value.ScaleY;
      DoAnimate    := Value.DoAnimate;
      AnimCount    := Value.AnimCount;
      AnimSpeed    := Value.AnimSpeed;
      Width        := Value.Width;
      Height       := Value.Height;
      DoAdditiveBlend := Value.DoAdditiveBlend;
      BlendMode    := Value.BlendMode;
      FRepeatCount := Value.FRepeatCount;
      FLoop        := Value.FLoop;
      FAnimate     := Value.FAnimate;
      FAnimateSpeed:= Value.FAnimateSpeed;
      FAnimateFrame:= Value.FAnimateFrame;
    end;
  end;
end;

//------------------------------------------------

procedure TEngineItem.SetNumOfParticles(Value: Integer);
var
  randTemp: Integer;
  i, i2: Integer;
  p: TParticle;
begin
  if Value < 0 then Value := 0;  // this way there is no error if the user accidently set then number of particles below zero

  if not TEngineCollection(Collection).FOmegaPE.FInitialized then
  begin
    FNumOfParticles := Value;
    Exit;
  end;

  if Value > FNumOfParticles then
  begin
    for i := FNumOfParticles to Value -1 do
    begin
      p := FCustomParticle.Create(TEngineCollection(Collection).FOmegaPE.FOmegaSprite);
      NewParticle(p);

      if FPEParticle.FRandomLocation then
      begin
        randTemp := RoundNormal(FPEParticle.FLifeTime);
        randTemp := Random(randTemp);
        for i2 := 0 to randTemp do
            p.Update(1);
      end;

      FParticles.Add(p);
    end;
    FNumOfParticles := Value;
  end
  else
  if Value < FNumOfParticles then
  begin
    for i := FNumOfParticles - 1 downto Value do
    begin
      if TParticle(FParticles.Items[i]) <> nil then TParticle(FParticles.Items[i]).Dead;
      FParticles.Delete(i);
    end;
    FNumOfParticles := Value;
    TEngineCollection(Collection).FOmegaPE.FOmegaSprite.Dead;
  end;
end;

//------------------------------------------------

procedure TEngineItem.NewParticle(Particle: TParticle);
begin
  with Particle do
  begin
    FEngine      := self;

    X            := self.FPEParticle.FX;
    Y            := self.FPEParticle.FY;
    Z            := self.FPEParticle.FZ;
    FXvel        := self.FPEParticle.FXvel;
    FYvel        := self.FPEParticle.FYvel;
    FXacc        := self.FPEParticle.FXacc;
    FYacc        := self.FPEParticle.FYacc;
    FLifeTime    := self.FPEParticle.FLifeTime;
    FDecay       := self.FPEParticle.FDecay;
    FRepeatCount := self.FPEParticle.FRepeatCount;
    FLoop        := Self.FPEParticle.FLoop;

    Red          := self.FPESprite.FRed;
    Green        := self.FPESprite.FGreen;
    Blue         := self.FPESprite.FBlue;
    Alpha        := self.FPESprite.FAlpha;
    DoAdditiveBlend := self.FPESprite.FDoAdditiveBlend;
    BlendMode := self.FPESprite.FBlendMode;
    ImageIndex   := self.FPESprite.FImageIndex;
    DoCollision  := self.FPESprite.FDoCollision;
    DoPixelCheck := self.FPESprite.FDoPixelCheck;
    Rotation     := self.FPESprite.FRotation;
    ScaleX       := self.FPESprite.FScaleX;
    ScaleY       := self.FPESprite.FScaleY;
    DoAnimate    := self.FPESprite.FDoAnimate;
    AnimCount    := self.FPESprite.FAnimCount;
    AnimSpeed    := self.FPESprite.FAnimSpeed;
    Width        := self.FPESprite.FImage.TileWidth;
    Height       := self.FPESprite.FImage.TileHeight; 
    Image        := self.FPESprite.FImage;
  end;
  Particle.Reset;
end;

//------------------------------------------------

procedure TEngineItem.Clear;
var
  i: integer;
begin
  for i := FParticles.Count - 1 downto 0 do
  begin
    TParticle(FParticles.Items[i]).Dead;
    FParticles.Delete(i);
  end;
  FNumOfParticles := 0;

  if TEngineCollection(Collection).FOmegaPE.FOmegaSprite <> nil then
     TEngineCollection(Collection).FOmegaPE.FOmegaSprite.Dead;
end;

//------------------------------------------------

procedure TEngineItem.UseCustomParticle(Particle: TParticleClass);
begin
  FCustomParticle := Particle;
end;

//------------------------------------------------

procedure TEngineItem.Draw;
var
  i: Integer;
  Particle: TParticle;
begin
  for i := FParticles.Count - 1 downto 0 do
  begin
    Particle := TParticle(FParticles.Items[i]);
    Particle.FDrawIt := True;
    Particle.Draw;
  end;
end;

//------------------------------------------------

procedure TEngineItem.Update(const UpdateSpeed: Single);
var
  i: Integer;
  Particle: TParticle;
begin
  for i := FParticles.Count - 1 downto 0 do
  begin
    Particle := TParticle(FParticles.Items[i]);
    Particle.Update(UpdateSpeed);
  end;
end;

//------------------------------------------------

procedure TEngineItem.SaveToFile(const FileName: TFileName);
var
  Ini: TIniFile;
begin
  Ini := TIniFile.Create(FileName);
  try
    // general info
    Ini.WriteBool   ('General', 'ChangeAll',      FChangeAll);
    Ini.WriteInteger('General', 'NumOfParticles', FNumOfParticles);
    Ini.WriteBool   ('General', 'DrawWithSpriteEngine', FDrawWithSpriteEngine);
    // particle info
    Ini.WriteBool   ('Particle', 'Animate',        FPEParticle.FAnimate);
    Ini.WriteInteger('Particle', 'AnimateCount',   FPEParticle.FAnimateCount);
    Ini.WriteInteger('Particle', 'AnimateIndex',   FPEParticle.FAnimateIndex);
    Ini.WriteFloat  ('Particle', 'Decay',          FPEParticle.FDecay);
    Ini.WriteFloat  ('Particle', 'LifeTime',       FPEParticle.FLifeTime);
    Ini.WriteBool   ('Particle', 'Loop',           FPEParticle.Loop);
    Ini.WriteBool   ('Particle', 'RandomLocation', FPEParticle.FRandomLocation);
    Ini.WriteInteger('Particle', 'RepeatCount',    FPEParticle.FRepeatCount);
    Ini.WriteFloat  ('Particle', 'X',              FPEParticle.FX);
    Ini.WriteFloat  ('Particle', 'Xacc',           FPEParticle.FXacc);
    Ini.WriteFloat  ('Particle', 'Xvel',           FPEparticle.FXvel);
    Ini.WriteFloat  ('Particle', 'Y',              FPEparticle.FY);
    Ini.WriteFloat  ('Particle', 'Yacc',           FPEParticle.FYacc);
    Ini.WriteFloat  ('Particle', 'Yvel',           FPEParticle.FYvel);
    Ini.WriteInteger('Particle', 'Z',              FPEParticle.FZ);
    // sprite info
    Ini.WriteInteger('Sprite', 'Alpha',        FPESprite.FAlpha);
    Ini.WriteInteger('Sprite', 'AnimCount',    FPESprite.FAnimCount);
    Ini.WriteInteger('Sprite', 'AnimSpeed',    FPESprite.FAnimSpeed);
    Ini.WriteInteger('Sprite', 'Blue',         FPESprite.FBlue);
    Ini.WriteBool   ('Sprite', 'DoAdditiveBlend', FPESprite.FDoAdditiveBlend);
    Ini.WriteInteger('Sprite', 'BlendMode',         FPESprite.FBlendMode);
    Ini.WriteBool   ('Sprite', 'DoAnimate',    FPESprite.DoAnimate);
    Ini.WriteBool   ('Sprite', 'DoPixelCheck', FPESprite.FDoPixelCheck);
    Ini.WriteInteger('Sprite', 'Green',        FPESprite.FGreen);
    Ini.WriteInteger('Sprite', 'ImageIndex',   FPESprite.FImageIndex);
    Ini.WriteInteger('Sprite', 'Red',          FPESprite.FRed);
    Ini.WriteFloat  ('Sprite', 'Rotation',     FPESprite.FRotation);
    Ini.WriteFloat  ('Sprite', 'ScaleX',       FPESprite.FScaleX);
    Ini.WriteFloat  ('Sprite', 'ScaleY',       FPESprite.FScaleY);
  finally
    Ini.Free;
  end;
end;

//------------------------------------------------

procedure TEngineItem.LoadFromFile(const FileName: TFileName);
var
  Ini: TIniFile;
  num: Integer;
begin
  num := 0;
  Ini := TIniFile.Create(FileName);
  try
    Clear;
    // general info
    FChangeAll := Ini.ReadBool   ('General', 'ChangeAll',      True) ;
    num        := Ini.ReadInteger('General', 'NumOfParticles', 0);
    FDrawWithSpriteEngine := Ini.ReadBool('General', 'DrawWithSpriteEngine', True);
    // particle info
    FPEParticle.FAnimate        := Ini.ReadBool   ('Particle', 'Animate',        False);
    FPEParticle.FAnimateCount   := Ini.ReadInteger('Particle', 'AnimateCount',   0);
    FPEParticle.FAnimateIndex   := Ini.ReadInteger('Particle', 'AnimateIndex',   0);
    FPEParticle.FDecay          := Ini.ReadFloat  ('Particle', 'Decay',          1);
    FPEParticle.FLifeTime       := Ini.ReadFloat  ('Particle', 'LifeTime',       0);
    FPEParticle.FLoop           := Ini.ReadBool   ('Particle', 'Loop',           True);
    FPEParticle.FRandomLocation := Ini.ReadBool   ('Particle', 'RandomLocation', True);
    FPEParticle.FRepeatCount    := Ini.ReadInteger('Particle', 'RepeatCount',    0);
    FPEParticle.FX              := Ini.ReadFloat  ('Particle', 'X',              0);
    FPEParticle.FXacc           := Ini.ReadFloat  ('Particle', 'Xacc',           0);
    FPEParticle.FXvel           := Ini.ReadFloat  ('Particle', 'Xvel',           0);
    FPEParticle.FY              := Ini.ReadFloat  ('Particle', 'Y',              0);
    FPEParticle.FYacc           := Ini.ReadFloat  ('Particle', 'Yacc',           0);
    FPEParticle.FYvel           := Ini.ReadFloat  ('Particle', 'Yvel',           0);
    FPEParticle.FZ              := Ini.ReadInteger('Particle', 'Z',              0);
    // sprite info
    FPESprite.FAlpha        := Ini.ReadInteger('Sprite', 'Alpha',        255);
    FPESprite.FAnimCount    := Ini.ReadInteger('Sprite', 'AnimCount',    0);
    FPESprite.FAnimSpeed    := Ini.ReadInteger('Sprite', 'AnimSpeed',    0);
    FPESprite.FBlue         := Ini.ReadInteger('Sprite', 'Blue',         255);
    FPESprite.FDoAdditiveBlend := Ini.ReadBool   ('Sprite', 'DoAdditiveBlend', False);
    FPESprite.FBlendmode         := Ini.ReadInteger('Sprite', 'Blendmode',         0);
    FPESprite.FDoAnimate    := Ini.ReadBool   ('Sprite', 'DoAnimate',    False);
    FPESprite.FDoPixelCheck := Ini.ReadBool   ('Sprite', 'DoPixelCheck', False);
    FPESprite.FGreen        := Ini.ReadInteger('Sprite', 'Green',        255);
    FPESprite.FImageIndex   := Ini.ReadInteger('Sprite', 'ImageIndex',   -1);
    FPESprite.FRed          := Ini.ReadInteger('Sprite', 'Red',          255);
    FPESprite.FRotation     := Ini.ReadFloat  ('Sprite', 'Rotation',     0);
    FPESprite.FScaleX       := Ini.ReadFloat  ('Sprite', 'ScaleX',       0);
    FPESprite.FScaleY       := Ini.ReadFloat  ('Sprite', 'ScaleY',       0);
  finally
    NumOfParticles := num;
    Ini.Free;
  end;
end;

// TEngineCollection
//------------------------------------------------

constructor TEngineCollection.Create(AOwner: TComponent);
begin
  inherited Create(AOwner, TEngineItem);
  if not (AOwner is TOmegaParticleEngine) then
     raise Exception.Create('Owner of TEngineCollection must be a TOmegaParticleEngine');
  FOmegaPE := TOmegaParticleEngine(AOwner);
end;

//------------------------------------------------

destructor TEngineCollection.Destroy;
begin
  inherited;
end;

//------------------------------------------------

function TEngineCollection.GetItem(Index: Integer): TEngineItem;
begin
  Result := TEngineItem(inherited GetItem(Index));
end;

//------------------------------------------------

procedure TEngineCollection.SetItem(Index: Integer; Value: TEngineItem);
begin
  inherited SetItem(Index, Value);
end;

//----------------------------------------------------------------

procedure TEngineCollection.Update(Item: TCollectionItem);
begin
  inherited Update(Item);
end;

//----------------------------------------------------------------

function TEngineCollection.Add: TEngineItem;
begin
  Result := TEngineItem(inherited Add);
end;

//----------------------------------------------------------------

function TEngineCollection.Insert(Index: Integer): TEngineItem;
begin
  Result := TEngineItem(inherited Insert(Index));
end;

//----------------------------------------------------------------

function TEngineCollection.Find(AName: String): TEngineItem;
var
  i: Integer;
begin
  Result := nil;

  for i := 0 to Count -1 do
    if Uppercase(AName) = Uppercase(Items[i].FName) then
    begin
      Result := Items[i];
      Exit;
    end;
end;

// TOmegaParticleEngine
//------------------------------------------------

constructor TOmegaParticleEngine.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FEngineList := TEngineCollection.Create(self);
  FInitialized := False;
end;

//------------------------------------------------

destructor TOmegaParticleEngine.Destroy;
begin
  FEngineList.Free;
  inherited;
end;

//------------------------------------------------

{procedure TOmegaParticleEngine.SetUpdateSpeed(Value: Single);
var
  i: Integer;
  Engine: TEngineItem;
begin
  FUpdateSpeed := Value;
  for i := 0 to FEngineList.Count - 1 do
  begin
    Engine := TEngineItem(EngineList.Items[i]);
    Engine.FPEParticle.FUpdateSpeed := Value;
  end;
end; }

//------------------------------------------------

procedure TOmegaParticleEngine.SetEngineList(const Value: TEngineCollection);
begin
  FEngineList.Assign(Value);
end;

//------------------------------------------------

procedure TOmegaParticleEngine.Init;
var
  Engine: TEngineItem;
  i: Integer;
  numTemp: Integer;
begin
  for i := 0 to EngineList.Count - 1 do
  begin
    Engine := TEngineItem(EngineList.Items[i]);
    numTemp := Engine.FNumOfParticles;
    Engine.Clear;
    self.FInitialized := True;
    Engine.NumOfParticles := numTemp;
  end;
end;

//------------------------------------------------

procedure TOmegaParticleEngine.UpdateAll(const UpdateSpeed: Single);
var
  Engine: TEngineItem;
  i: Integer;
begin
  for i := EngineList.Count - 1 downto 0 do
  begin
    Engine := TEngineItem(EngineList.Items[i]);
    Engine.Update(UpdateSpeed);
  end;
end;

//------------------------------------------------

procedure TOmegaParticleEngine.DrawAll;
var
  Engine: TEngineItem;
  i: Integer;
begin
  for i := EngineList.Count - 1 downto 0 do
  begin
    Engine := TEngineItem(EngineList.Items[i]);
    Engine.Draw;
  end;
end;

//------------------------------------------------

procedure TPESprite.SetBlendMode(const Value: Integer);
var
  i: Integer;
begin
  FBlendMode := Value;
  if FEngine.FChangeAll then
     for i := 0 to FEngine.FParticles.Count - 1 do
         TParticle(FEngine.FParticles.Items[i]).BlendMode := Value;
end;

end.
