{
//****************************************************************
// OmegaUtils unit
// author : Tobias Fleischer
// contact: tobybear@web.de
// comments are welcome!
//****************************************************************

version history:
05/15/05: function InterSectRect(const r1, r2: TRect): boolean; added to functions (Czar)
05/11/05: added function MakeOmegaVertex (Bazi)
07/25/04: modifications by baRman_ru & Tobybear:
	  new type TQuad = array[0..3] of TPoint;
          function PointInRectangle: is a point in the given rectangle?
          function PointInQuadrangle: is a point in the given quadrangle?
	  function InterSectRect(const r1, r2: TRect): check if two rectangles intersect
01/27/04: added Msg for single types
01/26/04: added Modify functions
01/14/04: added SetPixel for OmegaScreen
10/21/03: added Msg, GetDelphiVersion and GetDXVersion
08/31/03: added SetTransparency
08/28/03: added ChangeDesktopResolution and SetTransparentColor
08/13/03: first release
          no documentation yet!
}

unit OmegaUtils;
interface
uses Windows, Graphics, OmegaScreen, {$I UseD3D8.inc};

const MyLock = 0; // D3DLOCK_NOSYSLOCK;

type
 TQuad = array [0..3] of TPoint;
 PixelFunction = procedure(x, y: DWORD; var r, g, b, a: byte);

function InterSectRect(const r1, r2: TRect): boolean;        // Czar
function PointInRectangle(const aX, aY: single; const aRect: TRect): boolean;
function PointInQuadrangle(const aX, aY: single; const aQuad: TQuad): boolean;

procedure ApplyPixelFunction(texture: IDIRECT3DTEXTURE8;
 pf: PixelFunction); overload;
procedure ApplyPixelFunction(texture: IDIRECT3DTEXTURE8;
 rec: trect; pf: PixelFunction); overload;

procedure SetPixel(device: IDIRECT3DDEVICE8; x, y: DWORD; Red, Green,
 Blue, Alpha: byte); overload;
procedure SetPixel(texture: IDIRECT3DTEXTURE8; x, y: DWORD; Red, Green,
 Blue, Alpha: byte); overload;
procedure SetPixel(surface: IDIRECT3DSURFACE8; x, y: DWORD; Red, Green,
 Blue, Alpha: byte); overload;
procedure SetPixel(os: TOmegaScreen; x, y: DWORD; Red, Green,
 Blue, Alpha: byte); overload;

procedure GetPixel(device: IDIRECT3DDEVICE8; x, y: DWORD;
 var Red, Green, Blue, Alpha: byte); overload;
procedure GetPixel(texture: IDIRECT3DTEXTURE8; x, y: DWORD;
 var Red, Green, Blue, Alpha: byte); overload;
procedure GetPixel(surface: IDIRECT3DSURFACE8; x, y: DWORD;
 var Red, Green, Blue, Alpha: byte); overload;

procedure ColorToAlpha(texture: IDIRECT3DTEXTURE8; RedToAlpha,
 GreenToAlpha, BlueToAlpha, AlphatoAlpha: single);
procedure Grayscale(texture: IDIRECT3DTEXTURE8);
procedure SetTransparentColor(texture: IDIRECT3DTEXTURE8; red, green, blue: byte);
procedure SetTransparency(texture: IDIRECT3DTEXTURE8; red, green, blue, alpha: byte);

function RoundNormal(Value : Extended): Integer;

function ChangeDesktopResolution(width, height: word): boolean;

function GetDelphiVersion:integer;
function GetDXVersion:string;

procedure Msg(m:string); overload;
procedure Msg(i:integer); overload;
procedure Msg(s:single); overload;
procedure Msg(m:string;i:integer); overload;

procedure Modify(var x: single; var y:single;
 CenterX, CenterY, Angle, ScaleX, ScaleY,
 OfsX, OfsY: single; c: TOmegaColor); overload;
procedure Modify(var p: TPoint;
 CenterX, CenterY, Angle, ScaleX, ScaleY,
 OfsX, OfsY: single; c: TOmegaColor); overload;

function MakeOmegaVertex(x,y : single; Color : TOmegaColor; tu  : real = 0 ; tv : real = 0) : TOmegaVertex; // Bazi

implementation
uses sysutils, registry, math;

var rr, rg, rb, ra: single;
    rri, rgi, rbi, rai: integer;

function MakeOmegaVertex(x,y : single; Color : TOmegaColor; tu  : real = 0 ; tv : real = 0) : TOmegaVertex;
begin
  Result.x := x;
  Result.y := y;
  Result.z := 0;
  Result.rhw := 1;
  Result.Color := Color;
  Result.tu := tu ;
  Result.tv := tv ;
end;

function InterSectRect(const r1, r2: TRect): boolean;
var l, t, r, b: integer;
begin
 l := Max(R1.Left, R2.Left);
 t := Max(R1.Top, R2.Top);
 r := Min(R1.Right, R2.Right);
 b := Min(R1.Bottom, R2.Bottom);
 Result := (l <= r) and (t <= b);
end;

function PointInRectangle(const aX, aY: single; const aRect: TRect): boolean;
begin
  Result := (aX >= aRect.Left) and (aX <= aRect.Right) and (aY >= aRect.Top) and (aY <= aRect.Bottom);
end;

function PointInQuadrangle(const aX, aY: single; const aQuad: TQuad): boolean;
  var d1, d2, d3, d4: single;
begin
 d1 := (aQuad[2].X - aQuad[1].X) * (aX - aQuad[0].X) + (aQuad[2].Y - aQuad[1].Y) * (aY - aQuad[0].Y);
 d2 := (aQuad[3].X - aQuad[2].X) * (aX - aQuad[1].X) + (aQuad[3].Y - aQuad[2].Y) * (aY - aQuad[1].Y);
 d3 := (aQuad[0].X - aQuad[3].X) * (aX - aQuad[2].X) + (aQuad[0].Y - aQuad[3].Y) * (aY - aQuad[2].Y);
 d4 := (aQuad[1].X - aQuad[0].X) * (aX - aQuad[3].X) + (aQuad[1].Y - aQuad[0].Y) * (aY - aQuad[3].Y);
 if (d1 >= 0) and (d2 >= 0) and (d3 >= 0) and (d4 >= 0) then
  Result:= true
  else Result:= false;
end;

procedure Msg(m:string);
begin MessageBox(0,PChar(m),'',1); end;
procedure Msg(i:integer);
begin MessageBox(0,PChar(inttostr(i)),'',1); end;
procedure Msg(s:single);
begin MessageBox(0,PChar(floattostrf(s,fffixed,3,3)),'',1); end;
procedure Msg(m:string;i:integer);
begin MessageBox(0,PChar(m+' '+inttostr(i)),'',1); end;

function RoundNormal(Value : Extended) : Integer;
var
  CurrencyValue : Currency;
begin
  if Value < 0 then begin
    CurrencyValue := Value - 0.5;
    Result := Trunc(CurrencyValue);
  end else begin
    CurrencyValue := Value + 0.5;
    Result := Trunc(CurrencyValue);
  end;
end;

procedure SetPixel(surface: IDIRECT3DSURFACE8;
 x, y: DWORD;
 Red, Green, Blue, Alpha: byte);
var surf_desc: TD3DSURFACE_DESC;
    rect: TD3DLOCKED_RECT;
    tmp: word;
    pb: pbyte;
begin
 if (FAILED(surface.GetDesc(surf_desc))) then exit;
 if (FAILED(surface.LockRect(rect, nil, MyLock))) then exit;

 case surf_desc.Format of
 D3DFMT_X8R8G8B8,D3DFMT_A8R8G8B8:
 begin
  pb  := pbyte(longint(rect.pBits) + y*rect.pitch + x*4);
  pb^ := blue;  inc(pb);
  pb^ := green; inc(pb);
  pb^ := red;   inc(pb);
  pb^ := alpha;
 end;
 D3DFMT_R8G8B8:
 begin
  pb  := pbyte(longint(rect.pBits) + y*rect.pitch + x*3);
  pb^ := blue;  inc(pb);
  pb^ := green; inc(pb);
  pb^ := red;
 end;
 D3DFMT_A4R4G4B4:
 begin
  pb  := pbyte(longint(rect.pBits) + y*rect.pitch + x*2);
  tmp := (alpha shr 4) shl 12 + (red shr 4) shl 8 +
   (green shr 4) shl 4 + (blue shr 4);
  pb^ := lo(tmp); inc(pb);
  pb^ := hi(tmp);
 end;
 D3DFMT_R5G6B5:
 begin
  pb  := pbyte(longint(rect.pBits) + y*rect.pitch + x*2);
  tmp := (red shr 3) shl 11 + (green shr 2) shl 5 + (blue shr 3);
  pb^ := lo(tmp); inc(pb);
  pb^ := hi(tmp);
 end;
 end;
 surface.UnlockRect;
end;

procedure SetPixel(texture: IDIRECT3DTEXTURE8;
 x, y: DWORD;
 Red, Green, Blue, Alpha: byte);
var surf_desc: TD3DSURFACE_DESC;
    rect: TD3DLOCKED_RECT;
    tmp: word;
    pb: pbyte;
begin
 if (FAILED(texture.GetLevelDesc(0, surf_desc))) then exit;
 if (FAILED(texture.LockRect(0, rect, nil, MyLock))) then exit;

 case surf_desc.Format of
 D3DFMT_X8R8G8B8,D3DFMT_A8R8G8B8:
 begin
  pb  := pbyte(longint(rect.pBits) + y*rect.pitch + x*4);
  pb^ := blue;  inc(pb);
  pb^ := green; inc(pb);
  pb^ := red;   inc(pb);
  pb^ := alpha;
 end;
 D3DFMT_R8G8B8:
 begin
  pb  := pbyte(longint(rect.pBits) + y*rect.pitch + x*3);
  pb^ := blue;  inc(pb);
  pb^ := green; inc(pb);
  pb^ := red;
 end;
 D3DFMT_A4R4G4B4:
 begin
  pb  := pbyte(longint(rect.pBits) + y*rect.pitch + x*2);
  tmp := (alpha shr 4) shl 12 + (red shr 4) shl 8 +
   (green shr 4) shl 4 + (blue shr 4);
  pb^ := lo(tmp); inc(pb);
  pb^ := hi(tmp);
 end;
 D3DFMT_R5G6B5:
 begin
  pb  := pbyte(longint(rect.pBits) + y*rect.pitch + x*2);
  tmp := (red shr 3) shl 11 + (green shr 2) shl 5 + (blue shr 3);
  pb^ := lo(tmp); inc(pb);
  pb^ := hi(tmp);
 end;
 end;
 texture.UnlockRect(0);
end;

procedure SetPixel(device: IDIRECT3DDEVICE8; x, y: DWORD;
 Red, Green, Blue, Alpha: byte);
var surface: IDIRECT3DSURFACE8;
begin
 if device = nil then exit;
 device.GetRenderTarget(surface);
 SetPixel(surface, x, y, Red, Green, Blue, Alpha);
 surface._Release;
end;

procedure GetPixel(texture: IDIRECT3DTEXTURE8; x, y: DWORD;
  var Red, Green, Blue, Alpha: byte);
var surf_desc: TD3DSURFACE_DESC;
    rect: TD3DLOCKED_RECT;
    t1, t2, tmp: word;
    pb: pbyte;
begin
 red := 0; green := 0; blue := 0;

 if (FAILED(texture.GetLevelDesc(0, surf_desc))) then exit;
 if (FAILED(texture.LockRect(0, rect, nil, MyLock or D3DLOCK_READONLY))) then exit;

 case surf_desc.Format of
 D3DFMT_X8R8G8B8,D3DFMT_A8R8G8B8:
 begin
  pb    := pbyte(longint(rect.pBits) + y*rect.pitch + x*4);
  blue  := pb^; inc(pb);
  green := pb^; inc(pb);
  red   := pb^; inc(pb);
  alpha := pb^;
 end;
 D3DFMT_R8G8B8:
 begin
  pb    := pbyte(longint(rect.pBits) + y*rect.pitch + x*3);
  blue  := pb^; inc(pb);
  green := pb^; inc(pb);
  red   := pb^;
 end;
 D3DFMT_A4R4G4B4:
 begin
  pb    := pbyte(longint(rect.pBits) + y*rect.pitch + x*2);
  t2    := pb^; inc(pb);
  t1    := pb^;
  tmp   := t1 shl 8 + t2;
  blue  := (tmp and $0F) shl 4;
  green := ((tmp shr 4)  and $0F) shl 4;
  red   := ((tmp shr 8)  and $0F) shl 4;
  alpha := ((tmp shr 12) and $0F) shl 4;
 end;
 D3DFMT_R5G6B5:
 begin
  pb    := pbyte(longint(rect.pBits) + y*rect.pitch + x*2);
  t2    := pb^; inc(pb);
  t1    := pb^;
  tmp   := t1 shl 8 + t2;
  blue  := (tmp and $1F) shl 3;
  green := ((tmp shr 5)  and $3F) shl 2;
  red   := ((tmp shr 11) and $1F) shl 3;
 end;
 end;
 texture.UnlockRect(0);
end;

procedure GetPixel(surface: IDIRECT3DSURFACE8; x, y: DWORD;
  var Red, Green, Blue, Alpha: byte);
var surf_desc: TD3DSURFACE_DESC;
    rect: TD3DLOCKED_RECT;
    t1, t2, tmp: word;
    pb: pbyte;
begin
 red := 0; green := 0; blue := 0;

 if (FAILED(surface.GetDesc(surf_desc))) then exit;
 if (FAILED(surface.LockRect(rect, nil, MyLock or D3DLOCK_READONLY))) then exit;

 case surf_desc.Format of
 D3DFMT_X8R8G8B8,D3DFMT_A8R8G8B8:
 begin
  pb    := pbyte(longint(rect.pBits) + y*rect.pitch + x*4);
  blue  := pb^; inc(pb);
  green := pb^; inc(pb);
  red   := pb^; inc(pb);
  alpha := pb^;
 end;
 D3DFMT_R8G8B8:
 begin
  pb    := pbyte(longint(rect.pBits) + y*rect.pitch + x*3);
  blue  := pb^; inc(pb);
  green := pb^; inc(pb);
  red   := pb^;
 end;
 D3DFMT_A4R4G4B4:
 begin
  pb    := pbyte(longint(rect.pBits) + y*rect.pitch + x*2);
  t2    := pb^; inc(pb);
  t1    := pb^;
  tmp   := t1 shl 8 + t2;
  blue  := (tmp and $0F) shl 4;
  green := ((tmp shr 4)  and $0F) shl 4;
  red   := ((tmp shr 8)  and $0F) shl 4;
  alpha := ((tmp shr 12) and $0F) shl 4;
 end;
 D3DFMT_R5G6B5:
 begin
  pb    := pbyte(longint(rect.pBits) + y*rect.pitch + x*2);
  t2    := pb^; inc(pb);
  t1    := pb^;
  tmp   := t1 shl 8 + t2;
  blue  := (tmp and $1F) shl 3;
  green := ((tmp shr 5)  and $3F) shl 2;
  red   := ((tmp shr 11) and $1F) shl 3;
 end;
 end;
 surface.UnlockRect;
end;

procedure GetPixel(Device: IDIRECT3DDEVICE8; x, y: DWORD;
  var Red, Green, Blue, Alpha: byte);
var surface: IDIRECT3DSURFACE8;
begin
 red := 0; green := 0; blue := 0;
 if device = nil then exit;
 device.GetRenderTarget(surface);
 GetPixel(surface, x, y, Red, Green, Blue, Alpha);
 surface._Release;
end;

procedure PF_AlphaChannelMixer(x, y: DWORD; var r, g, b, a:byte);
begin
 a := round(rr*r + rg*g + rb*b + ra*a);
end;

procedure ColorToAlpha(
 texture: IDIRECT3DTEXTURE8; // the texture to be modified
 RedToAlpha, GreenToAlpha,
 BlueToAlpha, AlphatoAlpha: single); // weightening factors (0..1)
{This function takes any IDIRECT3DTEXTURE8 (this could for example
also be an OmegaImageList1.ImageList.Items[0].Image) and re-calculates
its alpha component by weightening it with the current red, green,
blue and alpha channels. For a greyscale image, you would usually
set RedToAlpha to 1.0, the other values to 0.0. This gives a dynamic
alpha channel especially useful for light effects.}
begin
 rr := RedToAlpha;
 rg := GreenToAlpha;
 rb := BlueToAlpha;
 ra := AlphaToAlpha;
 ApplyPixelFunction(texture, PF_AlphaChannelMixer);
end;

procedure PF_Gray(x, y: DWORD; var r, g, b, a:byte);
var s:byte;
begin
 s:=(r + g + b) div 3;
 r := s;
 g := s;
 b := s;
end;

procedure Grayscale(texture: IDIRECT3DTEXTURE8);
// converts a texture to grayscale
begin
 ApplyPixelFunction(texture, PF_Gray);
end;

procedure PF_Transp(x, y: DWORD; var r, g, b, a: byte);
begin
 if (r = rri) and (g = rgi) and (b = rbi) then
  a := 0 else a := 255;
end;

procedure SetTransparentColor(texture: IDIRECT3DTEXTURE8; red, green, blue: byte);
begin
 rri := red;
 rgi := green;
 rbi := blue;
 ApplyPixelFunction(texture, PF_Transp);
end;

procedure ApplyPixelFunction(texture: IDIRECT3DTEXTURE8;
  pf: PixelFunction);
var surf_desc: TD3DSURFACE_DESC;
    rect: TD3DLOCKED_RECT;
    x, y, pd: DWORD;
    t1, t2, tmp: word;
    pb: pbyte;
    red, green, blue, alpha: byte;
begin
 if (FAILED(texture.GetLevelDesc(0, surf_desc))) then exit;
 if (FAILED(texture.LockRect(0, rect, nil, MyLock))) then exit;

 case surf_desc.Format of
 D3DFMT_X8R8G8B8,D3DFMT_A8R8G8B8:
 begin
  pd := (rect.Pitch div 4) - surf_desc.width;
  pb := pbyte(rect.pBits);
  for y := 0 to surf_desc.height - 1 do
  begin
   for x := 0 to surf_desc.width - 1 do
   begin
    blue  := pb^; inc(pb);
    green := pb^; inc(pb);
    red   := pb^; inc(pb);
    alpha := pb^;
    pf(x, y, red, green, blue, alpha);
    pb^   := alpha; dec(pb);
    pb^   := red;   dec(pb);
    pb^   := green; dec(pb);
    pb^   := blue;
    inc(pb, 4);
   end;
   inc(pb, pd);
  end;
 end;
 D3DFMT_R8G8B8:
 begin
  pd := (rect.Pitch div 3) - surf_desc.width;
  pb := pbyte(rect.pBits);
  for y := 0 to surf_desc.height - 1 do
  begin
   for x := 0 to surf_desc.width - 1 do
   begin
    blue  := pb^; inc(pb);
    green := pb^; inc(pb);
    red   := pb^;
    pf(x, y, red, green, blue, alpha);
    pb^ := red;   dec(pb);
    pb^ := green; dec(pb);
    pb^ := blue;
    inc(pb, 3);
   end;
   inc(pb, pd);
  end;
 end;
 D3DFMT_A4R4G4B4:
 begin
  pd := (rect.Pitch div 2) - surf_desc.width;
  pb := pbyte(rect.pBits);
  for y := 0 to surf_desc.height - 1 do
  begin
   for x := 0 to surf_desc.width - 1 do
   begin
    t2    := pb^; inc(pb);
    t1    := pb^;
    tmp   := t1 shl 8 + t2;
    blue  := tmp and $0F;
    green := (tmp shr 4)  and $0F;
    red   := (tmp shr 8)  and $0F;
    alpha := (tmp shr 12) and $0F;
    pf(x, y, red, green, blue, alpha);
    tmp := alpha shl 12 + red shl 8 + green shl 4 + blue;
    pb^ := hi(tmp); dec(pb);
    pb^ := lo(tmp);
    inc(pb, 2);
   end;
   inc(pb, pd);
  end;
 end;
 D3DFMT_R5G6B5:
 begin
  pd := (rect.Pitch div 2) - surf_desc.width;
  pb := pbyte(rect.pBits);
  for y := 0 to surf_desc.height - 1 do
  begin
   for x := 0 to surf_desc.width - 1 do
   begin
    t2    := pb^; inc(pb);
    t1    := pb^;
    tmp   := t1 shl 8 + t2;
    blue  := tmp and $1F;
    green := (tmp shr 5)  and $3F;
    red   := (tmp shr 11) and $1F;
    tmp := red shl 11 + green shl 5 + blue;
    pf(x, y, red, green, blue, alpha);
    pb^ := hi(tmp); dec(pb);
    pb^ := lo(tmp);
    inc(pb, 2);
   end;
   inc(pb, pd);
  end;
 end;
 end;
 texture.UnlockRect(0);
end;

procedure ApplyPixelFunction(texture: IDIRECT3DTEXTURE8;
 rec: trect; pf:PixelFunction);
var surf_desc: TD3DSURFACE_DESC;
    rect: TD3DLOCKED_RECT;
    x, y, pd: DWORD;
    t1, t2, tmp: word;
    pb: pbyte;
    red, green, blue, alpha: byte;
begin
 if (FAILED(texture.GetLevelDesc(0, surf_desc))) then exit;
 if (FAILED(texture.LockRect(0, rect, nil, MyLock))) then exit;

 case surf_desc.Format of
 D3DFMT_X8R8G8B8,D3DFMT_A8R8G8B8:
 begin
  pd := (rect.Pitch div 4) - surf_desc.width +
   4 * (surf_desc.width - rec.right - 1);
  pb := pbyte(longint(rect.pBits) + rec.top * rect.pitch);
  for y := rec.top to rec.bottom do
  begin
   inc(pb, 4 * rec.left);
   for x := rec.left to rec.right do
   begin
    blue  := pb^; inc(pb);
    green := pb^; inc(pb);
    red   := pb^; inc(pb);
    alpha := pb^;
    pf(x, y, red, green, blue, alpha);
    pb^   := alpha; dec(pb);
    pb^   := red;   dec(pb);
    pb^   := green; dec(pb);
    pb^   := blue;
    inc(pb, 4);
   end;
   inc(pb, pd);
  end;
 end;
 D3DFMT_R8G8B8:
 begin
  pd := (rect.Pitch div 3) - surf_desc.width +
   3 * (surf_desc.width - rec.right - 1);
  pb := pbyte(longint(rect.pBits) + rec.top * rect.pitch);
  for y := rec.top to rec.bottom do
  begin
   inc(pb, 3 * rec.left);
   for x := rec.left to rec.right do
   begin
    blue  := pb^; inc(pb);
    green := pb^; inc(pb);
    red   := pb^;
    pf(x, y, red, green, blue, alpha);
    pb^ := red;   dec(pb);
    pb^ := green; dec(pb);
    pb^ := blue;
    inc(pb, 3);
   end;
   inc(pb, pd);
  end;
 end;
 D3DFMT_A4R4G4B4:
 begin
  pd := (rect.Pitch div 2) - surf_desc.width +
   2 * (surf_desc.width - rec.right - 1);
  pb := pbyte(longint(rect.pBits) + rec.top * rect.pitch);
  for y := rec.top to rec.bottom do
  begin
   inc(pb, 2 * rec.left);
   for x := rec.left to rec.right do
   begin
    t2    := pb^; inc(pb);
    t1    := pb^;
    tmp   := t1 shl 8 + t2;
    blue  := tmp and $0F;
    green := (tmp shr 4)  and $0F;
    red   := (tmp shr 8)  and $0F;
    alpha := (tmp shr 12) and $0F;
    pf(x, y, red, green, blue, alpha);
    tmp := alpha shl 12 + red shl 8 + green shl 4 + blue;
    pb^ := hi(tmp); dec(pb);
    pb^ := lo(tmp);
    inc(pb, 2);
   end;
   inc(pb, pd);
  end;
 end;
 D3DFMT_R5G6B5:
 begin
  pd := (rect.Pitch div 2) - surf_desc.width +
   2 * (surf_desc.width - rec.right - 1);
  pb := pbyte(longint(rect.pBits) + rec.top * rect.pitch);
  for y := rec.top to rec.bottom do
  begin
   inc(pb, 2 * rec.left);
   for x := rec.left to rec.right do
   begin
    t2    := pb^; inc(pb);
    t1    := pb^;
    tmp   := t1 shl 8 + t2;
    blue  := tmp and $1F;
    green := (tmp shr 5)  and $3F;
    red   := (tmp shr 11) and $1F;
    tmp := red shl 11 + green shl 5 + blue;
    pf(x, y, red, green, blue, alpha);
    pb^ := hi(tmp); dec(pb);
    pb^ := lo(tmp);
    inc(pb, 2);
   end;
   inc(pb, pd);
  end;
 end;
 end;
 texture.UnlockRect(0);
end;

function ChangeDesktopResolution(width, height: word): boolean;
var lpDevMode: TDeviceMode;
begin
 Result := EnumDisplaySettings(nil, 0, lpDevMode);
 if Result then
 begin
  lpDevMode.dmFields := DM_PELSWIDTH Or DM_PELSHEIGHT;
  lpDevMode.dmPelsWidth := width;
  lpDevMode.dmPelsHeight := height;
  Result := ChangeDisplaySettings(lpDevMode, CDS_UPDATEREGISTRY) =
   DISP_CHANGE_SUCCESSFUL;
 end;
end;

procedure PF_TranspEx(x, y: DWORD; var r, g, b, a: byte);
begin
 if (r = rri) and (g = rgi) and (b = rbi) then a := rai;
end;

procedure SetTransparency(texture: IDIRECT3DTEXTURE8; red, green, blue, alpha: byte);
begin
 rri := red;
 rgi := green;
 rbi := blue;
 rai := alpha;
 ApplyPixelFunction(texture, PF_TranspEx);
end;

function GetDelphiVersion:integer;
begin
 result:=1;
{$IFDEF VER90} result:=2; {$ENDIF}
{$IFDEF VER100} result:=3; {$ENDIF}
{$IFDEF VER120} result:=4; {$ENDIF}
{$IFDEF VER130} result:=5; {$ENDIF}
{$IFDEF VER140} result:=6; {$ENDIF}
{$IFDEF VER150} result:=7; {$ENDIF}
{$IFDEF VER160} result:=8; {$ENDIF}
end;

function GetDXVersion:string;
begin
 Result := '0.0';
 with TRegistry.Create do
 try
  RootKey := HKEY_LOCAL_MACHINE;
  if OpenKey('\SOFTWARE\Microsoft\DirectX', False) then
   result := copy(Readstring('Version'),4,4);
 finally
   Free;
 end;
end;

procedure SetPixel(os: TOmegaScreen; x, y: DWORD; Red, Green,
 Blue, Alpha: byte);
begin
 os.DrawRectangle(x, y, x, y, false, 1, red, green, blue);
end;

const afac = pi/180;

procedure Modify(var x: single; var y:single;
 CenterX, CenterY, Angle, ScaleX, ScaleY,
 OfsX, OfsY: single; c: TOmegaColor);
var a: integer;
    q, s1, s2, s3, s4: single;
begin
 q  := Angle * afac;
 s1 := ScaleX * cos(q);
 s2 := ScaleX * sin(q);
 s3 := - ScaleY * sin(q);
 s4 := ScaleY * cos(q);
 q := s1 * x + s3 * y + CenterX - CenterX * s1 - CenterY * s3;
 y := OfsY + s2 * x + s4 * y + CenterY - CenterX * s2 - CenterY * s4;
 x := OfsX + q;
end;

procedure Modify(var p: TPoint;
 CenterX, CenterY, Angle, ScaleX, ScaleY,
 OfsX, OfsY: single; c: TOmegaColor);
var x, y: single;
begin
 x := p.x;
 y := p.y;
 Modify(x, y, CenterX, CenterY, Angle, ScaleX, ScaleY,
  OfsX, OfsY, c);
 p.x := round(x);
 p.y := round(y);
end;

end.

