{
//****************************************************************
// OmegaVideo unit
// author : Tobias Fleischer
// contact: tobybear@web.de
// comments are welcome!
//****************************************************************

version history:
08/98/04: added transparent and transparent color property 
03/17/04: first release

"todo":
- add proper documentation
}
unit OmegaVideo;
interface
uses Windows, OmegaScreen, Classes,
 DirectDraw, DirectShow, 
 Graphics, ComObj, OmegaRenderer, {$I UseD3D8.inc};

type
  TOmegaVideo = class(TComponent)
  private
    FRenderer: TOmegaRenderer;
    SurfaceDC: HDC;
    tu, tv: single;
    FBitmap : TBitmap;
    Stream: TMemoryStream;
    DDSurface: IDirectDrawSurface;
    FDDraw: IDirectDraw;
    FTexture: IDirect3DTexture8;
    FBlendMode: Integer;
    FLooped: boolean;
    FBlue: Integer;
    FRed: Integer;
    FGreen: Integer;
    FAlpha: Integer;
    FScaleY: Single;
    FScaleX: Single;
    FRasterOP: integer;
    FFilename: WideString;
    FCol1, FCol2, FCol3, FCol4: TOmegaColor;
    FPlayAudio: boolean;
    FPlaying: boolean;
    FLoaded: boolean;
    FCenterY: Single;
    FCenterX: Single;
    FRotation: Single;
    FX1, FY1, FX2, FY2, FX3, FY3, FX4, FY4: Single;
    FMirrorV: boolean;
    FMirrorH: boolean;
    FTransparentColor: TColor;
    FTransparent: Boolean;
    procedure SetOmegaScreen(const Value: TOmegaScreen);
    procedure SetAlpha(const Value: Integer);
    procedure SetBlue(const Value: Integer);
    procedure SetGreen(const Value: Integer);
    procedure SetRed(const Value: Integer);
    procedure SetX(const Value: Single);
    procedure SetY(const Value: Single);
    procedure SetHeight(const Value: integer);
    procedure SetWidth(const Value: integer);
    function GetHeight: integer;
    function GetWidth: integer;
  protected
    FOmegaScreen: TOmegaScreen;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    VideoWidth, VideoHeight: integer;
    Sample: IDirectDrawStreamSample;
    MMStream: IMultiMediaStream;
    property MirrorHorizontally: boolean read FMirrorH write FMirrorH;
    property MirrorVertically: boolean read FMirrorV write FMirrorV;
    property X1: Single read FX1 write FX1;
    property Y1: Single read FY1 write FY1;
    property X2: Single read FX2 write FX2;
    property Y2: Single read FY2 write FY2;
    property X3: Single read FX3 write FX3;
    property Y3: Single read FY3 write FY3;
    property X4: Single read FX4 write FX4;
    property Y4: Single read FY4 write FY4;
    property Playing: boolean read FPlaying;
    property BlendMode: integer read FBlendMode write FBlendMode;
    property RasterOp: integer read FRasterOP write FRasterOp default SrcCopy;
    property Texture: IDirect3DTexture8 read FTexture write FTexture;
    property ColorTL: TOmegaColor read FCol1 write FCol1;
    property ColorTR: TOmegaColor read FCol2 write FCol2;
    property ColorBR: TOmegaColor read FCol3 write FCol3;
    property ColorBL: TOmegaColor read FCol4 write FCol4;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Draw;
    procedure AdvanceVideo;
    procedure LoadVideo(fn:WideString);overload;
    procedure LoadVideo;overload;
    procedure Stop;
    procedure Run;
    procedure Init;
    procedure UnInit;
  published
    property TransparentColor: TColor read FTransparentColor write FTransparentColor;
    property Transparent: Boolean read FTransparent write FTransparent;
    property Filename: WideString read FFilename write FFilename;
    property ScaleX: Single read FScaleX write FScaleX;
    property ScaleY: Single read FScaleY write FScaleY;
    property CenterX: Single read FCenterX write FCenterX;
    property CenterY: Single read FCenterY write FCenterY;
    property Width: integer read GetWidth write SetWidth;
    property Height: integer read GetHeight write SetHeight;
    property X: Single read FX1 write SetX;
    property Y: Single read FY1 write SetY;
    property Rotation: Single read FRotation write FRotation;
    property Red: Integer read FRed write SetRed;
    property Green: Integer read FGreen write SetGreen;
    property Blue: Integer read FBlue write SetBlue;
    property Alpha: Integer read FAlpha write SetAlpha;
    property OmegaScreen: TOmegaScreen read FOmegaScreen write SetOmegaScreen;
    property Looped: boolean read FLooped write FLooped;
    property PlayAudio: boolean read FPlayAudio write FPlayAudio;
  end;

implementation
uses D3DX8;

procedure TOmegaVideo.SetOmegaScreen(const Value: TOmegaScreen);
begin
 FOmegaScreen := Value;
end;

constructor TOmegaVideo.Create(AOwner: TComponent);
begin
 FRenderer := TOmegaRenderer.Create;
 inherited;
 FTransparentColor := clBlack;
 FTransparent := false;
 DirectDrawCreate(nil, FDDraw, nil);
 FRasterOp := SRCCOPY;
 FLoaded := false;
 FBitmap := TBitmap.Create;
 Stream := TMemoryStream.Create;
 FScaleX := 1;
 FScaleY := 1;
 FRed := 255;
 FGreen := 255;
 FBlue := 255;
 FAlpha := 255;
 tu := 1;
 tv := 1;
 FTexture := nil;
end;

destructor TOmegaVideo.Destroy;
begin
 if FRenderer <> nil then FRenderer.free;
 inherited;
 FDDraw := nil;
 Stream.Free;
 FBitmap.Free;
 if FTexture <> nil then FTexture := nil;
end;

function n_larger(number:uint):uint;
var numb:uint;
begin
 numb:=1;
 while (numb<number) do numb:=numb shl 1;
 result:=numb;
end;

procedure TOmegaVideo.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
 inherited;
 case Operation of
  opInsert: if (AComponent is TOmegaScreen)and(not Assigned(OmegaScreen)) then
   SetOmegaScreen(TOmegaScreen(AComponent));
  opRemove: if (AComponent = OmegaScreen) then OmegaScreen:= nil;
 end;
end;

procedure TOmegaVideo.AdvanceVideo;
var h: HRESULT;
    c: longint;
begin
 if not FLoaded then exit;
 h := Sample.Update(SSUPDATE_ASYNC,0,nil,0);
// if h=MS_S_PENDING then Sample.CompletionStatus(0,0) else
 if h=MS_S_ENDOFSTREAM then
  begin
   if FLooped then MMStream.Seek(0)
   else Stop;
  end;
 DDSurface.GetDC(SurfaceDC);
 try
  Stream.Seek(0,0);
  BitBlt(FBitmap.Canvas.Handle, 0, 0,
   VideoWidth, VideoHeight, SurfaceDC, 0, 0, FRasterOp);
  FBitmap.SaveToStream(Stream);
  FTexture := nil;
  if FTransparent then
   c := $FF000000 or TColor(OmegaColor(FTransparentColor, 0))
  else
   c := 0;
  if failed(D3DXCreateTextureFromFileInMemoryEx(
    FOmegaScreen.Device, Stream.Memory^, Stream.Size,
    0, 0, 0, 0, D3DFMT_UNKNOWN, D3DPOOL_MANAGED,
    D3DX_FILTER_NONE, D3DX_FILTER_NONE,
    c, nil, nil, FTexture)) then
   begin
    MessageBox(0, pChar('Error creating video texture'), 'Error', MB_ICONERROR);
    Stop;
   end;
 finally
  DDSurface.ReleaseDC(SurfaceDC);
 end;
end;

procedure TOmegaVideo.SetAlpha(const Value: Integer);
begin
  FAlpha := Value;
  FCol1 := OmegaColor(FRed, FGreen, FBlue, FAlpha);
  FCol2 := FCol1; FCol3 := FCol1; FCol4 := FCol1;
end;

procedure TOmegaVideo.SetBlue(const Value: Integer);
begin
  FBlue := Value;
  FCol1 := OmegaColor(FRed, FGreen, FBlue, FAlpha);
  FCol2 := FCol1; FCol3 := FCol1; FCol4 := FCol1;
end;

procedure TOmegaVideo.SetGreen(const Value: Integer);
begin
  FGreen := Value;
  FCol1 := OmegaColor(FRed, FGreen, FBlue, FAlpha);
  FCol2 := FCol1; FCol3 := FCol1; FCol4 := FCol1;
end;

procedure TOmegaVideo.SetRed(const Value: Integer);
begin
  FRed := Value;
  FCol1 := OmegaColor(FRed, FGreen, FBlue, FAlpha);
  FCol2 := FCol1; FCol3 := FCol1; FCol4 := FCol1;
end;

procedure TOmegaVideo.LoadVideo(fn: WideString);
var ds: TDDSurfaceDesc;
    z: cardinal;
    p: IDirectDrawPalette;
    AMStream: IAMMultiMediaStream;
    PrimaryVidStream: IMediaStream;
    DDStream: IDirectDrawMediaStream;
    DesiredSurface: TDDSurfaceDesc;
    tg:TGUID;
begin
 FDDraw.SetCooperativeLevel(GetDesktopWindow(), DDSCL_NORMAL);
 AMStream := IAMMultiMediaStream(CreateComObject(CLSID_AMMultiMediaStream));
 AMStream.Initialize(STREAMTYPE_READ, 0, nil);
 tg:=MSPID_PrimaryVideo;
 AMStream.AddMediaStream(FDDraw, @tg, 0, IMediaStream(nil^));
//t AMStream.AddMediaStream(FDDraw, tg, 0, IMediaStream(nil^));
 if FPlayAudio then
 begin
  try
   tg:=MSPID_PrimaryAudio;
   AMStream.AddMediaStream(nil, @tg, AMMSF_ADDDEFAULTRENDERER,IMediaStream(nil^));
//t   AMStream.AddMediaStream(nil, tg, AMMSF_ADDDEFAULTRENDERER,IMediaStream(nil^));
  except
  end;
 end;
 AMStream.OpenFile(PWideChar(fn),0);
 MMStream := AMStream as IMultiMediaStream;
 MMStream.GetMediaStream(MSPID_PrimaryVideo, PrimaryVidStream);
 DDStream := PrimaryVidStream as IDirectDrawMediaStream;
 DDStream.GetFormat(ds,p,ds,z);
 VideoWidth := ds.dwWidth;
 VideoHeight := ds.dwHeight;
 ZeroMemory(@DesiredSurface, SizeOf(DesiredSurface));
 DesiredSurface.dwSize := Sizeof(DesiredSurface);
 DDStream.GetFormat(TDDSurfaceDesc(nil^),
  IDirectDrawPalette(nil^), DesiredSurface, DWord(nil^));
 DesiredSurface.ddsCaps.dwCaps := DesiredSurface.ddsCaps.dwCaps;
 DesiredSurface.dwFlags := DesiredSurface.dwFlags or DDSD_CAPS
  or DDSD_PIXELFORMAT;
 FDDraw.CreateSurface(DesiredSurface, DDSurface, nil);
 DDStream.CreateSample(DDSurface, TRect(nil^), 0, Sample);
 FBitmap.Width := VideoWidth;
 FBitmap.Height := VideoHeight;
 tu := VideoWidth/n_larger(VideoWidth);
 tv := VideoHeight/n_larger(VideoHeight);
 FLoaded:= true;
end;

procedure TOmegaVideo.LoadVideo;
begin
 LoadVideo(filename);
end;

procedure TOmegaVideo.Run;
begin
 if not FLoaded then exit;
 MMStream.SetState(STREAMSTATE_RUN);
 FPlaying := true;
end;

procedure TOmegaVideo.Stop;
begin
 if not FLoaded then exit;
 MMStream.SetState(STREAMSTATE_STOP);
 FPlaying := false;
end;

procedure TOmegaVideo.Draw;
begin
 if not FLoaded then exit;
 FOmegaScreen.Device.SetTexture(0, FTexture);
 if FBlendMode <> bmIgnore then FOmegaScreen.SetBlendMode(FBlendMode);
 FRenderer.MirrorHorizontally := FMirrorH;
 FRenderer.MirrorVertically := FMirrorV;
 FRenderer.SetPositions(FX1, FY1, FX2, FY2,
  FX3, FY3, FX4, FY4);
 FRenderer.SetColors(FCol1, FCol2, FCol3, FCol4);
 FRenderer.SetTexturePositions(0, 0, tu, 0, tu, tv, 0, tv);
 FRenderer.RotateAndScale(FX1 + FCenterX, FY1 + FCenterY,
  FRotation, FScaleX, FScaleY);
 FRenderer.Draw;
end;

procedure TOmegaVideo.Init;
begin
 if FOmegaScreen = nil then exit;
 if FRenderer = nil then exit;
 FRenderer.OmegaScreen := FOmegaScreen;
 FRenderer.Init;
end;

procedure TOmegaVideo.UnInit;
begin
 if FRenderer <> nil then FRenderer.UnInit;
end;

procedure TOmegaVideo.SetX(const Value: Single);
begin
 FX1 := Value;
 FX2 := Value + Width;
 FX3 := FX2;
 FX4 := FX1;
end;

procedure TOmegaVideo.SetY(const Value: Single);
begin
 FY1 := Value;
 FY2 := FY1;
 FY3 := Value + Height;
 FY4 := FY3;
end;

procedure TOmegaVideo.SetHeight(const Value: integer);
begin
 FY3 := FY1 + Value;
 FY4 := FY3;
end;

procedure TOmegaVideo.SetWidth(const Value: integer);
begin
 FX2 := FX1 + Value;
 FX3 := FX2;
end;

function TOmegaVideo.GetHeight: integer;
begin
 result := round(Y3 - Y2);
end;

function TOmegaVideo.GetWidth: integer;
begin
 result := round(X2 - X1);
end;

end.

